* ======================================================================== *
* === 		An Equilibrium Model of the Afrcan HIV/AIDS Epidemic 	   === *
* === Jeremy Greenwood, Philipp Kircher, Cezar Santos, Michele Tertilt === * 
* === 						MALAWI (DHS 2004) 				  	  	   === *
* ======================================================================== *


clear
capture log close
set more off
pause on
set linesize 80
*macro drop _all
set memory 500000 


*** Definition of paths ***

*** ADJUST GLOBALS TO YOUR PATHS!

global dofile "C:\Users\YOU\moments1996_2004\dofiles"
global path "C:\Users\YOU\moments1996_2004"
global year "$path\rawdata"
global files "$path\workdata"

global outdat= "..\outdat\"

global gender "male female both"
log using "$files\Malawi_HIV_2004_logfile.log",replace


* =================================================================================================== *
* ======================================= DATA MATCHING ============================================= *
* =================================================================================================== *



* == Individual Questionnaire (IR and MR) == *

*** WOMEN'S QUESTIONNAIRE ***
use "$year\MWIR4EFL.dta", clear 
keep v001 v002 v003 v005 v012 v013 v021 v022 v024 v025 v023 v106 v107 v190  v5* v7* shiv03
gen id=(v001*100+v002)*100+v003
gen id_hh=(v001*100+v002)*100
gen gender=1
rename v005 sample_weight
rename v012 age
rename v013 age_group
rename shiv03 hiv_test
rename v024 region
rename v025 urban
replace urban=0 if urban==2
rename v106 educ
rename v107 educ_year
rename v190 wealth

cd "$path\workdata" 
save female_indiv_age_2004,replace

*** MEN'S QUESTIONNAIRE ***
use "$year\MWMR4EFL.dta", clear
keep mv001 mv002 mv003 mv005 mv012 mv013 mv021 mv022 mv023 mv024 mv025 mv106 mv107 mv190  mv5* mv7* mhiv03
gen id=(mv001*100+mv002)*100+mv003
gen id_hh=(mv001*100+mv002)*100
gen gender=0
rename mv005 sample_weight
rename mv012 age
rename mv013 age_group
rename mhiv03 hiv_test
rename mv024 region
rename mv025 urban
replace urban=0 if urban==2
rename mv106 educ
rename mv107 educ_year
rename mv190 wealth

cd "$path\workdata" 
save male_indiv_age_2004,replace

* change variables in order to just append (not merge) the men's file to the women's file
renpfix mv v
save male_indiv_age_2004_append,replace

*** HOUSEHOLDS'S QUESTIONNAIRE ***
use "$year\MWHR4EFL.dta", clear
gen id_hh=(hv001*100+hv002)*100
keep id_hh hv005
cd "$files" 
save household_sampling_weights_2004,replace

* == Append both files and merge with HIV tests == *


use "$path\workdata\female_indiv_age_2004.dta", clear 
append using "$path\workdata\male_indiv_age_2004_append.dta"
/*
merge 1:1 id using "$files\hiv_2004.dta"
drop if gender==.
drop _merge
merge m:1 id_hh using "$files\household_sampling_weights_2004.dta"
cd "$files"*/
save all_indiv_age_2004, replace

* ============================================================================================================== *
* =========================================== DIFFERENT MOMENTS ================================================ *
* ============================================================================================================== *
use "$path\workdata\all_indiv_age_2004", clear


* ================================================ *
* == MOMENT 1: Fraction of HIV-infected by age  == *
* ================================================ *
* Weighted (sample weight)

* all people
/*sum hiv03 [aweight=hiv05] if gender==0 &hiv03!=.
sum hiv03 [aweight=hiv05] if gender==1 &hiv03!=.
sum hiv03 [aweight=hiv05] if hiv03!=.


gen age_2=age^2
gen age_3=age^3

levelsof age, local (levels)
foreach l of local levels{
sum hiv03 [aweight=sample_weight] if age==`l' &hiv03!=.
}
preserve
collapse (mean) gender hiv_rate=hiv03 age_* (sd) hiv_rate_sd=hiv03 (count) observations=hiv03 [aweight=sample_weight] if hiv03!=., by (age)
drop age_group
reg hiv_rate age*
predict hiv_rate_3p
drop age_*
save "$files\1_both_HIVrate_weighted_2004.dta",replace
restore
 
* women
levelsof age, local (levels)
foreach l of local levels{
sum hiv03 [aweight=sample_weight] if age==`l' & hiv03!=. &gender==1
} 
preserve
collapse (mean) gender hiv_rate=hiv03 age_* (sd) hiv_rate_sd=hiv03 (count) observations=hiv03 [aweight=sample_weight] if gender==1 &hiv03!=., by (age)
drop age_group
reg hiv_rate age*
predict hiv_rate_3p
drop age_*
save "$files\1_female_HIVrate_weighted_2004.dta",replace
restore

* men
levelsof age, local (levels)
foreach l of local levels{
sum hiv03 [aweight=sample_weight] if age==`l' &hiv03!=.&gender==0
} 
preserve
collapse (mean) gender hiv_rate=hiv03 age_* (sd) hiv_rate_sd=hiv03 (count) observations=hiv03 [aweight=sample_weight] if gender==0 &hiv03!=., by (age)
drop age_group
reg hiv_rate age*
predict hiv_rate_3p
drop age_*
save "$files\1_male_HIVrate_weighted_2004.dta",replace
restore*/

* ================================================= *
* == MOMENT 2: Casual sex (cond. on having sex)  == *
* ================================================= *
* Conditional on having sex last period, prob that the partner was short-term
* Weighted (sample weight)

* LAST QUARTER
* sex within last quarter: dropping inconsistent answer according to the imputed time in months (v529) from raw time (v527)
* for men: also including the answers to question "paid for sex"
gen sex_time=v527 
gen sex_quarter_imp=1 if sex_time<=304
replace sex_quarter_imp=0 if sex_time!=999 & sex_time>304 & sex_time!=.
replace sex_quarter_imp=1 if v792<=304 & gender==0

* sex within last quarter: using raw time (v527) => there might be some inconsistencies (check v529 for women)!!!!!!!!!!!!!!
* for men: also including the answers to question "paid for sex"
gen sex_quarter=1 if v527<=304
replace sex_quarter=0 if v527!=999 & v527!=.& v527>304
replace sex_quarter=1 if v792<=304 & gender==0

* sex within last quarter in short-term market: boyfriend/fiancé is coded as "short-term" 
* for men: on top the answers to question "paid for sex" (v792), long-term partner can be replaced by short-term partner if the men paid for sex in the last quarter
gen partner_quarter_short=0 if v767a==1
replace partner_quarter_short=1 if (v767a>1 &v767a<99)|(v792<=304 & gender==0)

* all people
levelsof age, local (levels)
foreach l of local levels{
sum partner_quarter_short [aweight=sample_weight] if age==`l' &sex_quarter==1
} 
preserve
collapse (mean) gender_quarter=gender partner_quarter_short=partner_quarter_short (sd) partner_quarter_short_sd=partner_quarter_short (count) observations_quarter=partner_quarter_short [aweight=sample_weight] if sex_quarter==1, by (age)
save "$files\2_both_shortTerm1_quarter_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_quarter_imp=gender partner_quarter_short_imp=partner_quarter_short (sd) partner_quarter_short_imp_sd=partner_quarter_short (count) observations_quarter_imp=partner_quarter_short [aweight=sample_weight] if sex_quarter_imp==1, by (age)
save "$files\2_both_shortTerm2_quarter_weighted_2004.dta",replace
restore

* women
levelsof age, local (levels)
foreach l of local levels{
sum partner_quarter_short [aweight=sample_weight] if age==`l' &sex_quarter==1 &gender==1
} 
preserve
collapse (mean) gender_quarter=gender partner_quarter_short=partner_quarter_short (sd) partner_quarter_short_sd=partner_quarter_short (count) observations_quarter=partner_quarter_short [aweight=sample_weight] if sex_quarter==1 &gender==1, by (age)
save "$files\2_female_shortTerm1_quarter_weighted_2004.dta",replace
restore
preserve
collapse (mean)gender_quarter_imp=gender partner_quarter_short_imp=partner_quarter_short (sd) partner_quarter_short_imp_sd=partner_quarter_short (count) observations_quarter_imp=partner_quarter_short [aweight=sample_weight] if sex_quarter_imp==1 &gender==1, by (age)
save "$files\2_female_shortTerm2_quarter_weighted_2004.dta",replace
restore

* men
levelsof age, local (levels)
foreach l of local levels{
sum partner_quarter_short [aweight=sample_weight] if age==`l' &sex_quarter==1 &gender==0
} 
preserve
collapse (mean) gender_quarter=gender partner_quarter_short=partner_quarter_short (sd) partner_quarter_short_sd=partner_quarter_short (count) observations_quarter=partner_quarter_short [aweight=sample_weight] if sex_quarter==1 &gender==0, by (age)
save "$files\2_male_shortTerm1_quarter_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_quarter_imp=gender partner_quarter_short_imp=partner_quarter_short (sd) partner_quarter_short_imp_sd=partner_quarter_short (count) observations_quarter_imp=partner_quarter_short [aweight=sample_weight] if sex_quarter_imp==1 &gender==0, by (age)
save "$files\2_male_shortTerm2_quarter_weighted_2004.dta",replace
restore

* LAST YEAR
* sex within last year: dropping inconsistent answer according to the imputed time in months (v529) from raw time (v527)
* this also includes information on sex with 2 other men/women
* for men: also including the answers to question "paid for sex"
gen sex_year_imp=1 if sex_time<=401 
replace sex_year_imp=0 if sex_time!=999 & sex_time>401 & sex_time!=. 
replace sex_year_imp=0 if v536==0
replace sex_year_imp=1 if v761b!=. | v761c!=.
replace sex_year_imp=1 if v792<=401 & gender==0

* sex within last year: using raw time (v527) => there might be some inconsistencies (check v529 for women)!!!!!!!!!!!!!!
gen sex_year=1 if v527<=401
replace sex_year=0 if v527!=999 &v527!=.& v527>401
replace sex_year=0 if v536==0
*(there shouldn't be any changes: sex with 2 other men/women)
replace sex_year=1 if v761b!=. | v761c!=.
replace sex_year=1 if v792<=401 & gender==0

* sex within last year in short-term market: boyfriend/fiancé is coded as "short-term" 
* furthermore if there is one sexual intercourse with a short-term partner the variable is coded as short-term market
gen partner_year_short=0 if v767a==1| v767b==1|v767c==1 
replace partner_year_short=1 if (v767a>1 &v767a<99)|(v767b>1 &v767b<99)|(v767c>1 &v767c<99)|(v792<=401 & gender==0)

* all people
levelsof age, local (levels)
foreach l of local levels{
sum partner_year_short [aweight=sample_weight] if age==`l' &sex_year==1
} 
preserve
collapse (mean) gender_year=gender partner_year_short=partner_year_short (sd) partner_year_short_sd=partner_year_short (count) observations_year=partner_year_short [aweight=sample_weight] if sex_year==1, by (age)
save "$files\2_both_shortTerm1_year_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_year_imp=gender partner_year_short_imp=partner_year_short (sd) partner_year_short_imp_sd=partner_year_short (count) observations_year_imp=partner_year_short [aweight=sample_weight] if sex_year_imp==1, by (age)
save "$files\2_both_shortTerm2_year_weighted_2004.dta",replace
restore

* women
levelsof age, local (levels)
foreach l of local levels{
sum partner_year_short [aweight=sample_weight] if age==`l' &sex_year==1 &gender==1
} 
preserve
collapse (mean) gender_year=gender partner_year_short=partner_year_short (sd) partner_year_short_sd=partner_year_short (count) observations_year=partner_year_short [aweight=sample_weight] if sex_year==1 &gender==1, by (age)
save "$files\2_female_shortTerm1_year_weighted_2004.dta",replace
restore
preserve
collapse (mean)gender_year_imp=gender partner_year_short_imp=partner_year_short (sd) partner_year_short_imp_sd=partner_year_short (count) observations_year_imp=partner_year_short [aweight=sample_weight] if sex_year_imp==1 &gender==1, by (age)
save "$files\2_female_shortTerm2_year_weighted_2004.dta",replace
restore

* men
levelsof age, local (levels)
foreach l of local levels{
sum partner_year_short [aweight=sample_weight] if age==`l' &sex_year==1 &gender==0
} 
preserve
collapse (mean) gender_year=gender partner_year_short=partner_year_short (sd) partner_year_short_sd=partner_year_short (count) observations_year=partner_year_short [aweight=sample_weight] if sex_year==1 &gender==0, by (age)
save "$files\2_male_shortTerm1_year_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_year_imp=gender partner_year_short_imp=partner_year_short (sd) partner_year_short_imp_sd=partner_year_short (count) observations_year_imp=partner_year_short [aweight=sample_weight] if sex_year_imp==1 &gender==0, by (age)
save "$files\2_male_shortTerm2_year_weighted_2004.dta",replace
restore

* ALTERNATIVE (YEAR)
gen sex_short_moment2=0 if v766a==0
replace sex_short_moment2=1 if v766a!=0&v766a!=99&v766a!=.

* all people
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_moment2 [aweight=sample_weight] if age==`l' &v766b!=0 &v766b!=99
} 
preserve
collapse (mean) gender sex_short=sex_short_moment2 (sd) sex_short_sd=sex_short_moment2 (count) observations=sex_short_moment2 [aweight=sample_weight] if v766b!=0 &v766b!=99, by (age)
save "$files\2_both_shortTerm_alternative_weighted_2004.dta",replace
restore

* women
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_moment2 [aweight=sample_weight] if age==`l' &v766b!=0 &v766b!=99 &gender==1
} 
preserve
collapse (mean) gender sex_short=sex_short_moment2 (sd) sex_short_sd=sex_short_moment2 (count) observations=sex_short_moment2 [aweight=sample_weight] if v766b!=0 &v766b!=99 & gender==1, by (age)
save "$files\2_female_shortTerm_alternative_weighted_2004.dta",replace
restore

* men
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_moment2 [aweight=sample_weight] if age==`l' &v766b!=0 &v766b!=99 &gender==0
} 
preserve
collapse (mean) gender sex_short=sex_short_moment2 (sd) sex_short_sd=sex_short_moment2 (count) observations=sex_short_moment2 [aweight=sample_weight] if v766b!=0 &v766b!=99 & gender==0, by (age)
save "$files\2_male_shortTerm_alternative_weighted_2004.dta",replace
restore 

* ================================================= *
* == MOMENT 3: Casual sex and use of condom      == *
* ================================================= *
* Conditional on having sex in the short-term market, what fraction used a condom
* Weighted (sample weight)

* LAST QUARTER
* short-term market includes here also boyfriend and fiance

* conditional on short-term sex within last quarter: dropping inconsistent answer according to the imputed time in months (v529) from raw time (v527)
* this also includes information on sex with 2 other men/women 
* for men: also including the answers to question "paid for sex"
gen sex_short_quarter_condom_imp=1 if sex_quarter_imp==1 &partner_quarter_short==1 & (v761==1|(v792<=304 & gender==0 &v761d==1))
replace sex_short_quarter_condom_imp=0 if sex_quarter_imp==1 &partner_quarter_short==1 & (v761==0|(v792<=304 & gender==0 &v761d==0))

* conditional on short-term  sex within last quarter: using raw time (v527) => there might some inconsistencies (check v529 for women)!!!!!!!!!!!!!!
* for men: also including the answers to question "paid for sex"
gen sex_short_quarter_condom=1 if sex_quarter==1 &partner_quarter_short==1 & (v761==1|(v792<=304 & gender==0 &v761d==1))
replace sex_short_quarter_condom=0 if sex_quarter==1 &partner_quarter_short==1 & (v761==0|(v792<=304 & gender==0 &v761d==0))

** all people
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_quarter_condom [aweight=sample_weight] if age==`l' 
} 
preserve
collapse (mean) gender_quarter=gender sex_short_quarter_condom=sex_short_quarter_condom (sd) sex_short_quarter_condom_sd=sex_short_quarter_condom (count) observations_quarter=sex_short_quarter_condom [aweight=sample_weight], by (age)
save "$files\3_both_short_quarter1_condom_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_quarter_imp=gender sex_short_quarter_condom_imp=sex_short_quarter_condom_imp (sd) sex_short_quarter_condom_imp_sd=sex_short_quarter_condom_imp (count) observations_quarter_imp=sex_short_quarter_condom_imp [aweight=sample_weight], by (age)
save "$files\3_both_short_quarter2_condom_weighted_2004.dta",replace
restore

** women
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_quarter_condom [aweight=sample_weight] if age==`l'&gender==1
} 
preserve
collapse (mean) gender_quarter=gender sex_short_quarter_condom=sex_short_quarter_condom (sd) sex_short_quarter_condom_sd=sex_short_quarter_condom  (count) observations_quarter=sex_short_quarter_condom [aweight=sample_weight] if gender==1, by (age)
save "$files\3_female_short_quarter1_condom_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_quarter_imp=gender sex_short_quarter_condom_imp=sex_short_quarter_condom_imp (sd) sex_short_quarter_condom_imp_sd=sex_short_quarter_condom_imp  (count) observations_quarter_imp=sex_short_quarter_condom_imp [aweight=sample_weight] if gender==1, by (age)
save "$files\3_female_short_quarter2_condom_weighted_2004.dta",replace
restore

** men
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_quarter_condom [aweight=sample_weight] if age==`l' &gender==0
} 
preserve
collapse (mean) gender_quarter=gender sex_short_quarter_condom=sex_short_quarter_condom (sd) sex_short_quarter_condom_sd=sex_short_quarter_condom  (count) observations_quarter=sex_short_quarter_condom [aweight=sample_weight] if gender==0, by (age)
save "$files\3_male_short_quarter1_condom_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_quarter_imp=gender sex_short_quarter_condom_imp=sex_short_quarter_condom_imp (sd) sex_short_quarter_condom_imp_sd=sex_short_quarter_condom_imp  (count) observations_quarter_imp=sex_short_quarter_condom_imp [aweight=sample_weight] if gender==0, by (age)
save "$files\3_male_short_quarter2_condom_weighted_2004.dta",replace
restore


* LAST YEAR
* short-term market includes here also boyfriend and fiance
* conditional on short-term sex within last year: using the imputed time in months (v529) instead of raw time (v527); for men it's the same
* for men: also including the answers to question "paid for sex"
* this also includes information on sex with 2 other men/women => if one intercourse was without condom the variable is coded equal to zero (no condom used)
gen sex_short_year_condom_imp=1 if sex_year_imp==1 &partner_year_short==1 & (v761==1|v761b==1|v761c==1|(v792<=401 & gender==0 &v761d==1))
replace sex_short_year_condom_imp=0 if sex_year_imp==1 &partner_year_short==1 & (v761==0|v761b==0|v761c==0|(v792<=401 & gender==0 &v761d==0)) 

* conditional on short-term  sex within last year: using raw time (v527) => there might some inconsistencies (check v529 for women)!!!!!!!!!!!!!!
* for men: also including the answers to question "paid for sex"
gen sex_short_year_condom=1 if sex_year==1 &partner_year_short==1 & (v761==1|v761b==1|v761c==1|(v792<=401 & gender==0 &v761d==1))
replace sex_short_year_condom=0 if sex_year==1 &partner_year_short==1 & (v761==0|v761b==0|v761c==0|(v792<=401 & gender==0 &v761d==0)) 


** all people
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_year_condom [aweight=sample_weight] if age==`l' 
} 
preserve
collapse (mean) gender_year=gender sex_short_year_condom=sex_short_year_condom (sd) sex_short_year_condom_sd=sex_short_year_condom (count) observations_year=sex_short_year_condom [aweight=sample_weight], by (age)
save "$files\3_both_short_year1_condom_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_year_imp=gender sex_short_year_condom_imp=sex_short_year_condom_imp (sd) sex_short_year_condom_imp_sd=sex_short_year_condom_imp (count) observations_year_imp=sex_short_year_condom_imp [aweight=sample_weight], by (age)
save "$files\3_both_short_year2_condom_weighted_2004.dta",replace
restore

** women
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_year_condom [aweight=sample_weight] if age==`l'&gender==1
} 
preserve
collapse (mean) gender_year=gender sex_short_year_condom=sex_short_year_condom (sd) sex_short_year_condom_sd=sex_short_year_condom  (count) observations_year=sex_short_year_condom [aweight=sample_weight] if gender==1, by (age)
save "$files\3_female_short_year1_condom_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_year_imp=gender sex_short_year_condom_imp=sex_short_year_condom_imp (sd) sex_short_year_condom_imp_sd=sex_short_year_condom_imp  (count) observations_year_imp=sex_short_year_condom_imp [aweight=sample_weight] if gender==1, by (age)
save "$files\3_female_short_year2_condom_weighted_2004.dta",replace
restore

** men
levelsof age, local (levels)
foreach l of local levels{
sum sex_short_year_condom [aweight=sample_weight] if age==`l' &gender==0
} 
preserve
collapse (mean) gender_year=gender sex_short_year_condom=sex_short_year_condom (sd) sex_short_year_condom_sd=sex_short_year_condom  (count) observations_year=sex_short_year_condom [aweight=sample_weight] if gender==0, by (age)
save "$files\3_male_short_year1_condom_weighted_2004.dta",replace
restore
preserve
collapse (mean) gender_year_imp=gender sex_short_year_condom_imp=sex_short_year_condom_imp (sd) sex_short_year_condom_imp_sd=sex_short_year_condom_imp  (count) observations_year_imp=sex_short_year_condom_imp [aweight=sample_weight] if gender==0, by (age)
save "$files\3_male_short_year2_condom_weighted_2004.dta",replace
restore

* ================================================= *
* == MOMENT 7: Percentage ever married   	     == *
* ================================================= *
* Percentage ever married
* Weighted (sampling weights)

gen married=0 if v502==0
replace married=1 if v502!=0

** all people
levelsof age, local (levels)
foreach l of local levels{
sum married [aweight=sample_weight] if age==`l'
} 
preserve
collapse (mean) gender married=married (sd) married_sd=married  (count) observations= married [aweight=sample_weight], by (age)
save "$files\7_both_married_weighted_2004.dta",replace
restore

** women
levelsof age, local (levels)
foreach l of local levels{
sum married [aweight=sample_weight] if age==`l'& gender==1
} 
preserve
collapse (mean) gender married=married (sd) married_sd=married (count) observations= married [aweight=sample_weight] if gender==1, by (age)
save "$files\7_female_married_weighted_2004.dta",replace
restore

** men
levelsof age, local (levels)
foreach l of local levels{
sum married [aweight=sample_weight] if age==`l'& gender==0
} 
preserve
collapse (mean) gender married=married (sd) married_sd=married (count) observations= married [aweight=sample_weight] if gender==0, by (age)
save "$files\7_male_married_weighted_2004.dta",replace
restore

* ================================================== *
* == MOMENT Single: Percentage of current singles == *
* ================================================== *
* Percentage of current singles 
* Weighted (sampling weights)

* single 1:= living together is also coded as "single"
gen single=0 if v501==1
replace single=1 if v501!=1

* single 2:= living together is coded as "no single"
gen single_living=0 if v501==1|v501==2
replace single_living=1 if v501!=1&v501!=2 

** all people
* single 1
levelsof age, local (levels)
foreach l of local levels{
sum single [aweight=sample_weight] if age==`l'
} 
preserve
collapse (mean) gender=gender single=single (sd) single_sd=single (count) observations=single [aweight=sample_weight], by (age)
save "$files\7_both_single1_weighted_2004.dta",replace
restore

* single 2
levelsof age, local (levels)
foreach l of local levels{
sum single_living [aweight=sample_weight] if age==`l'
} 
preserve
collapse (mean) gender_living=gender single_living=single_living (sd) single_living_sd=single_living (count) observations_living=single [aweight=sample_weight], by (age)
save "$files\7_both_single2_weighted_2004.dta",replace
restore

** women
* single 1
levelsof age, local (levels)
foreach l of local levels{
sum single [aweight=sample_weight] if age==`l' &gender==1
} 
preserve
collapse (mean) gender single=single (sd) single_sd=single (count) observations=single [aweight=sample_weight] if gender==1, by (age)
save "$files\7_female_single1_weighted_2004.dta",replace
restore

*single 2
levelsof age, local (levels)
foreach l of local levels{
sum single_living [aweight=sample_weight] if age==`l' &gender==1
} 
preserve
collapse (mean) gender_living=gender single_living=single_living (sd) single_living_sd=single_living (count) observations_living=single_living [aweight=sample_weight] if gender==1, by (age)
save "$files\7_female_single2_weighted_2004.dta",replace
restore

** men
* single 1
levelsof age, local (levels)
foreach l of local levels{
sum single [aweight=sample_weight] if age==`l' &gender==0
} 
preserve
collapse (mean) gender single=single (sd) single_sd=single (count) observations=single [aweight=sample_weight] if gender==0, by (age)
save "$files\7_male_single1_weighted_2004.dta",replace
restore

* single 2
levelsof age, local (levels)
foreach l of local levels{
sum single_living [aweight=sample_weight] if age==`l' &gender==0
} 
preserve
collapse (mean) gender_living=gender single_living=single_living (sd) single_living_sd=single_living (count) observations_living=single_living [aweight=sample_weight] if gender==0, by (age)
save "$files\7_male_single2_weighted_2004.dta",replace
restore


* ========================================================== *
* == MOMENT Single Sex: Percentage of singles having sex  == *
* ========================================================== *
* Weighted (sampling weights)
* single 1:= living together is also coded as "single"

*** last quarter
** all people
preserve
collapse (mean) gender=gender single_sex_quarter_imp=sex_quarter_imp (sd) single_sex_quarter_imp_sd=sex_quarter_imp (count) observations_single_sex_quarter=sex_quarter_imp [aweight=sample_weight] if single==1 & gender==1, by (age)
save "$files\9_both_single_sex_quarter_weighted_2004.dta",replace
restore

** women
preserve
collapse (mean) gender=gender single_sex_quarter_imp=sex_quarter_imp (sd) single_sex_quarter_imp_sd=sex_quarter_imp (count) observations_single_sex_quarter=sex_quarter_imp [aweight=sample_weight] if single==1 & gender==1, by (age)
save "$files\9_female_single_sex_quarter_weighted_2004.dta",replace
restore

** men
preserve
collapse (mean) gender=gender single_sex_quarter_imp=sex_quarter_imp (sd) single_sex_quarter_imp_sd=sex_quarter_imp (count) observations_single_sex_quarter=sex_quarter_imp [aweight=sample_weight] if single==1 & gender==0, by (age)
save "$files\9_male_single_sex_quarter_weighted_2004.dta",replace
restore

*** last year
** all people
preserve

collapse (mean) gender=gender single_sex_year_imp=sex_year_imp (sd) single_sex_year_imp_sd=sex_year_imp (count) observations_single_sex_year=sex_year_imp [aweight=sample_weight], by (age)

save "$files\9_both_single_sex_year_weighted_2004.dta",replace
restore

** women
preserve
collapse (mean) gender=gender single_sex_year_imp=sex_year_imp (sd) single_sex_year_imp_sd=sex_year_imp (count) observations_single_sex_year=sex_year_imp [aweight=sample_weight] if single==1 & gender==1, by (age)
save "$files\9_female_single_sex_year_weighted_2004.dta",replace
restore

** men
preserve
collapse (mean) gender=gender single_sex_year_imp=sex_year_imp (sd) single_sex_year_imp_sd=sex_year_imp (count) observations_single_sex_year=sex_year_imp [aweight=sample_weight] if single==1 & gender==0, by (age)
save "$files\9_male_single_sex_year_weighted_2004.dta",replace
restore



**** Computation of mean moments by gender 

*** mean by gender

preserve
collapse (mean) single_sex_year_imp=sex_year_imp (sd) single_sex_year_imp_sd=sex_year_imp (count) observations_single_sex_year=sex_year_imp [aweight=sample_weight] if single==1, by (gender)
save "$outdat\single_sex_incliving_bygender2004.dta",replace
restore

preserve
collapse (mean) single_sex_year_imp=sex_year_imp (sd) single_sex_year_imp_sd=sex_year_imp (count) observations_single_sex_year=sex_year_imp [aweight=sample_weight] if single_living==1, by (gender)
save "$outdat\single_sex_noliving_bygender2004.dta",replace
restore

preserve
collapse (mean) single [aweight=sample_weight], by (gender)
save "$outdat\frac_single_incliving_bygender2004.dta",replace
restore

preserve
collapse (mean) single_living [aweight=sample_weight], by (gender)
save "$outdat\frac_single_noliving_bygender2004.dta",replace
restore



preserve

collapse (mean) partner_year_short [aweight=sample_weight], by (gender)
gen fraction=partner_year_short/sex_year 

save "$outdat\frac_casualsex_bygender2004.dta",replace
restore





* ======================================================= *
* ==               SOME DATA SAVING 			       == *
* ======================================================= *
/*
** Moment 1 - HIV fraction
use "$files\1_both_HIVrate_weighted_2004.dta", clear 
append using  "$files\1_male_HIVrate_weighted_2004.dta"
append using  "$files\1_female_HIVrate_weighted_2004.dta"
outsheet using "$files\1_all_HIVrate_weighted_2004.csv", delimiter (",") replace
export excel  "$files\1_all_HIVrate_weighted_2004", firstrow(variables) replace
egen index=seq()
save "$files\1_all_HIVrate_weighted_2004", replace
*/
** Moment 2 - fraction of causal sex
* quarter and year
use "$files\2_both_shortTerm1_quarter_weighted_2004.dta",clear
merge 1:1 age using "$files\2_both_shortTerm2_quarter_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\2_both_shortTerm1_year_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\2_both_shortTerm2_year_weighted_2004.dta"
drop _merge
save "$files\2_both_shortTerm_weighted_2004.dta",replace

use "$files\2_female_shortTerm1_quarter_weighted_2004.dta",clear
merge 1:1 age using "$files\2_female_shortTerm2_quarter_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\2_female_shortTerm1_year_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\2_female_shortTerm2_year_weighted_2004.dta"
drop _merge
save "$files\2_female_shortTerm_weighted_2004.dta",replace

use "$files\2_male_shortTerm1_quarter_weighted_2004.dta",clear
merge 1:1 age using "$files\2_male_shortTerm2_quarter_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\2_male_shortTerm1_year_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\2_male_shortTerm2_year_weighted_2004.dta"
drop _merge
save "$files\2_male_shortTerm_weighted_2004.dta",replace

use "$files\2_both_shortTerm_weighted_2004.dta", clear 
append using  "$files\2_male_shortTerm_weighted_2004.dta"
append using  "$files\2_female_shortTerm_weighted_2004.dta"
outsheet using "$files\2_all_shortTerm_weighted_2004.csv", delimiter (",") replace
export excel  "$files\2_all_shortTerm_weighted_2004", firstrow(variables) replace
egen index=seq()
save "$files\2_all_shortTerm_weighted_2004", replace

* alternative
use "$files\2_both_shortTerm_alternative_weighted_2004.dta", clear 
append using  "$files\2_male_shortTerm_alternative_weighted_2004.dta"
append using  "$files\2_female_shortTerm_alternative_weighted_2004.dta"
outsheet using "$files\2_all_shortTerm_alternative_weighted_2004.csv", delimiter (",") replace

** Moment 3 - condom usage
* quarter and year
use "$files\3_both_short_quarter1_condom_weighted_2004.dta",clear
merge 1:1 age using "$files\3_both_short_quarter2_condom_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\3_both_short_year1_condom_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\3_both_short_year2_condom_weighted_2004.dta"
drop _merge
save "$files\3_both_short_condom_weighted_2004.dta",replace

use "$files\3_female_short_quarter1_condom_weighted_2004.dta",clear
merge 1:1 age using "$files\3_female_short_quarter2_condom_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\3_female_short_year1_condom_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\3_female_short_year2_condom_weighted_2004.dta"
drop _merge
save "$files\3_female_short_condom_weighted_2004.dta",replace

use "$files\3_male_short_quarter1_condom_weighted_2004.dta",clear
merge 1:1 age using "$files\3_male_short_quarter2_condom_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\3_male_short_year1_condom_weighted_2004.dta"
drop _merge
merge 1:1 age using "$files\3_male_short_year2_condom_weighted_2004.dta"
drop _merge
save "$files\3_male_short_condom_weighted_2004.dta",replace

use "$files\3_both_short_condom_weighted_2004.dta", clear 
append using  "$files\3_male_short_condom_weighted_2004.dta"
append using  "$files\3_female_short_condom_weighted_2004.dta"
outsheet using "$files\3_all_short_condom_weighted_2004.csv", delimiter (",") replace
export excel  "$files\3_all_short_condom_weighted_2004", firstrow(variables) replace
egen index=seq()
save "$files\3_all_short_condom_weighted_2004", replace

* Moment 7 - ever married
use "$files\7_both_married_weighted_2004.dta",clear
append using "$files\7_male_married_weighted_2004.dta"
append using "$files\7_female_married_weighted_2004.dta"
outsheet using "$files\7_all_married_weighted_2004.csv", delimiter (",") replace
export excel  "$files\7_all_married_weighted_2004", firstrow(variables) replace
egen index=seq()
save "$files\7_all_married_weighted_2004", replace

* Moment single
use "$files\7_both_single1_weighted_2004.dta",clear
merge 1:1 age using "$files\7_both_single2_weighted_2004.dta"
drop _merge
save "$files\7_both_single_weighted_2004",replace

use "$files\7_female_single1_weighted_2004.dta",clear
merge 1:1 age using "$files\7_female_single2_weighted_2004.dta"
drop _merge
save "$files\7_female_single_weighted_2004",replace

use "$files\7_male_single1_weighted_2004.dta",clear
merge 1:1 age using "$files\7_male_single2_weighted_2004.dta"
drop _merge
save "$files\7_male_single_weighted_2004",replace

use "$files\7_both_single_weighted_2004.dta",clear
append using "$files\7_male_single_weighted_2004.dta"
append using "$files\7_female_single_weighted_2004.dta"
outsheet using "$files\7_all_single_weighted_2004.csv", delimiter (",") replace
export excel  "$files\7_all_single_weighted_2004", firstrow(variables) replace
egen index=seq()
save "$files\7_all_single_weighted_2004", replace



* Moment "singles having sex"
* quarter and year
use "$files\9_both_single_sex_quarter_weighted_2004.dta",clear
merge 1:1 age using "$files\9_both_single_sex_year_weighted_2004.dta"
drop _merge
save "$files\9_both_single_sex_weighted_2004.dta",replace

use "$files\9_female_single_sex_quarter_weighted_2004.dta",clear
merge 1:1 age using "$files\9_female_single_sex_year_weighted_2004.dta"
drop _merge
save "$files\9_female_single_sex_weighted_2004.dta",replace

use "$files\9_male_single_sex_quarter_weighted_2004.dta",clear
merge 1:1 age using "$files\9_male_single_sex_year_weighted_2004.dta"
drop _merge
save "$files\9_male_single_sex_weighted_2004.dta",replace

use "$files\9_both_single_sex_weighted_2004.dta", clear 
append using  "$files\9_male_single_sex_weighted_2004.dta"
append using "$files\9_female_single_sex_weighted_2004.dta"
outsheet using "$files\9_all_single_sex_weighted_2004.csv", delimiter (",") replace
export excel  "$files\9_all_single_sex_weighted_2004", firstrow(variables) replace
egen index=seq()
save "$files\9_all_single_sex_weighted_2004", replace

