set more off, permanently

* Insert absolute path of HIV_proj folder on your disk here
cd "C:\Users\YOU\subj_HIV_reduction"

clear all

* Set relative paths
global workdata= ".\workdata\"
global rawdata= ".\rawdata\"
global outfig= ".\outfig\"
global outdat= ".\outdat\"


*1992
* women
use "$rawdata\MWIR22FL.dta"

gen year=1992

gen abstinence=(s808a==1)
gen condom=(s808c==1)
gen mosquito=(s804g==1)
gen healthy=(s805==1)
rename v005 weight

keep year abstinence condom mosquito healthy weight


save "$workdata\dat92.dta",replace

* men
use "$rawdata\MWMR21FL.dta"

gen year=1992

gen abstinence=(mv754b==1)
gen condom=(mv754c==1)
gen mosquito=(sm504g==1)
gen healthy=(mv756==1)
rename mv005 weight

keep year abstinence condom mosquito healthy weight


save "$workdata\dat92m.dta",replace






*1996
*women
use "$rawdata\MWIQ34FL.dta",replace

gen year=1996

gen healthy=(q507==1)
gen abstinence=(strpos(qa509a,"B")>0)
replace abstinence= . if missing(qa509a)
gen condom=(strpos(qa504,"D")>0)
replace condom= . if missing(qa504)
*gen mosquito=(strpos(q509a,"I")>0)
rename qweight weight

keep year healthy condom abstinence weight

save "$workdata\dat96.dta", replace

*men
use "$rawdata\mwml34fl.dta",replace

gen year=1996

gen healthy=(qm507==1)
gen abstinence=(strpos(qma509a,"B")>0)
replace abstinence= . if missing(qma509a)
gen condom=(strpos(qma504,"D")>0)
replace condom= . if missing(qma504)
*gen mosquito=(strpos(qm509a,"I")>0)
rename qweight weight

keep year healthy condom abstinence weight

save "$workdata\dat96m.dta", replace





*2000
* women
use "$rawdata\MWIR41FL.dta",replace

gen year=2000

gen abstinence=(v754b==1)
gen condom=(v754c==1)
gen mosquito=(v754jp==1)
gen healthy=(v756==1)
rename v005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat00.dta",replace

* men
use "$rawdata\MWMR41FL.dta",replace

gen year=2000

gen abstinence=(mv754b==1)
gen condom=(mv754c==1)
gen mosquito=(mv754jp==1)
gen healthy=(mv756==1)
rename mv005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat00m.dta",replace


*2004
*women
use "$rawdata\MWIR4EFL.dta", replace

gen year=2004

gen abstinence=(v754b==1)
gen condom=(v754c==1)
gen mosquito=(v754jp==1)
gen healthy=(v756==1)
rename v005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat04.dta",replace

*men
use "$rawdata\MWMR4EFL.dta", replace

gen year=2004

gen abstinence=(mv754b==1)
gen condom=(mv754c==1)
gen mosquito=(mv754jp==1)
gen healthy=(mv756==1)
rename mv005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat04m.dta",replace







*2010
*women
use "$rawdata\MWIR61FL.dta", replace

gen year=2010

gen abstinence=(v754b==1)
gen condom=(v754c==1)
gen mosquito=(v754jp==1)
gen healthy=(v756==1)
rename v005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat10.dta",replace


* men
use "$rawdata\MWMR61FL.dta", replace

gen year=2010

gen abstinence=(mv754b==1)
gen condom=(mv754c==1)
gen mosquito=(mv754jp==1)
gen healthy=(mv756==1)
rename mv005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat10m.dta",replace



*2015/16
*women
use "$rawdata\MWIR7HFL.dta", replace

gen year=2015

gen abstinence=(v754b==1)
gen condom=(v754c==1)
gen mosquito=(v754jp==1)
gen healthy=(v756==1)
rename v005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat1516.dta",replace


*men
use "$rawdata\MWMR7HFL.dta", replace

gen year=2015

gen abstinence=(mv754b==1)
gen condom=(mv754c==1)
gen mosquito=(mv754jp==1)
gen healthy=(mv756==1)
rename mv005 weight

keep year abstinence condom mosquito healthy weight

save "$workdata\dat1516m.dta",replace





use "$workdata\dat92.dta",replace


foreach i in 96 00 04 10 1516{

append using "$workdata\dat`i'.dta"

}


foreach i in 92 96 00 04 10 1516{

append using "$workdata\dat`i'm.dta"

}


*!!! Remark: Averages are computed using sample weights (here men and women are not weighted equally but according to the sample weights).
preserve

collapse abstinence condom healthy mosquito [w=weight], by(year)
outsheet using "$outdat\subj_HIV_means.csv", delimiter(";") replace

restore



preserve

collapse abstinence condom healthy mosquito, by(year)
outsheet using "$outdat\subj_HIV_means_noweights.csv", delimiter(";") replace

restore




replace abstinence=100*abstinence
replace condom=100*condom
replace mosquito=100*mosquito
replace healthy=100*healthy

*drop if (missing(abstinence)|missing(condom)|missing(healthy)|missing(mosquito))&(year!=1996)

graph bar (mean) abstinence [w=weight],over(year) yscale(range(0,100)) ylabel(0(20)100) ytitle(Percent) title(Percentage identifying abstinence reduces risk)
graph export "$outfig\abstinence.pdf", replace

graph bar (mean) condom [w=weight],over(year) yscale(range(0,100)) ylabel(0(20)100)  ytitle(Percent) title(Percentage identifying condom reduces risk)
graph export "$outfig\condom.pdf", replace


graph bar (mean) healthy [w=weight],over(year) yscale(range(0,100)) ylabel(0(20)100)  ytitle(Percent) title("Percentage identifying that a" "healthy looking person can have AIDS")
graph export "$outfig\healthy.pdf", replace


graph bar (mean) mosquito [w=weight],over(year) yscale(range(0,100)) ylabel(0(20)100)  ytitle(Percent) title("Percentage believing AIDS can be transmitted" "through mosquito bites")
graph export "$outfig\mosquito.pdf", replace

bys year: su abstinence
bys year: su condom
bys year: su healthy
bys year: su mosquito









