program aids_vfi
use mod_param
implicit none

!Define variables:
real(mykind) :: guess(3), excess_vector(3), step(3)
integer :: ii

!Number of variables in optimization (prices):
integer, parameter :: n = 3

!Declaring variables for newuoa:
integer :: MAXFUN, iprint
integer, parameter :: mv = n
integer, parameter :: NPT = 2*n+1
real(mykind) ::  RHOBEG, RHOEND
real(mykind) :: ww((NPT+11)*(NPT+n)+n*(3*n+11)/2)
real(mykind) :: v_err(mv)

!---------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------

!Read in the parameters from Matlab (they're declared in mod_param)
call read_parameters_all

!Allocate arrays:
call allocate_arrays

!Read type-specific parameters:
call read_parameters_bytype

!Read the search effort grids:
call read_fit_search

!Start flags for convergence:
conv_flags = 0

!Determine initial guesses
call initial_guess

!Set prices to guess vector:
guess(1) = t_p
guess(2) = t_u
guess(3) = t_l

!Set the step for the prices:
step(1) = ge_step
step(2) = ge_step
step(3) = ge_step

!Set flag not to write distributions and policy functions during iterations:
write_more_files = 0

!Call the GE version (if chosen):
if (do_ge==1 .and. exec_bench==1) then
	guess_optimization = guess
	step_optimization = step
	guess = 0.

	IPRINT = 3
	MAXFUN = ge_max_iter !100*(n+1)
	RHOBEG = 1.
	RHOEND = 10**(-8)

	CALL NEWUOA_H(n,NPT,guess,RHOBEG,RHOEND,IPRINT,MAXFUN,ww,MV)

endif

!Normalize the optimizer (if needed):
if (do_ge==1 .and. exec_bench==1) then
	prices = guess_optimization + guess*step_optimization
else
	prices = guess
endif

!Change flag to write all files with the optimizer:
write_more_files = 1

!Call excess demand with final prices:
if (exec_bench==1) then
    call excess_demand(prices,excess_vector)
    ed_vector = excess_vector !ed_vector is global variable that is written out
endif

!Call epidemiological model (if needed)
if (exec_epid==1) then
    !Call the file that runs the experiment:
    call epid(prices,excess_vector)
endif

!Call small scale field experiment (if needed)
if (exec_smallexp==1) then
    !Call the file that runs the experiment:
    call smallexp(prices,excess_vector)
endif

!Check if the excess demands are "small enough":
if (sum(abs(excess_vector))<ge_tol) then
	conv_flags(2) = 1
endif

!Write things to a file:
call writeout_files

!Deallocate arrays:
call deallocate_arrays

print *, 'Fortran is done!!!'

end program aids_vfi
