subroutine epid(guess,excess_vector)
!This function solves epidemiological experiment for the AIDS/Malawi project given the prices.
!The inputs are the prices of each market (t_p,t_u,t_l): protected, unprotected and long term markets, respectively.
use mod_param
implicit none

!Define dummy variables:
real(mykind), intent(in) :: guess(3)
real(mykind), intent(out) :: excess_vector(3)

!Define variables for the excess demands:
real(mykind) :: ed_p
real(mykind) :: ed_u
real(mykind) :: ed_l
real(mykind) :: normalize

!Define variables for search effort (pi's):
real(mykind) :: pi_p_y_m(n_status,ntypes_m)
real(mykind) :: pi_p_m(n_status,ntypes_m)
real(mykind) :: pi_p_y_f(n_status,ntypes_f)
real(mykind) :: pi_p_f(n_status,ntypes_f)
real(mykind) :: pi_u_y_m(n_status,ntypes_m)
real(mykind) :: pi_u_m(n_status,ntypes_m)
real(mykind) :: pi_u_y_f(n_status,ntypes_f)
real(mykind) :: pi_u_f(n_status,ntypes_f)
real(mykind) :: pi_l_y_m(n_status,ntypes_m)
real(mykind) :: pi_l_m(n_status,ntypes_m)
real(mykind) :: pi_l_y_f(n_status,ntypes_f)
real(mykind) :: pi_l_f(n_status,ntypes_f)

!Define variables for distributions - singles:
real(mykind) :: dist_S_m(n_status,ntypes_m)
real(mykind) :: dist_S_y_m(n_status,ntypes_m)
real(mykind) :: dist_S_f(n_status,ntypes_f)
real(mykind) :: dist_S_y_f(n_status,ntypes_f)
real(mykind) :: dist_S_m0(n_status,ntypes_m)
real(mykind) :: dist_S_y_m0(n_status,ntypes_m)
real(mykind) :: dist_S_f0(n_status,ntypes_f)
real(mykind) :: dist_S_y_f0(n_status,ntypes_f)

!Define variables for temporary distributions - singles:
real(mykind) :: dist_S_m_pr(n_status,ntypes_m)
real(mykind) :: dist_S_m_prpr(n_status,ntypes_m)
real(mykind) :: dist_S_f_pr(n_status,ntypes_f)
real(mykind) :: dist_S_f_prpr(n_status,ntypes_f)
real(mykind) :: dist_S_y_m_pr(n_status,ntypes_m)
real(mykind) :: dist_S_y_m_prpr(n_status,ntypes_m)
real(mykind) :: dist_S_y_f_pr(n_status,ntypes_f)
real(mykind) :: dist_S_y_f_prpr(n_status,ntypes_f)

!Define variables for distributions - married:
real(mykind) :: dist_L_m(n_status,ntypes_m)
real(mykind) :: dist_L_y_m(n_status,ntypes_m)
real(mykind) :: dist_L_f(n_status,ntypes_f)
real(mykind) :: dist_L_y_f(n_status,ntypes_f)
real(mykind) :: dist_L_opp_m(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_m(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_f(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_f(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_m0(n_status,ntypes_m)
real(mykind) :: dist_L_y_m0(n_status,ntypes_m)
real(mykind) :: dist_L_f0(n_status,ntypes_f)
real(mykind) :: dist_L_y_f0(n_status,ntypes_f)
real(mykind) :: dist_L_opp_m0(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_m0(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_f0(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_f0(n_status,n_status,ntypes_f)

!Define variables for temporary distributions - married:
real(mykind) :: dist_L_opp_m_pr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_m_prpr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_f_pr(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_f_prpr(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_m_pr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_m_prpr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_f_pr(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_f_prpr(n_status,n_status,ntypes_f)

!Define variables for auxiliary distributions for circumcision status of husbands:
real(mykind) :: dist_L_opp_bycirc_f(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_f0(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_f_pr(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_f_prpr(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f0(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f_pr(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f_prpr(n_status,n_circ,ntypes_f)

!Define variables for temporary distributions - with symptoms:
real(mykind) :: dist_A_m(n_status,ntypes_m)
real(mykind) :: dist_A_f(n_status,ntypes_f)
real(mykind) :: dist_A_m0(n_status,ntypes_m)
real(mykind) :: dist_A_f0(n_status,ntypes_f)

!Define variables for infectiousness:
real(mykind) :: upsilon_p_m
real(mykind) :: upsilon_u_m
real(mykind) :: upsilon_p_f
real(mykind) :: upsilon_u_f
real(mykind) :: upsilon_all(4)

!Define variables for who goes into each market:
real(mykind) :: measure_p_m(n_status)
real(mykind) :: measure_p_y_m(n_status)
real(mykind) :: measure_u_m(n_status)
real(mykind) :: measure_u_y_m(n_status)
real(mykind) :: measure_l_m(n_status)
real(mykind) :: measure_l_y_m(n_status)
real(mykind) :: measure_a_m(n_status)
real(mykind) :: measure_a_y_m(n_status)
real(mykind) :: measure_p_f(n_status)
real(mykind) :: measure_p_y_f(n_status)
real(mykind) :: measure_u_f(n_status)
real(mykind) :: measure_u_y_f(n_status)
real(mykind) :: measure_l_f(n_status)
real(mykind) :: measure_l_y_f(n_status)
real(mykind) :: measure_a_f(n_status)
real(mykind) :: measure_a_y_f(n_status)

!Define variables for fraction of people with different statuses in long term relationships:
real(mykind) :: frac_healthy_nocirc_m
real(mykind) :: frac_healthy_circ_m
real(mykind) :: frac_sick_m
real(mykind) :: frac_treated_m
real(mykind) :: frac_sick_f
real(mykind) :: frac_treated_f
real(mykind) :: frac_l_m_vec(n_status)
real(mykind) :: frac_l_f_vec(n_status)
real(mykind) :: frac_l_healthy_bycirc_m(n_circ)

!Define index variables:
integer :: iter, ii, jj, kk, which_entry

!Define other variables:
real(mykind) :: norm, norm_vec(22), prices_vec(3)

!----------------------------------------------------------------
!----------------------------------------------------------------
!----------------------------------------------------------------

!GET PRICES:
open(11,file='prices_bench.txt',status='old')
    read(11,*) prices_vec
close(11)
t_p = prices_vec(1)
t_u = prices_vec(2)
t_l = prices_vec(3)

!GET THE POLICY FUNCTIONS FROM THE BENCHMARK:
!Males:
open(11,file='pi_p_m_bench.txt',status='old')
do ii=1,ntypes_m
    do jj=1,n_status
        read(11,*) pi_p_m(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_p_y_m_bench.txt',status='old')
do ii=1,ntypes_m
    do jj=1,n_status
        read(11,*) pi_p_y_m(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_u_m_bench.txt',status='old')
do ii=1,ntypes_m
    do jj=1,n_status
        read(11,*) pi_u_m(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_u_y_m_bench.txt',status='old')
do ii=1,ntypes_m
    do jj=1,n_status
        read(11,*) pi_u_y_m(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_l_m_bench.txt',status='old')
do ii=1,ntypes_m
    do jj=1,n_status
        read(11,*) pi_l_m(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_l_y_m_bench.txt',status='old')
do ii=1,ntypes_m
    do jj=1,n_status
        read(11,*) pi_l_y_m(jj,ii)
    enddo
enddo
close(11)
!Females:
open(11,file='pi_p_f_bench.txt',status='old')
do ii=1,ntypes_f
    do jj=1,n_status
        read(11,*) pi_p_f(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_p_y_f_bench.txt',status='old')
do ii=1,ntypes_f
    do jj=1,n_status
        read(11,*) pi_p_y_f(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_u_f_bench.txt',status='old')
do ii=1,ntypes_f
    do jj=1,n_status
        read(11,*) pi_u_f(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_u_y_f_bench.txt',status='old')
do ii=1,ntypes_f
    do jj=1,n_status
        read(11,*) pi_u_y_f(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_l_f_bench.txt',status='old')
do ii=1,ntypes_f
    do jj=1,n_status
        read(11,*) pi_l_f(jj,ii)
    enddo
enddo
close(11)
open(11,file='pi_l_y_f_bench.txt',status='old')
do ii=1,ntypes_f
    do jj=1,n_status
        read(11,*) pi_l_y_f(jj,ii)
    enddo
enddo
close(11)

print*, 'Evaluating the epidemiological experiment at the following prices'
print *, 't_p=', t_p, 't_u=', t_u, 't_l=',t_l
print *, ''

!TAKE INITIAL GUESSES:
dist1_m = dist_next_m
dist1_f = dist_next_f
v1_m = v_next_m
v1_f = v_next_f
prev1 = prev_next

!PREVALENCE RATES:
phi_p_m = prev1(1)
phi_u_m = prev1(2)
phi_l_m = prev1(3)
phi_p_f = prev1(4)
phi_u_f = prev1(5)
phi_l_f = prev1(6)
phi_n_m = prev1(7)
phi_n_f = prev1(8)

!Fraction of sick/treated people in the long term market - males:
frac_sick_m = 0.10
frac_treated_m = 0.0
frac_l_m_vec(1) = 1.0 - frac_sick_m - frac_treated_m
frac_l_m_vec(2) = frac_sick_m
frac_l_m_vec(3) = frac_treated_m
frac_healthy_nocirc_m = (1.0 - frac_sick_m - frac_treated_m)*(1.0-frac_circ)
frac_healthy_circ_m = (1.0 - frac_sick_m - frac_treated_m)*frac_circ
frac_l_healthy_bycirc_m(i_nocirc) = frac_healthy_nocirc_m
frac_l_healthy_bycirc_m(i_circ)   = frac_healthy_circ_m

!Fraction of sick/treated people in the long term market - females:
frac_sick_f = 0.10
frac_treated_f = 0.0
frac_l_f_vec(1) = 1.0 - frac_sick_f - frac_treated_f
frac_l_f_vec(2) = frac_sick_f
frac_l_f_vec(3) = frac_treated_f

!DISTRIBUTIONS:

!Males - short term:
do ii=1,ntypes_m
    !old:
    which_entry = (ii-1)*n_status*n_age
    dist_S_m(i_healthy,ii) = dist1_m(which_entry+i_healthy)
    dist_S_m(i_sick,ii)    = dist1_m(which_entry+i_sick)
    dist_S_m(i_treated,ii) = dist1_m(which_entry+i_treated)
    !young:
    which_entry = (ii-1)*n_status*n_age + n_status
    dist_S_y_m(i_healthy,ii) = dist1_m(which_entry+i_healthy)
    dist_S_y_m(i_sick,ii)    = dist1_m(which_entry+i_sick)
    dist_S_y_m(i_treated,ii) = dist1_m(which_entry+i_treated)
enddo

!Females - short term:
do ii=1,ntypes_f
    !old:
    which_entry = (ii-1)*n_status*n_age
    dist_S_f(i_healthy,ii) = dist1_f(which_entry+i_healthy)
    dist_S_f(i_sick,ii)    = dist1_f(which_entry+i_sick)
    dist_S_f(i_treated,ii) = dist1_f(which_entry+i_treated)
    !young:
    which_entry = (ii-1)*n_status*n_age + n_status
    dist_S_y_f(i_healthy,ii) = dist1_f(which_entry+i_healthy)
    dist_S_y_f(i_sick,ii)    = dist1_f(which_entry+i_sick)
    dist_S_y_f(i_treated,ii) = dist1_f(which_entry+i_treated)
enddo

!Males - long term:
dist_L_m = 2.0*dist_S_m
dist_L_y_m = 2.0*dist_S_y_m
do jj=1,n_status
    dist_L_opp_m(:,jj,:) = 1.0/3.0*dist_L_m
    dist_L_opp_y_m(:,jj,:) = 1.0/3.0*dist_L_y_m
enddo

!Females - long term:
dist_L_f = 2.0*dist_S_f
dist_L_y_f = 2.0*dist_S_y_f
do jj=1,n_status
    dist_L_opp_f(:,jj,:) = 1.0/3.0*dist_L_f
    dist_L_opp_y_f(:,jj,:) = 1.0/3.0*dist_L_y_f
enddo

!With symptoms:
dist_A_m = alpha*dist_S_m
dist_A_f = alpha*dist_S_f

!Infectiousness in short term market:
upsilon_p_m = (1.0-gammap_m)*phi_p_m
upsilon_u_m = (1.0-gammau_m)*phi_u_m
upsilon_p_f = (1.0-gammap_f)*phi_p_f
upsilon_u_f = (1.0-gammau_f)*phi_u_f

!SET INITIAL GUESSES:
dist_S_m0 = dist_S_m
dist_S_y_m0 = dist_S_y_m
dist_L_m0 = dist_L_m
dist_L_y_m0 = dist_L_y_m
dist_L_opp_y_m0 = dist_L_opp_y_m
dist_A_m0 = dist_A_m
dist_S_f0 = dist_S_f
dist_S_y_f0 = dist_S_y_f
dist_L_f0 = dist_L_f
dist_L_y_f0 = dist_L_y_f
dist_L_opp_y_f0 = dist_L_opp_y_f
dist_A_f0 = dist_A_f
dist_L_opp_bycirc_f(:,i_nocirc,:) = dist_L_opp_f(:,i_healthy,:) * (1.0-frac_circ)
dist_L_opp_bycirc_f(:,i_circ,:) = dist_L_opp_f(:,i_healthy,:) * frac_circ
dist_L_opp_bycirc_f0 = dist_L_opp_bycirc_f
dist_L_opp_bycirc_y_f(:,i_nocirc,:) = dist_L_opp_y_f(:,i_healthy,:) * (1.0-frac_circ)
dist_L_opp_bycirc_y_f(:,i_circ,:) = dist_L_opp_y_f(:,i_healthy,:) * frac_circ
dist_L_opp_bycirc_y_f0 = dist_L_opp_bycirc_y_f

!MAIN LOOP:
iter = 0

do iter=1,max_iter

    !=======================================================
    ! PRELIMINARIES
    !=======================================================

    !Update distributions - singles:
    dist_S_m0 = ddd*dist_S_m + (1.0-ddd)*dist_S_m0
    dist_S_y_m0 = ddd*dist_S_y_m + (1.0-ddd)*dist_S_y_m0
    dist_S_f0 = ddd*dist_S_f + (1.0-ddd)*dist_S_f0
	dist_S_y_f0 = ddd*dist_S_y_f + (1.0-ddd)*dist_S_y_f0

	!Update distributions - married:
	dist_L_m0 = ddd*dist_L_m + (1.0-ddd)*dist_L_m0
	dist_L_y_m0 = ddd*dist_L_y_m + (1.0-ddd)*dist_L_y_m0
    dist_L_opp_m0 = ddd*dist_L_opp_m + (1.0-ddd)*dist_L_opp_m0
    dist_L_opp_y_m0 = ddd*dist_L_opp_y_m + (1.0-ddd)*dist_L_opp_y_m0
    dist_L_f0 = ddd*dist_L_f + (1.0-ddd)*dist_L_f0
	dist_L_y_f0 = ddd*dist_L_y_f + (1.0-ddd)*dist_L_y_f0
    dist_L_opp_f0 = ddd*dist_L_opp_f + (1.0-ddd)*dist_L_opp_f0
    dist_L_opp_y_f0 = ddd*dist_L_opp_y_f + (1.0-ddd)*dist_L_opp_y_f0
    dist_L_opp_bycirc_f0 = ddd*dist_L_opp_bycirc_f +  (1.0-ddd)*dist_L_opp_bycirc_f0
    dist_L_opp_bycirc_y_f0 = ddd*dist_L_opp_bycirc_y_f +  (1.0-ddd)*dist_L_opp_bycirc_y_f0

    !Update distributions - with symptoms:
    dist_A_m0 = ddd*dist_A_m + (1.0-ddd)*dist_A_m0
    dist_A_f0 = ddd*dist_A_f + (1.0-ddd)*dist_A_f0

    !=======================================================
    ! DISTRIBUTIONS
    !=======================================================

    !Distributions for males:
    do ii=1,ntypes_m
        !SINGLES:
        !Before changing discount factors or treatment status - old agents:
        dist_S_m_pr(i_healthy,ii) = (1.0-delta) * ( &
                                    (1.0-pi_l_m(i_healthy,ii))*dist_S_m0(i_healthy,ii)*( 1.0-pi_p_m(i_healthy,ii)-pi_u_m(i_healthy,ii) + pi_p_m(i_healthy,ii)*(1.0-upsilon_p_f*chi_m(ii)) + pi_u_m(i_healthy,ii)*(1.0-upsilon_u_f*chi_m(ii)) ) + &
                                    sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_m(i_healthy,ii)*dist_S_m0(i_healthy,ii) ) ) &
                                    )
        dist_S_m_pr(i_sick,ii)    = (1.0-delta) * ( &
                                    (1.0-pi_l_m(i_healthy,ii))*dist_S_m0(i_healthy,ii)*( pi_p_m(i_healthy,ii)*upsilon_p_f*chi_m(ii) + pi_u_m(i_healthy,ii)*upsilon_u_f*chi_m(ii) ) + &
                                    (1.0-alpha)*(1.0-pi_l_m(i_sick,ii))*dist_S_m0(i_sick,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii)*( dist_L_opp_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_m(i_healthy,ii)*dist_S_m0(i_healthy,ii) ) ) + &
                                    (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_m0(i_sick,:,ii) + frac_l_f_vec*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) ) ) &
                                    )
        dist_S_m_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                    (1.0-pi_l_m(i_treated,ii))*dist_S_m0(i_treated,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_m0(i_treated,:,ii) + frac_l_f_vec*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) ) ) &
                                    )
        !Before changing discount factors or treatment status - young agents:
        dist_S_y_m_pr(i_healthy,ii) = (1.0-delta) * ( &
                                      (1.0-pi_l_y_m(i_healthy,ii))*dist_S_y_m0(i_healthy,ii)*( 1.0-pi_p_y_m(i_healthy,ii)-pi_u_y_m(i_healthy,ii) + pi_p_y_m(i_healthy,ii)*(1.0-upsilon_p_f*chi_m(ii)) + pi_u_y_m(i_healthy,ii)*(1.0-upsilon_u_f*chi_m(ii)) ) + &
                                      sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_y_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_y_m(i_healthy,ii)*dist_S_y_m0(i_healthy,ii) ) ) &
                                      ) + mu_m(ii)
        dist_S_y_m_pr(i_sick,ii)    = (1.0-delta) * ( &
                                      (1.0-pi_l_y_m(i_healthy,ii))*dist_S_y_m0(i_healthy,ii)*( pi_p_y_m(i_healthy,ii)*upsilon_p_f*chi_m(ii) + pi_u_y_m(i_healthy,ii)*upsilon_u_f*chi_m(ii) ) + &
                                      (1.0-alpha)*(1.0-pi_l_y_m(i_sick,ii))*dist_S_y_m0(i_sick,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii)*( dist_L_opp_y_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_y_m(i_healthy,ii)*dist_S_y_m0(i_healthy,ii) ) ) + &
                                      (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_m0(i_sick,:,ii) + frac_l_f_vec*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) ) ) &
                                      )
        dist_S_y_m_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                      (1.0-pi_l_y_m(i_treated,ii))*dist_S_y_m0(i_treated,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_m0(i_treated,:,ii) + frac_l_f_vec*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) ) ) &
                                      )
        !Changing the discount factors:
        dist_S_m_prpr(:,ii) = dist_S_m_pr(:,ii) + eta*dist_S_y_m_pr(:,ii)
        dist_S_y_m_prpr(:,ii) = (1.0-eta)*dist_S_y_m_pr(:,ii)
        !Changing treatment status:
        dist_S_m(i_healthy,ii) = dist_S_m_prpr(i_healthy,ii)
        dist_S_m(i_sick,ii)    = dist_S_m_prpr(i_sick,ii)*(1.0-q)
        dist_S_m(i_treated,ii) = dist_S_m_prpr(i_sick,ii)*q + dist_S_m_prpr(i_treated,ii)
        dist_S_y_m(i_healthy,ii) = dist_S_y_m_prpr(i_healthy,ii)
        dist_S_y_m(i_sick,ii)    = dist_S_y_m_prpr(i_sick,ii)*(1.0-q)
        dist_S_y_m(i_treated,ii) = dist_S_y_m_prpr(i_sick,ii)*q + dist_S_y_m_prpr(i_treated,ii)

        !MARRIED:
        !Before changing discount factors or treatment status - old agents:
        dist_L_opp_m_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_m(i_healthy,ii)*dist_S_m0(i_healthy,ii) )
        dist_L_opp_m_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_m*( dist_L_opp_m0(i_sick,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) )
        dist_L_opp_m_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_m0(i_sick,i_sick,ii) + frac_sick_f*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) )
        dist_L_opp_m_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_m0(i_sick,i_treated,ii) + frac_treated_f*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) )
        dist_L_opp_m_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_m_treat*( dist_L_opp_m0(i_treated,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) )
        dist_L_opp_m_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_m0(i_treated,i_sick,ii) + frac_sick_f*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) )
        dist_L_opp_m_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_m0(i_treated,i_treated,ii) + frac_treated_f*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) )
        !Before changing discount factors or treatment status - young agents:
        dist_L_opp_y_m_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_y_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_y_m(i_healthy,ii)*dist_S_y_m0(i_healthy,ii) )
        dist_L_opp_y_m_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_m*( dist_L_opp_y_m0(i_sick,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) )
        dist_L_opp_y_m_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_y_m0(i_sick,i_sick,ii) + frac_sick_f*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) )
        dist_L_opp_y_m_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_y_m0(i_sick,i_treated,ii) + frac_treated_f*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) )
        dist_L_opp_y_m_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_m_treat*( dist_L_opp_y_m0(i_treated,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) )
        dist_L_opp_y_m_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_y_m0(i_treated,i_sick,ii) + frac_sick_f*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) )
        dist_L_opp_y_m_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_y_m0(i_treated,i_treated,ii) + frac_treated_f*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) )
        !Changing the discount factors:
        dist_L_opp_m_prpr(:,:,ii) = dist_L_opp_m_pr(:,:,ii) + eta*dist_L_opp_y_m_pr(:,:,ii)
        dist_L_opp_y_m_prpr(:,:,ii) = (1.0-eta)*dist_L_opp_y_m_pr(:,:,ii)
        !Chaging treatment status - old agents:
        dist_L_opp_m(i_healthy,i_healthy,ii) = dist_L_opp_m_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_m(i_healthy,i_sick,ii)    = dist_L_opp_m_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_m(i_healthy,i_treated,ii) = dist_L_opp_m_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_m_prpr(i_healthy,i_treated,ii)
        dist_L_opp_m(i_sick,i_healthy,ii)    = dist_L_opp_m_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_m(i_sick,i_sick,ii)       = dist_L_opp_m_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_m(i_sick,i_treated,ii)    = dist_L_opp_m_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_m_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_m(i_treated,i_healthy,ii) = dist_L_opp_m_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_m_prpr(i_treated,i_healthy,ii)
        dist_L_opp_m(i_treated,i_sick,ii)    = dist_L_opp_m_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_m_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_m(i_treated,i_treated,ii) = dist_L_opp_m_prpr(i_treated,i_treated,ii) + dist_L_opp_m_prpr(i_sick,i_treated,ii)*q + dist_L_opp_m_prpr(i_treated,i_sick,ii)*q + dist_L_opp_m_prpr(i_sick,i_sick,ii)*q*q
        !Changing treatment status - young agents:
        dist_L_opp_y_m(i_healthy,i_healthy,ii) = dist_L_opp_y_m_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_y_m(i_healthy,i_sick,ii)    = dist_L_opp_y_m_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_y_m(i_healthy,i_treated,ii) = dist_L_opp_y_m_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_y_m_prpr(i_healthy,i_treated,ii)
        dist_L_opp_y_m(i_sick,i_healthy,ii)    = dist_L_opp_y_m_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_y_m(i_sick,i_sick,ii)       = dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_y_m(i_sick,i_treated,ii)    = dist_L_opp_y_m_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_y_m(i_treated,i_healthy,ii) = dist_L_opp_y_m_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_y_m_prpr(i_treated,i_healthy,ii)
        dist_L_opp_y_m(i_treated,i_sick,ii)    = dist_L_opp_y_m_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_y_m(i_treated,i_treated,ii) = dist_L_opp_y_m_prpr(i_treated,i_treated,ii) + dist_L_opp_y_m_prpr(i_sick,i_treated,ii)*q + dist_L_opp_y_m_prpr(i_treated,i_sick,ii)*q + dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*q*q
        !Consolidating across partner's type:
        dist_L_m(:,ii) = sum(dist_L_opp_m(:,:,ii),dim=2)
        dist_L_y_m(:,ii) = sum(dist_L_opp_y_m(:,:,ii),dim=2)

        !WITH SYMPTOMS:
        dist_A_m(:,ii) = (1.0-delta)*( (1.0-delta2)*dist_A_m0(:,ii) + (dist_S_m0(:,ii) + dist_S_y_m0(:,ii) + dist_L_m(:,ii) + dist_L_y_m(:,ii))*alpha_vec )

    enddo

    !Distributions for females:
    do ii=1,ntypes_f
        !SINGLES:
        !Before changing discount factors or treatment status - old agents:
        dist_S_f_pr(i_healthy,ii) = (1.0-delta) * ( &
                                    (1.0-pi_l_f(i_healthy,ii))*dist_S_f0(i_healthy,ii)*( 1.0-pi_p_f(i_healthy,ii)-pi_u_f(i_healthy,ii) + pi_p_f(i_healthy,ii)*(1.0-upsilon_p_m*chi_f(ii)) + pi_u_f(i_healthy,ii)*(1.0-upsilon_u_m*chi_f(ii)) ) + &
                                    sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_f(i_healthy,ii)*dist_S_f0(i_healthy,ii) ) ) &
                                    )
        dist_S_f_pr(i_sick,ii)    = (1.0-delta) * ( &
                                    (1.0-pi_l_f(i_healthy,ii))*dist_S_f0(i_healthy,ii)*( pi_p_f(i_healthy,ii)*upsilon_p_m*chi_f(ii) + pi_u_f(i_healthy,ii)*upsilon_u_m*chi_f(ii) ) + &
                                    (1.0-alpha)*(1.0-pi_l_f(i_sick,ii))*dist_S_f0(i_sick,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii)*( dist_L_opp_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_f(i_healthy,ii)*dist_S_f0(i_healthy,ii) ) ) + &
                                    (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_f0(i_sick,:,ii) + frac_l_m_vec*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) ) ) &
                                    )
        dist_S_f_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                    (1.0-pi_l_f(i_treated,ii))*dist_S_f0(i_treated,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_f0(i_treated,:,ii) + frac_l_m_vec*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) ) ) &
                                    )
        !Before changing discount factors or treatment status - young agents:
        dist_S_y_f_pr(i_healthy,ii) = (1.0-delta) * ( &
                                      (1.0-pi_l_y_f(i_healthy,ii))*dist_S_y_f0(i_healthy,ii)*( 1.0-pi_p_y_f(i_healthy,ii)-pi_u_y_f(i_healthy,ii) + pi_p_y_f(i_healthy,ii)*(1.0-upsilon_p_m*chi_f(ii)) + pi_u_y_f(i_healthy,ii)*(1.0-upsilon_u_m*chi_f(ii)) ) + &
                                      sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_y_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_y_f(i_healthy,ii)*dist_S_y_f0(i_healthy,ii) ) ) &
                                      ) + mu_f(ii)
        dist_S_y_f_pr(i_sick,ii)    = (1.0-delta) * ( &
                                      (1.0-pi_l_y_f(i_healthy,ii))*dist_S_y_f0(i_healthy,ii)*( pi_p_y_f(i_healthy,ii)*upsilon_p_m*chi_f(ii) + pi_u_y_f(i_healthy,ii)*upsilon_u_m*chi_f(ii) ) + &
                                      (1.0-alpha)*(1.0-pi_l_y_f(i_sick,ii))*dist_S_y_f0(i_sick,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii)*( dist_L_opp_y_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_y_f(i_healthy,ii)*dist_S_y_f0(i_healthy,ii) ) ) + &
                                      (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_f0(i_sick,:,ii) + frac_l_m_vec*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) ) ) &
                                      )
        dist_S_y_f_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                      (1.0-pi_l_y_f(i_treated,ii))*dist_S_y_f0(i_treated,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_f0(i_treated,:,ii) + frac_l_m_vec*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) ) ) &
                                      )
        !Changing the discount factors:
        dist_S_f_prpr(:,ii) = dist_S_f_pr(:,ii) + eta*dist_S_y_f_pr(:,ii)
        dist_S_y_f_prpr(:,ii) = (1.0-eta)*dist_S_y_f_pr(:,ii)
        !Changing treatment status:
        dist_S_f(i_healthy,ii) = dist_S_f_prpr(i_healthy,ii)
        dist_S_f(i_sick,ii)    = dist_S_f_prpr(i_sick,ii)*(1.0-q)
        dist_S_f(i_treated,ii) = dist_S_f_prpr(i_sick,ii)*q + dist_S_f_prpr(i_treated,ii)
        dist_S_y_f(i_healthy,ii) = dist_S_y_f_prpr(i_healthy,ii)
        dist_S_y_f(i_sick,ii)    = dist_S_y_f_prpr(i_sick,ii)*(1.0-q)
        dist_S_y_f(i_treated,ii) = dist_S_y_f_prpr(i_sick,ii)*q + dist_S_y_f_prpr(i_treated,ii)

        !MARRIED:
        !Before changing discount factors or treatment status - old agents:
        dist_L_opp_f_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_f(i_healthy,ii)*dist_S_f0(i_healthy,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_f_pr(i_sick,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_bycirc_f0(i_sick,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
            dist_L_opp_bycirc_f_pr(i_sick,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-(1.0-gammau_f)*chi_circ)*( dist_L_opp_bycirc_f0(i_sick,i_circ,ii) + frac_healthy_circ_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
            dist_L_opp_f_pr(i_sick,i_healthy,ii) = dist_L_opp_bycirc_f_pr(i_sick,i_nocirc,ii) + dist_L_opp_bycirc_f_pr(i_sick,i_circ,ii)
        else
            dist_L_opp_f_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_f0(i_sick,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
        endif
        dist_L_opp_f_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_f0(i_sick,i_sick,ii) + frac_sick_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
        dist_L_opp_f_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_f0(i_sick,i_treated,ii) + frac_treated_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_f_pr(i_treated,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_bycirc_f0(i_treated,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
            dist_L_opp_bycirc_f_pr(i_treated,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-(1.0-gammau_f_treat)*chi_circ)*( dist_L_opp_bycirc_f0(i_treated,i_circ,ii) + frac_healthy_circ_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
            dist_L_opp_f_pr(i_treated,i_healthy,ii) = dist_L_opp_bycirc_f_pr(i_treated,i_nocirc,ii) + dist_L_opp_bycirc_f_pr(i_treated,i_circ,ii)
        else
            dist_L_opp_f_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_f0(i_treated,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
        endif
        dist_L_opp_f_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_f0(i_treated,i_sick,ii) + frac_sick_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
        dist_L_opp_f_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_f0(i_treated,i_treated,ii) + frac_treated_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
        !Before changing discount factors or treatment status - young agents:
        dist_L_opp_y_f_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_y_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_y_f(i_healthy,ii)*dist_S_y_f0(i_healthy,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_y_f_pr(i_sick,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_bycirc_y_f0(i_sick,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
            dist_L_opp_bycirc_y_f_pr(i_sick,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-(1.0-gammau_f)*chi_circ)*( dist_L_opp_bycirc_y_f0(i_sick,i_circ,ii) + frac_healthy_circ_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
            dist_L_opp_y_f_pr(i_sick,i_healthy,ii) = dist_L_opp_bycirc_y_f_pr(i_sick,i_nocirc,ii) + dist_L_opp_bycirc_y_f_pr(i_sick,i_circ,ii)
        else
            dist_L_opp_y_f_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_y_f0(i_sick,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
        endif
        dist_L_opp_y_f_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_y_f0(i_sick,i_sick,ii) + frac_sick_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
        dist_L_opp_y_f_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_y_f0(i_sick,i_treated,ii) + frac_treated_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_y_f_pr(i_treated,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_bycirc_y_f0(i_treated,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
            dist_L_opp_bycirc_y_f_pr(i_treated,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-(1.0-gammau_f_treat)*chi_circ)*( dist_L_opp_bycirc_y_f0(i_treated,i_circ,ii) + frac_healthy_circ_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
            dist_L_opp_y_f_pr(i_treated,i_healthy,ii) = dist_L_opp_bycirc_y_f_pr(i_treated,i_nocirc,ii) + dist_L_opp_bycirc_y_f_pr(i_treated,i_circ,ii)
        else
            dist_L_opp_y_f_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_y_f0(i_treated,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
        endif
        dist_L_opp_y_f_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_y_f0(i_treated,i_sick,ii) + frac_sick_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
        dist_L_opp_y_f_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_y_f0(i_treated,i_treated,ii) + frac_treated_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
        !Changing the discount factors:
        dist_L_opp_f_prpr(:,:,ii) = dist_L_opp_f_pr(:,:,ii) + eta*dist_L_opp_y_f_pr(:,:,ii)
        dist_L_opp_y_f_prpr(:,:,ii) = (1.0-eta)*dist_L_opp_y_f_pr(:,:,ii)
        !Chaging treatment status - old agents:
        dist_L_opp_f(i_healthy,i_healthy,ii) = dist_L_opp_f_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_f(i_healthy,i_sick,ii)    = dist_L_opp_f_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_f(i_healthy,i_treated,ii) = dist_L_opp_f_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_f_prpr(i_healthy,i_treated,ii)
        dist_L_opp_f(i_sick,i_healthy,ii)    = dist_L_opp_f_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_f(i_sick,i_sick,ii)       = dist_L_opp_f_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_f(i_sick,i_treated,ii)    = dist_L_opp_f_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_f_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_f(i_treated,i_healthy,ii) = dist_L_opp_f_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_f_prpr(i_treated,i_healthy,ii)
        dist_L_opp_f(i_treated,i_sick,ii)    = dist_L_opp_f_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_f_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_f(i_treated,i_treated,ii) = dist_L_opp_f_prpr(i_treated,i_treated,ii) + dist_L_opp_f_prpr(i_sick,i_treated,ii)*q + dist_L_opp_f_prpr(i_treated,i_sick,ii)*q + dist_L_opp_f_prpr(i_sick,i_sick,ii)*q*q
        !Changing treatment status - young agents:
        dist_L_opp_y_f(i_healthy,i_healthy,ii) = dist_L_opp_y_f_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_y_f(i_healthy,i_sick,ii)    = dist_L_opp_y_f_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_y_f(i_healthy,i_treated,ii) = dist_L_opp_y_f_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_y_f_prpr(i_healthy,i_treated,ii)
        dist_L_opp_y_f(i_sick,i_healthy,ii)    = dist_L_opp_y_f_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_y_f(i_sick,i_sick,ii)       = dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_y_f(i_sick,i_treated,ii)    = dist_L_opp_y_f_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_y_f(i_treated,i_healthy,ii) = dist_L_opp_y_f_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_y_f_prpr(i_treated,i_healthy,ii)
        dist_L_opp_y_f(i_treated,i_sick,ii)    = dist_L_opp_y_f_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_y_f(i_treated,i_treated,ii) = dist_L_opp_y_f_prpr(i_treated,i_treated,ii) + dist_L_opp_y_f_prpr(i_sick,i_treated,ii)*q + dist_L_opp_y_f_prpr(i_treated,i_sick,ii)*q + dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*q*q
        !Change discount factors and treatment status for distributions by circumcision:
        if (do_circ==1) then
            !Change the discount factors:
            dist_L_opp_bycirc_f_prpr(:,:,ii) = dist_L_opp_bycirc_f_pr(:,:,ii) + eta*dist_L_opp_bycirc_y_f_pr(:,:,ii)
            dist_L_opp_bycirc_y_f_prpr(:,:,ii) = (1.0-eta)*dist_L_opp_bycirc_y_f_pr(:,:,ii)
            !Change the treatment status:
            dist_L_opp_bycirc_f(i_sick,:,ii) = dist_L_opp_bycirc_f_prpr(i_sick,:,ii)*(1.0-q)
            dist_L_opp_bycirc_f(i_treated,:,ii) = dist_L_opp_bycirc_f_prpr(i_sick,:,ii)*q + dist_L_opp_bycirc_f_prpr(i_treated,:,ii)
            dist_L_opp_bycirc_y_f(i_sick,:,ii) = dist_L_opp_bycirc_y_f_prpr(i_sick,:,ii)*(1.0-q)
            dist_L_opp_bycirc_y_f(i_treated,:,ii) = dist_L_opp_bycirc_y_f_prpr(i_sick,:,ii)*q + dist_L_opp_bycirc_y_f_prpr(i_treated,:,ii)
        endif
        !Consolidating across partner's type:
        dist_L_f(:,ii) = sum(dist_L_opp_f(:,:,ii),dim=2)
        dist_L_y_f(:,ii) = sum(dist_L_opp_y_f(:,:,ii),dim=2)

        !WITH SYMPTOMS:
        dist_A_f(:,ii) = (1.0-delta)*( (1.0-delta2)*dist_A_f0(:,ii) + (dist_S_f0(:,ii) + dist_S_y_f0(:,ii) + dist_L_f(:,ii) + dist_L_y_f(:,ii))*alpha_vec )

    enddo

    !=======================================================
    ! AGGREGATES
    !=======================================================

    !Who goes to each market? - males:
    measure_p_m = sum((1.0-pi_l_m)*pi_p_m*dist_S_m,dim=2)
    measure_u_m = sum((1.0-pi_l_m)*pi_u_m*dist_S_m,dim=2)
    measure_a_m = sum((1.0-pi_l_m)*(1.0-pi_p_m-pi_u_m)*dist_S_m,dim=2)

    measure_l_m = sum(pi_l_m*dist_S_m,dim=2)
    measure_p_y_m = sum((1.0-pi_l_y_m)*pi_p_y_m*dist_S_y_m,dim=2)
    measure_u_y_m = sum((1.0-pi_l_y_m)*pi_u_y_m*dist_S_y_m,dim=2)
    measure_a_y_m = sum((1.0-pi_l_y_m)*(1.0-pi_p_y_m-pi_u_y_m)*dist_S_y_m,dim=2)
    measure_l_y_m = sum(pi_l_y_m*dist_S_y_m,dim=2)

    !Who goes to each market? - females:
    measure_p_f = sum((1.0-pi_l_f)*pi_p_f*dist_S_f,dim=2)
    measure_u_f = sum((1.0-pi_l_f)*pi_u_f*dist_S_f,dim=2)
    measure_a_f = sum((1.0-pi_l_f)*(1.0-pi_p_f-pi_u_f)*dist_S_f,dim=2)
    measure_l_f = sum(pi_l_f*dist_S_f,dim=2)
    measure_p_y_f = sum((1.0-pi_l_y_f)*pi_p_y_f*dist_S_y_f,dim=2)
    measure_u_y_f = sum((1.0-pi_l_y_f)*pi_u_y_f*dist_S_y_f,dim=2)
    measure_a_y_f = sum((1.0-pi_l_y_f)*(1.0-pi_p_y_f-pi_u_y_f)*dist_S_y_f,dim=2)
    measure_l_y_f = sum(pi_l_y_f*dist_S_y_f,dim=2)

    !Update infectiousness and non-prevalence rates:
    if (sum(measure_p_m+measure_p_y_m)>0.0d0) then
        upsilon_p_m = sum( (1.0-gamma_vec(:,i_male,i_protected))*measure_p_m + (1.0-gamma_vec(:,i_male,i_protected))*measure_p_y_m ) / sum(measure_p_m+measure_p_y_m)
        phi_p_m = (measure_p_m(i_healthy)+measure_p_y_m(i_healthy)) / sum(measure_p_m+measure_p_y_m)
    endif
    if (sum(measure_u_m+measure_u_y_m)>0.0d0) then
        upsilon_u_m = sum( (1.0-gamma_vec(:,i_male,i_unprotected))*measure_u_m + (1.0-gamma_vec(:,i_male,i_unprotected))*measure_u_y_m ) / sum(measure_u_m+measure_u_y_m)
        phi_u_m = (measure_u_m(i_healthy)+measure_u_y_m(i_healthy)) / sum(measure_u_m+measure_u_y_m)
    endif
    if (sum(measure_p_f+measure_p_y_f)>0.0d0) then
        upsilon_p_f = sum( (1.0-gamma_vec(:,i_female,i_protected))*measure_p_f + (1.0-gamma_vec(:,i_female,i_protected))*measure_p_y_f ) / sum(measure_p_f+measure_p_y_f)
        phi_p_f = (measure_p_f(i_healthy)+measure_p_y_f(i_healthy)) / sum(measure_p_f+measure_p_y_f)
    endif
    if (sum(measure_u_f+measure_u_y_f)>0.0d0) then
        upsilon_u_f = sum( (1.0-gamma_vec(:,i_female,i_unprotected))*measure_u_f + (1.0-gamma_vec(:,i_female,i_unprotected))*measure_u_y_f ) / sum(measure_u_f+measure_u_y_f)
        phi_u_f = (measure_u_f(i_healthy)+measure_u_y_f(i_healthy)) / sum(measure_u_f+measure_u_y_f)
    endif
    if (sum(measure_l_m+measure_l_y_m)>0.0d0) then
        frac_sick_m    = (measure_l_m(i_sick)+measure_l_y_m(i_sick)) / sum(measure_l_m+measure_l_y_m)
        frac_treated_m = (measure_l_m(i_treated)+measure_l_y_m(i_treated)) / sum(measure_l_m+measure_l_y_m)
        if (do_circ==1) then
            call compute_frac_healthy_longterm_bycirc(pi_l_m,pi_l_y_m,dist_S_m,dist_S_y_m,sum(measure_l_m+measure_l_y_m),frac_healthy_nocirc_m,frac_healthy_circ_m)
        endif
        phi_l_m = (measure_l_m(i_healthy)+measure_l_y_m(i_healthy)) / sum(measure_l_m+measure_l_y_m)
    endif
    if (sum(measure_l_f+measure_l_y_f)>0.0d0) then
        frac_sick_f    = (measure_l_f(i_sick)+measure_l_y_f(i_sick)) / sum(measure_l_f+measure_l_y_f)
        frac_treated_f = (measure_l_f(i_treated)+measure_l_y_f(i_treated)) / sum(measure_l_f+measure_l_y_f)
        phi_l_f = (measure_l_f(i_healthy)+measure_l_y_f(i_healthy)) / sum(measure_l_f+measure_l_y_f)
    endif
    if (sum(measure_a_m+measure_a_y_m)>0.0d0) then
        phi_n_m = (measure_a_m(i_healthy)+measure_a_y_m(i_healthy)) / sum(measure_a_m+measure_a_y_m)
    endif
    if (sum(measure_a_f+measure_a_y_f)>0.0d0) then
        phi_n_f = (measure_a_f(i_healthy)+measure_a_y_f(i_healthy)) / sum(measure_a_f+measure_a_y_f)
    endif

    !Vector with fraction of sick/treated in the long term market - males:
    frac_l_m_vec(i_healthy) = 1.0 - frac_sick_m - frac_treated_m
    frac_l_m_vec(i_sick)    = frac_sick_m
    frac_l_m_vec(i_treated) = frac_treated_m
    frac_l_healthy_bycirc_m(i_nocirc) = frac_healthy_nocirc_m
    frac_l_healthy_bycirc_m(i_circ)   = frac_healthy_circ_m

    !Vector with fraction of sick/treated in the long term market - females:
    frac_l_f_vec(i_healthy) = 1.0 - frac_sick_f - frac_treated_f
    frac_l_f_vec(i_sick)    = frac_sick_f
    frac_l_f_vec(i_treated) = frac_treated_f

    !Save all prevalences in one vector:
    prev1(1) = phi_p_m
    prev1(2) = phi_u_m
    prev1(3) = phi_l_m
    prev1(4) = phi_p_f
    prev1(5) = phi_u_f
    prev1(6) = phi_l_f
    prev1(7) = phi_n_m
    prev1(8) = phi_n_f

    !=======================================================
    ! CLOSING THIS ITERATION
    !=======================================================

    !Compute the difference between this iteration and the last:
    norm_vec = 0.0d0
    norm_vec(1)  = maxval(abs(dist_S_m - dist_S_m0))
    norm_vec(2)  = maxval(abs(dist_S_y_m - dist_S_y_m0))
    norm_vec(3)  = maxval(abs(dist_S_f - dist_S_f0))
    norm_vec(4)  = maxval(abs(dist_S_y_f - dist_S_y_f0))
    norm_vec(5)  = maxval(abs(dist_L_m - dist_L_m0))
    norm_vec(6)  = maxval(abs(dist_L_y_m - dist_L_y_m0))
    norm_vec(7)  = maxval(abs(dist_L_opp_m - dist_L_opp_m0))
    norm_vec(8)  = maxval(abs(dist_L_opp_y_m - dist_L_opp_y_m0))
    norm_vec(9)  = maxval(abs(dist_L_f - dist_L_f0))
    norm_vec(10) = maxval(abs(dist_L_y_f - dist_L_y_f0))
    norm_vec(11) = maxval(abs(dist_L_opp_f - dist_L_opp_f0))
    norm_vec(12) = maxval(abs(dist_L_opp_y_f - dist_L_opp_y_f0))
    norm_vec(13) = maxval(abs(dist_A_m - dist_A_m0))
    norm_vec(14) = maxval(abs(dist_A_f - dist_A_f0))
!    norm_vec(15) = maxval(abs(v_l_m - v_l_m0))
!    norm_vec(16) = maxval(abs(v_l_y_m - v_l_y_m0))
!    norm_vec(17) = maxval(abs(v_l_f - v_l_f0))
!    norm_vec(18) = maxval(abs(v_l_y_f - v_l_y_f0))
!    norm_vec(19) = maxval(abs(vtilde_l_opp_m - vtilde_l_opp_m0))
!    norm_vec(20) = maxval(abs(vtilde_l_opp_y_m - vtilde_l_opp_y_m0))
!    norm_vec(21) = maxval(abs(vtilde_l_opp_f - vtilde_l_opp_f0))
!    norm_vec(22) = maxval(abs(vtilde_l_opp_y_f - vtilde_l_opp_y_f0))
    norm = maxval(norm_vec)

    !Check for convergence:
    if (norm < tol) exit

enddo

!How many iterations?
print *, "number of iterations: ", iter

!Set the convergence flag for the inner loop:
if (iter>=max_iter-1) then
	conv_flags(1) = 0
else
	conv_flags(1) = 1
endif

!EXCESS DEMANDS:
!constant for normalization -> we're taking the steady state value of the
!population size disregarding deaths due to AIDS. We're also assuming that
!the singles account for about 10% of the entire population. Then, we use
!this number of singles to normalize the excess demands.
normalize = (1.0/10.0)*sum(mu_m)/delta

!PROTECTED
ed_p = sum(measure_p_m+measure_p_y_m) - sum(measure_p_f+measure_p_y_f)
ed_p = ed_p/normalize

!UNPROTECTED
ed_u = sum(measure_u_m+measure_u_y_m) - sum(measure_u_f+measure_u_y_f)
ed_u = ed_u/normalize

!LONG TERM
ed_l = sum(measure_l_m+measure_l_y_m) - sum(measure_l_f+measure_l_y_f)
ed_l = ed_l/normalize

!ALL MARKETS INTO A VECTOR:
excess_vector(1) = ed_p
excess_vector(2) = ed_u
excess_vector(3) = ed_l

if (iter>=max_iter-1) then
	excess_vector = 100.0d0
endif

!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------

!In the following, we write out the distributions, policy functions for search effort
!files that will be read by Matlab in order to compute the desired model statistics.

if (write_more_files==1) then

    !DISTRIBUTIONS:
    !Males:
    open(11,file='dist_S_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_S_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_S_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_S_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_L_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_L_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_A_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_A_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_m.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_m(jj,kk,ii)
            enddo
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_y_m(jj,kk,ii)
            enddo
        enddo
    enddo
    !Females:
    open(11,file='dist_S_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_S_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_S_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_S_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_L_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_L_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_A_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_A_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_f.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_f(jj,kk,ii)
            enddo
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_y_f.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_y_f(jj,kk,ii)
            enddo
        enddo
    enddo

    !SEARCH EFFORT:
    !Males:
    open(11,file='pi_p_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_p_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_p_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_p_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_u_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_u_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_l_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_l_y_m(jj,ii)
        enddo
    enddo
    close(11)
    !Females:
    open(11,file='pi_p_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_p_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_p_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_p_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_u_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_u_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_l_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_l_y_f(jj,ii)
        enddo
    enddo
    close(11)

    !FRACTION OF PEOPLE IN THE LONG TERM MARKET:
    !Males:
    open(11,file='frac_l_m.txt',status='replace')
    do jj=1,n_status
        write(11,*) frac_l_m_vec(jj)
    enddo
    close(11)
    !Females:
    open(11,file='frac_l_f.txt',status='replace')
    do jj=1,n_status
        write(11,*) frac_l_f_vec(jj)
    enddo
    close(11)

    !INFECTIOUSNESS IN THE SHORT TERM MARKETS:
    upsilon_all(1) = upsilon_p_m
    upsilon_all(2) = upsilon_u_m
    upsilon_all(3) = upsilon_p_f
    upsilon_all(4) = upsilon_u_f
    open(11,file='upsilon_all.txt',status='replace')
    do jj=1,4
        write(11,*) upsilon_all(jj)
    enddo
    close(11)

endif !write_more_files

end subroutine epid
