%This m-file attempts to estimate the parameters for the AIDS project. This
%is based on a minimization routine applied to the f_objective.m function

global weight_matrix use_perc_deviations punish_nonmc

%Do you want to run the minimization routine?
run_minimization = 0;

%Do you want to read the initial guess from a file?
read_initialguess = 1;

%Set flags for the objective function:
use_perc_deviations = 0;
punish_nonmc = 0;

%Set the initial guess:
if read_initialguess==1
    %load guess from file:
    load myinitialguess.txt
    guess = myinitialguess;
    clear myinitialguess
else
    %set the guess:
    p_guess = 2;
    u_guess = 8.5;
    l_guess = -6.5;
    beta_max_guess = 0.995;
    beta_min_guess = 0.8955;
    iota_change_guess = 0.90080702125;
    A_guess = 5;
    eta_guess = 0.06;
    omega_s_guess = 0.25;
    omega_l_guess = 32;
    kappa_guess = 0.2;
    %compile guess on a vector:
    guess = zeros(11,1);
    guess(1) = p_guess;
    guess(2) = u_guess;
    guess(3) = l_guess;
    guess(4) = beta_max_guess;
    guess(5) = beta_min_guess;
    guess(6) = iota_change_guess;
    guess(7) = A_guess;
    guess(8) = eta_guess;
    guess(9) = omega_s_guess;
    guess(10) = omega_l_guess;
    guess(11) = kappa_guess;
    
end

%Weight matrix:
weight_matrix = zeros(21,1);
weight_matrix(1) = 2; %HIV prevalence - males (all)
weight_matrix(2) = 2; %HIV prevalence - females (all)
weight_matrix(3) = 0; %HIV prevalence - males (20 years old)
weight_matrix(4) = 0; %HIV prevalence - males (30 years old)
weight_matrix(5) = 0; %HIV prevalence - males (40 years old)
weight_matrix(6) = 0; %HIV prevalence - females (20 years old)
weight_matrix(7) = 0; %HIV prevalence - females (30 years old)
weight_matrix(8) = 0; %HIV prevalence - females (40 years old)
weight_matrix(9) = 1; %Fraction of sex that is casual
weight_matrix(10)= 1; %Condom usage
weight_matrix(11)= 1; %Fraction of singles that had casual sex last year
weight_matrix(12)= 1; %Fraction of singles
weight_matrix(13)= 0; %Ever married at age 20 - males
weight_matrix(14)= 0; %Ever married at age 20 - females
weight_matrix(15)= 0; %Ever married at age 22 - males
weight_matrix(16)= 0; %Ever married at age 22 - females
weight_matrix(17)= 1; %Ever married at age 25 - males
weight_matrix(18)= 1; %Ever married at age 25 - females
weight_matrix(19)= 1; %Ever married at age 50 - males
weight_matrix(20)= 1; %Ever married at age 50 - females
weight_matrix(21)= 1; %Percentage of deaths due to HIV/AIDS

%Call the minimization routine:
if run_minimization==1
    options = optimset('MaxIter',100);
    [thesolution, thevalue] = patternsearch('f_objective',guess,options);
    %Write optimal parameters to a file:
    small = .00001;
    thesolution(1) = max(small,thesolution(1));                           %p
    thesolution(2) = max(small,thesolution(2));                           %u
    thesolution(3) = thesolution(3);                                      %l
    thesolution(4) = max(small,min(1-small,thesolution(4)));              %beta_max
    thesolution(5) = max(small,min(thesolution(4)-small,thesolution(5))); %beta_min
    thesolution(6) = max(small,min(1-small,thesolution(6)));              %iota_change
    thesolution(7) = max(small,thesolution(7));                           %A
    thesolution(8) = max(small,min(1-small,thesolution(8)));              %eta
    thesolution(9) = max(small,thesolution(9));                           %omega_s
    thesolution(10)= max(small,thesolution(10));                          %omega_l
    thesolution(11)= max(small,thesolution(11));                          %kappa
    dlmwrite('parameters_optimal.txt',thesolution)
else
    thesolution = guess;
end

%Run once with the optimizer:
f_objective(thesolution);




