function out = f_objective(guess)
%This file computes the sum of squared errors between the data targets and
%the model moments to be used in the estimation of the parameters.

global weight_matrix use_perc_deviations punish_nonmc

%Some useful stuff:
small = .00001;

%Read the guess:
%%%%%%%%%%%%%%%%
p_guess = max(small,guess(1));
u_guess = max(small,guess(2));
l_guess = guess(3);
beta_max_guess = max(small,min(1-small,guess(4)));
beta_min_guess = max(small,min(beta_max_guess-small,guess(5)));
iota_change_guess = max(small,min(1-small,guess(6)));
A_guess = max(small,guess(7));
eta_guess = max(small,min(1-small,guess(8)));
omega_s_guess = max(small,guess(9));
omega_l_guess = max(small,guess(10));
kappa_guess = max(small,guess(11));

%Replace selected entries in the parameters_all.txt file:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%load parameters:
load parameters_all.txt
param_temp = parameters_all;
clear parameters_all

%replace joys for different relationships:
param_temp(26:27) = p_guess;
param_temp(28:29) = u_guess;
param_temp(30:31) = l_guess;

%replace search cost function parameters:
param_temp(33) = omega_s_guess;
param_temp(36) = omega_s_guess;
param_temp(39) = omega_l_guess;
param_temp(34) = kappa_guess;
param_temp(37) = kappa_guess;
param_temp(40) = kappa_guess;

%replace probability of switching discount factors:
param_temp(32) = eta_guess;

%replace value of living with symptoms:
param_temp(42) = A_guess;

%replace variables related to discount factors:
param_temp(46) = beta_max_guess;
param_temp(47) = beta_min_guess;
param_temp(48) = iota_change_guess;

%write parameters to file:
dlmwrite('parameters_all.txt',param_temp,'precision',10)

%Call main file:
%%%%%%%%%%%%%%%%

mainp

%Data targets:
%%%%%%%%%%%%%%

data_targets = zeros(21,1);

data_targets(1) = 0.10; %HIV prevalence - males (all)
data_targets(2) = 0.13; %HIV prevalence - females (all)
data_targets(3) = 0.0288; %HIV prevalence - males (20 years old)
data_targets(4) = 0.1605; %HIV prevalence - males (30 years old)
data_targets(5) = 0.2096; %HIV prevalence - males (40 years old)
data_targets(6) = 0.1187; %HIV prevalence - females (20 years old)
data_targets(7) = 0.1822; %HIV prevalence - females (30 years old)
data_targets(8) = 0.2067; %HIV prevalence - females (40 years old)
data_targets(9) = 0.18; %Fraction of sex that is casual
data_targets(10)= 0.39; %Condom usage
data_targets(11)= 0.37; %Fraction of singles that had casual sex last year
data_targets(12)= 0.33; %Fraction of singles
data_targets(13)= 0.198; %Ever married at age 20 - males
data_targets(14)= 0.787; %Ever married at age 20 - females
data_targets(15)= 0.58; %Ever married at age 22 - males
data_targets(16)= 0.90; %Ever married at age 22 - females
data_targets(17)= 0.657; %Ever married at age 25 - males
data_targets(18)= 0.955; %Ever married at age 25 - females
data_targets(19)= 1.00; %Ever married at age 50 - males
data_targets(20)= 1.00; %Ever married at age 50 - females
data_targets(21)= 0.29; %Percentage of deaths due to HIV/AIDS

%Model moments:
%%%%%%%%%%%%%%%

model_moments = zeros(21,1);

model_moments(1) = percentage_aids_m;
model_moments(2) = percentage_aids_f;
model_moments(3) = hiv_byage_m(20);
model_moments(4) = hiv_byage_m(60);
model_moments(5) = hiv_byage_m(100);
model_moments(6) = hiv_byage_f(20);
model_moments(7) = hiv_byage_f(60);
model_moments(8) = hiv_byage_f(100);
model_moments(9) = prob_s;
model_moments(10)= prob_condom;
model_moments(11)= prob_shortsex2;
model_moments(12)= fraction_single;
model_moments(13)= evermarried_m(20);
model_moments(14)= evermarried_f(20);
model_moments(15)= evermarried_m(28);
model_moments(16)= evermarried_f(28);
model_moments(17)= evermarried_m(40);
model_moments(18)= evermarried_f(40);
model_moments(19)= evermarried_m(140);
model_moments(20)= evermarried_f(140);
model_moments(21)= 1-frac_natdeath;

%Function output (sum of squared errors):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Basic:
distance = sqrt(weight_matrix).*(data_targets-model_moments);

%If using percentage deviations:
if use_perc_deviations==1
    distance = distance./data_targets;
end

%If punish non-market clearing runs:
if punish_nonmc==1 && opt_flag~=1
    distance = distance + 5;
end

%Final value for the function:
out = sum((distance).^2);











