%This m-file fits polynomials for the search effort functions (for each
%type)

%A SIMPLE MESSAGE
disp('---------------------------------------------')
disp('Computing the solutions for the search effort')
disp('Please wait...')

%PRELIMINARIES
top = 5*max([abs(p_m-u_m); abs(p_f-u_f)]);
mygrid = linspace(.0000000001,top,ngrid-10)'; %grid for unconstrained decision
mygrid_con = linspace(-top,top,ngrid-20)'; %grid for constrained decision

ahelp = zeros(ntypes_m+ntypes_f,1);
for i=1:ntypes_m
    mytype = i;
    ahelp(i) = (1/(1-beta_m(mytype)))*log(2*y_m)+u_m;
end
for i=ntypes_m+1:ntypes_m+ntypes_f
    mytype = i-ntypes_m;
    ahelp(i) = (1/(1-beta_f(mytype)))*log(2*y_f)+u_f;
end
top = max(ahelp);
mygrid = [mygrid; linspace(mygrid(ngrid-10)+10,top,10)'];
mygrid_con = [linspace(-top,mygrid_con(1)-10,10)'; mygrid_con; linspace(mygrid(ngrid-10)+10,top,10)'];

%PRE-ALLOCATING MEMORY FOR THE SEARCH EFFORT DECISIONS
search_p = zeros(ngrid,1);
search_u = zeros(ngrid,1);
search_l = zeros(ngrid,1);
search_ucon = zeros(ngrid,1);

%FINDING THE ZERO OF THE FIRST ORDER CONDITIONS:
for i=1:ngrid
    search_p(i) = bisection(mygrid(i),omega_p,kappa_p,b_p); %solving protected mkt - unconstrained
    search_u(i) = bisection(mygrid(i),omega_u,kappa_u,b_u); %solving unprotected mkt - unconstrained
    search_l(i) = bisection(mygrid(i),omega_l,kappa_l,b_l); %solving long term mkt
    if b_p + b_u > 1
        search_ucon(i) = bisection_con(mygrid_con(i),omega_p,omega_u,kappa_p,kappa_u,b_p,b_u); %constrained - short term
    end
end

%Writing these search efforts to a file:
fid = fopen( 'search_p.txt', 'w' );
fprintf(fid, '%18.10f\n', search_p);
fclose(fid);
fid = fopen( 'search_u.txt', 'w' );
fprintf(fid, '%18.10f\n', search_u);
fclose(fid);
fid = fopen( 'search_l.txt', 'w' );
fprintf(fid, '%18.10f\n', search_l);
fclose(fid);
fid = fopen( 'search_ucon.txt', 'w' );
fprintf(fid, '%18.10f\n', search_ucon);
fclose(fid);

%Writing the grids to a file:
fid = fopen( 'mygrid.txt', 'w' );
fprintf(fid, '%18.10f\n', mygrid);
fclose(fid);
fid = fopen( 'mygrid_con.txt', 'w' );
fprintf(fid, '%18.10f\n', mygrid_con);
fclose(fid);


%ANOTHER SIMPLE MESSAGE
disp('---------------------------------------------')
disp('DONE!!!')
disp('---------------------------------------------')
