subroutine longterm_f(mytype,mygender,mystatus,ntypes,income,joy,frac_healthy_nocirc,frac_healthy_circ,frac_sick,frac_treated,v_l,vtilde_opp_last,vtilde_opp_bycirc_last,vtilde,vtilde_opp,vtilde_opp_bycirc)
!This function computes the utility from being matched in the long term market for old agents
use mod_param
implicit none

!Define dummy variables:
integer     , intent(in)  :: mytype
integer     , intent(in)  :: mygender
integer     , intent(in)  :: mystatus
integer     , intent(in)  :: ntypes
real(mykind), intent(in)  :: income
real(mykind), intent(in)  :: joy
real(mykind), intent(in)  :: frac_healthy_nocirc
real(mykind), intent(in)  :: frac_healthy_circ
real(mykind), intent(in)  :: frac_sick
real(mykind), intent(in)  :: frac_treated
real(mykind), intent(in)  :: v_l(n_status,ntypes)
real(mykind), intent(in)  :: vtilde_opp_last(n_status,n_status,ntypes)
real(mykind), intent(in)  :: vtilde_opp_bycirc_last(n_status,n_circ,ntypes)
real(mykind), intent(out) :: vtilde
real(mykind), intent(out) :: vtilde_opp(n_status)
real(mykind), intent(out) :: vtilde_opp_bycirc(n_circ)

!Define temporary value functions:
real(mykind) :: vtilde_opp_healthy_nocirc
real(mykind) :: vtilde_opp_healthy_circ
real(mykind) :: vtilde_opp_sick
real(mykind) :: vtilde_opp_treated

!Define parameters for this problem:
real(mykind) :: beta, A, chi
real(mykind) :: gamma_mine, gamma_opp_sick, gamma_opp_treat
real(mykind) :: alpha_mine
real(mykind) :: q_mine
integer :: other_gender

!Define other variables:
real(mykind) :: util

!------------------------------------------------------------

!Get variables:
if (mygender==i_male) then
    beta = beta_m(mytype)
    A = A_m(mytype)
    chi = chi_m(mytype)
    other_gender = i_female
elseif (mygender==i_female) then
    beta = beta_f(mytype)
    A = A_f(mytype)
    chi = chi_f(mytype)
    other_gender = i_male
else
    print *, 'wrong gender in longterm'
    read *, beta
endif

!Parameters for non-transmission rates:
gamma_mine      = gamma_vec(mystatus,mygender,i_unprotected)
gamma_opp_sick  = gamma_vec(i_sick,other_gender,i_unprotected)
gamma_opp_treat = gamma_vec(i_treated,other_gender,i_unprotected)

!Probability that I develop symptoms next period:
alpha_mine = alpha_vec(mystatus)

!Probability that I get treatment next period:
q_mine = q_vec(mystatus)

!Utility from consumption:
call utility(income,util)

!Check whether this person is healthy or not. Then, compute his value conditional on knowing the partner:
if (mystatus==i_healthy) then
    print *, 'we should not be here!!!!'
    print *, 'longterm_f with healthy female'
    read *, beta
else
    !Partner is healthy - not circumcised:
    vtilde_opp_healthy_nocirc = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_mine)*gamma_mine*beta*(q_mine*vtilde_opp_bycirc_last(i_treated,i_nocirc,mytype)+(1.0-q_mine)*vtilde_opp_bycirc_last(i_sick,i_nocirc,mytype)) + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_mine)*(1.0-gamma_mine)*beta*(q_mine*q*vtilde_opp_last(i_treated,i_treated,mytype)+q_mine*(1.0-q)*vtilde_opp_last(i_treated,i_sick,mytype)+(1.0-q_mine)*q*vtilde_opp_last(i_sick,i_treated,mytype)+(1.0-q_mine)*(1.0-q)*vtilde_opp_last(i_sick,i_sick,mytype)) + &
                         (1.0-(1.0-epsilon)*(1.0-delta))*(1.0-alpha_mine)*beta*(q_mine*v_l(i_treated,mytype)+(1.0-q_mine)*v_l(i_sick,mytype)) + &
                         alpha_mine*beta*A
    !Partner is healthy - circumcised:
    vtilde_opp_healthy_circ = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_mine)*(1.0-(1.0-gamma_mine)*chi_circ)*beta*(q_mine*vtilde_opp_bycirc_last(i_treated,i_circ,mytype)+(1.0-q_mine)*vtilde_opp_bycirc_last(i_sick,i_circ,mytype)) + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_mine)*(1.0-gamma_mine)*chi_circ*beta*(q_mine*q*vtilde_opp_last(i_treated,i_treated,mytype)+q_mine*(1.0-q)*vtilde_opp_last(i_treated,i_sick,mytype)+(1.0-q_mine)*q*vtilde_opp_last(i_sick,i_treated,mytype)+(1.0-q_mine)*(1.0-q)*vtilde_opp_last(i_sick,i_sick,mytype)) + &
                         (1.0-(1.0-epsilon)*(1.0-delta))*(1.0-alpha_mine)*beta*(q_mine*v_l(i_treated,mytype)+(1.0-q_mine)*v_l(i_sick,mytype)) + &
                         alpha_mine*beta*A
    !Partner is sick:
    vtilde_opp_sick    = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha)*(1.0-alpha_mine)*beta*(q_mine*q*vtilde_opp_last(i_treated,i_treated,mytype)+q_mine*(1.0-q)*vtilde_opp_last(i_treated,i_sick,mytype)+(1.0-q_mine)*q*vtilde_opp_last(i_sick,i_treated,mytype)+(1.0-q_mine)*(1.0-q)*vtilde_opp_last(i_sick,i_sick,mytype)) + &
                         (1.0-(1.0-epsilon)*(1.0-delta)*(1.0-alpha))*(1.0-alpha_mine)*beta*(q_mine*v_l(i_treated,mytype)+(1.0-q_mine)*v_l(i_sick,mytype)) + &
                         alpha_mine*beta*A
    !Partner is treated
    vtilde_opp_treated = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat)*(1.0-alpha_mine)*beta*(q_mine*vtilde_opp_last(i_treated,i_treated,mytype)+(1.0-q_mine)*vtilde_opp_last(i_sick,i_treated,mytype)) + &
                         (1.0-(1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat))*(1.0-alpha_mine)*beta*(q_mine*v_l(i_treated,mytype)+(1.0-q_mine)*v_l(i_sick,mytype)) + &
                         alpha_mine*beta*A
endif

!Consolidate this person's matched value function with each type of partner:
vtilde_opp_bycirc(i_nocirc) = vtilde_opp_healthy_nocirc
vtilde_opp_bycirc(i_circ) = vtilde_opp_healthy_circ
vtilde_opp(i_healthy) = huge_negative !this should never be used, see above
vtilde_opp(i_sick)    = vtilde_opp_sick
vtilde_opp(i_treated) = vtilde_opp_treated

!Compute this person's ex ante value (without knowing the partner's type):
vtilde = frac_healthy_nocirc*vtilde_opp_healthy_nocirc + frac_healthy_circ*vtilde_opp_healthy_circ + frac_sick*vtilde_opp_sick + frac_treated*vtilde_opp_treated

end subroutine longterm_f
