subroutine longterm_y(mytype,mygender,mystatus,ntypes,income,joy,frac_sick,frac_treated,v_l,v_l_y,vtilde_opp_last,vtilde_opp_y_last,vtilde,vtilde_opp)
!This function computes the utility from being matched in the long term market for young agents
use mod_param
implicit none

!Define dummy variables:
integer     , intent(in)  :: mytype
integer     , intent(in)  :: mygender
integer     , intent(in)  :: mystatus
integer     , intent(in)  :: ntypes
real(mykind), intent(in)  :: income
real(mykind), intent(in)  :: joy
real(mykind), intent(in)  :: frac_sick
real(mykind), intent(in)  :: frac_treated
real(mykind), intent(in)  :: v_l(n_status,ntypes)
real(mykind), intent(in)  :: v_l_y(n_status,ntypes)
real(mykind), intent(in)  :: vtilde_opp_last(n_status,n_status,ntypes)
real(mykind), intent(in)  :: vtilde_opp_y_last(n_status,n_status,ntypes)
real(mykind), intent(out) :: vtilde
real(mykind), intent(out) :: vtilde_opp(n_status)

!Define temporary value functions:
real(mykind) :: vtilde_opp_healthy
real(mykind) :: vtilde_opp_sick
real(mykind) :: vtilde_opp_treated

!Define parameters for this problem:
real(mykind) :: iota, A, chi
real(mykind) :: gamma_mine, gamma_opp_sick, gamma_opp_treat
real(mykind) :: alpha_mine
real(mykind) :: q_mine
integer :: other_gender

!Define other variables:
real(mykind) :: util

!------------------------------------------------------------

!Get variables:
if (mygender==i_male) then
    iota = iota_m(mytype)
    A = A_m(mytype)
    chi = chi_m(mytype)
    other_gender = i_female
elseif (mygender==i_female) then
    iota = iota_f(mytype)
    A = A_f(mytype)
    chi = chi_f(mytype)
    other_gender = i_male
else
    print *, 'wrong gender in longterm_y'
    read *, iota
endif

!Parameters for non-transmission rates:
gamma_mine      = gamma_vec(mystatus,mygender,i_unprotected)
gamma_opp_sick  = gamma_vec(i_sick,other_gender,i_unprotected)
gamma_opp_treat = gamma_vec(i_treated,other_gender,i_unprotected)

!Probability that I develop symptoms next period:
alpha_mine = alpha_vec(mystatus)

!Probability that I get treatment next period:
q_mine = q_vec(mystatus)

!Utility from consumption:
call utility(income,util)

!Check whether this person is healthy or not. Then, compute his value conditional on knowing the partner:
if (mystatus==i_healthy) then
    !Partner is healthy:
    vtilde_opp_healthy = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*iota*( eta*vtilde_opp_last(i_healthy,i_healthy,mytype) + (1.0-eta)*vtilde_opp_y_last(i_healthy,i_healthy,mytype) ) + &
                         (1.0-(1.0-epsilon)*(1.0-delta))*iota*( eta*v_l(i_healthy,mytype) + (1.0-eta)*v_l_y(i_healthy,mytype) )
    !Partner is sick:
    vtilde_opp_sick    = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha)*(1.0-(1.0-gamma_opp_sick)*chi)*iota*(q*( eta*vtilde_opp_last(i_healthy,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_healthy,i_treated,mytype) )+(1.0-q)*( eta*vtilde_opp_last(i_healthy,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_healthy,i_sick,mytype) )) + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha)*(1.0-gamma_opp_sick)*chi*iota*(q*q*( eta*vtilde_opp_last(i_treated,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_treated,mytype) )+q*(1.0-q)*( eta*vtilde_opp_last(i_treated,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_sick,mytype) )+(1.0-q)*q*( eta*vtilde_opp_last(i_sick,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_treated,mytype) )+(1.0-q)*(1.0-q)*( eta*vtilde_opp_last(i_sick,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_sick,mytype) )) + &
                         (1.0-(1.0-epsilon)*(1.0-delta)*(1.0-alpha))*iota*((1.0-(1.0-gamma_opp_sick)*chi)*( eta*v_l(i_healthy,mytype) + (1.0-eta)*v_l_y(i_healthy,mytype) )+(1.0-gamma_opp_sick)*chi*((1.0-q)*( eta*v_l(i_sick,mytype) + (1.0-eta)*v_l_y(i_sick,mytype) )+q*( eta*v_l(i_treated,mytype) + (1.0-eta)*v_l_y(i_treated,mytype) )))
    !Partner is treated
    vtilde_opp_treated = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat)*(1.0-(1.0-gamma_opp_treat)*chi)*iota*( eta*vtilde_opp_last(i_healthy,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_healthy,i_treated,mytype) ) + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat)*(1.0-gamma_opp_treat)*chi*iota*(q*( eta*vtilde_opp_last(i_treated,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_treated,mytype) )+(1.0-q)*( eta*vtilde_opp_last(i_sick,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_treated,mytype) )) + &
                         (1.0-(1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat))*iota*((1.0-(1.0-gamma_opp_treat)*chi)*( eta*v_l(i_healthy,mytype) + (1.0-eta)*v_l_y(i_healthy,mytype) )+(1.0-gamma_opp_treat)*chi*((1.0-q)*( eta*v_l(i_sick,mytype) + (1.0-eta)*v_l_y(i_sick,mytype) )+q*( eta*v_l(i_treated,mytype) + (1.0-eta)*v_l_y(i_treated,mytype) )))
else
    !Partner is healthy:
    vtilde_opp_healthy = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_mine)*gamma_mine*iota*(q_mine*( eta*vtilde_opp_last(i_treated,i_healthy,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_healthy,mytype) )+(1.0-q_mine)*( eta*vtilde_opp_last(i_sick,i_healthy,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_healthy,mytype) )) + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_mine)*(1.0-gamma_mine)*iota*(q_mine*q*( eta*vtilde_opp_last(i_treated,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_treated,mytype) )+q_mine*(1.0-q)*( eta*vtilde_opp_last(i_treated,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_sick,mytype) )+(1.0-q_mine)*q*( eta*vtilde_opp_last(i_sick,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_treated,mytype) )+(1.0-q_mine)*(1.0-q)*( eta*vtilde_opp_last(i_sick,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_sick,mytype) )) + &
                         (1.0-(1.0-epsilon)*(1.0-delta))*(1.0-alpha_mine)*iota*(q_mine*( eta*v_l(i_treated,mytype) + (1.0-eta)*v_l_y(i_treated,mytype) )+(1.0-q_mine)*( eta*v_l(i_sick,mytype) + (1.0-eta)*v_l_y(i_sick,mytype) )) + &
                         alpha_mine*iota*A
    !Partner is sick:
    vtilde_opp_sick    = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha)*(1.0-alpha_mine)*iota*(q_mine*q*( eta*vtilde_opp_last(i_treated,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_treated,mytype) )+q_mine*(1.0-q)*( eta*vtilde_opp_last(i_treated,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_sick,mytype) )+(1.0-q_mine)*q*( eta*vtilde_opp_last(i_sick,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_treated,mytype) )+(1.0-q_mine)*(1.0-q)*( eta*vtilde_opp_last(i_sick,i_sick,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_sick,mytype) )) + &
                         (1.0-(1.0-epsilon)*(1.0-delta)*(1.0-alpha))*(1.0-alpha_mine)*iota*(q_mine*( eta*v_l(i_treated,mytype) + (1.0-eta)*v_l_y(i_treated,mytype) )+(1.0-q_mine)*( eta*v_l(i_sick,mytype) + (1.0-eta)*v_l_y(i_sick,mytype) )) + &
                         alpha_mine*iota*A
    !Partner is treated
    vtilde_opp_treated = util + joy + &
                         (1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat)*(1.0-alpha_mine)*iota*(q_mine*( eta*vtilde_opp_last(i_treated,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_treated,i_treated,mytype) )+(1.0-q_mine)*( eta*vtilde_opp_last(i_sick,i_treated,mytype) + (1.0-eta)*vtilde_opp_y_last(i_sick,i_treated,mytype) )) + &
                         (1.0-(1.0-epsilon)*(1.0-delta)*(1.0-alpha_treat))*(1.0-alpha_mine)*iota*(q_mine*( eta*v_l(i_treated,mytype) + (1.0-eta)*v_l_y(i_treated,mytype) )+(1.0-q_mine)*( eta*v_l(i_sick,mytype) + (1.0-eta)*v_l_y(i_sick,mytype) )) + &
                         alpha_mine*iota*A
endif

!Consolidate this person's matched value function with each type of partner:
vtilde_opp(i_healthy) = vtilde_opp_healthy
vtilde_opp(i_sick)    = vtilde_opp_sick
vtilde_opp(i_treated) = vtilde_opp_treated

!Compute this person's ex ante value (without knowing the partner's type):
vtilde = (1.0-frac_sick-frac_treated)*vtilde_opp_healthy + frac_sick*vtilde_opp_sick + frac_treated*vtilde_opp_treated

end subroutine longterm_y
