%This is the main file that computes the equilibrium for the AIDS model. It
%sets some parameters and initial guesses. Then, it calls the fortran
%executable that performs the value function iteration and market clearing
%calculations.

%Start timer:
tic

%Determine which operating system you're using:
findos %returns os='win' or os='unix'

%Read the parameters from file:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
read_parameters

%Make sure we're not running any small scale experiment:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
exec_epid=0; %do not run epidemiological experiment (for now)
exec_smallexp=0; %do not run small field experiment (for now)
fid = fopen( 'exec_epid.txt', 'w' );
fprintf(fid, '%18.0f\n', exec_epid);
fclose(fid);
fid = fopen( 'exec_smallexp.txt', 'w' );
fprintf(fid, '%18.0f\n', exec_smallexp);
fclose(fid);

%Set initial guesses:
%%%%%%%%%%%%%%%%%%%%%

if use_oldstuff==1 && exist(fullfile(cd,'oldstuff.mat'),'file')==2
    %error('For now, you cannot load oldstuff. Please change parameters_all.txt file')
else
    %AVERAGE RATES OF NON-PREVALENCE
    prev_next=[phi_p_m;phi_u_m;phi_l_m;phi_p_f;phi_u_f;phi_l_f;0;0]; %the two zeros are initial guesses for the abstinent population
    
    %something very crude but same as before:
    %VALUE FUNCTION
    v_next_m = 10*ones(n_age*n_status*ntypes_m,1);
    v_next_f = 10*ones(n_age*n_status*ntypes_f,1);
    
    %DISTRIBUTIONS
    
    dist_next_m = (1/(n_age*n_status*ntypes_m))*ones(n_age*n_status*ntypes_m,1);
    dist_next_f = (1/(n_age*n_status*ntypes_f))*ones(n_age*n_status*ntypes_f,1);
    
    %Writing to files:
    fid = fopen( 'dist_next_m.txt', 'w' );
    fprintf(fid, '%18.10f\n', dist_next_m);
    fclose(fid);
    fid = fopen( 'dist_next_f.txt', 'w' );
    fprintf(fid, '%18.10f\n', dist_next_f);
    fclose(fid);
    fid = fopen( 'v_next_m.txt', 'w' );
    fprintf(fid, '%18.10f\n', v_next_m);
    fclose(fid);
    fid = fopen( 'v_next_f.txt', 'w' );
    fprintf(fid, '%18.10f\n', v_next_f);
    fclose(fid);
    fid = fopen( 'prev_next.txt', 'w' );
    fprintf(fid, '%18.10f\n', prev_next);
    fclose(fid);
end

%Fitting search effort functions:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fit_search

%Call the executable:
if strcmp(os,'unix') == 1
    !./aids
elseif strcmp(os,'win')==1
    !aids.exe
else
    error('Specify which operating system you are using; variable "os"')
end

%Read solution from Fortran executable:
read_solution

%Compute and show/record the results:
if strcmp(os,'unix') == 1
    !rm output.txt
elseif strcmp(os,'win')==1
    !del output.txt
end
diary output.txt

%Call results file:
results

if opt_flag==1
    disp('------------------------------------------------------------')
    disp('----------------MARKET CLEARING PRICES:---------------------')
    disp('obs: positive price = men pay')
    disp('------------------------------------------------------------')
    fprintf('protected:   %.4f \n', solution(1))
    fprintf('unprotected: %.4f \n', solution(2))
    fprintf('long term:   %.4f \n', solution(3))
else
    disp('------------------------------------------------------------')
    disp('THE ALGORITHM WAS UNABLE TO FIND MARKET CLEARING PRICES!!!')
    disp('------------------------------------------------------------')
end

diary off

%End timer:
time_all=toc;  %total time
fprintf('Total time was %.4f seconds\n\n',time_all)

%Output file for Excel:
results_m = [results_m opt_flag t_p t_u t_l time_all];
fid = fopen( 'exceloutput.txt', 'w' );
fprintf(fid, '%18.10f\n', results_m);
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Other experiments:
%%%%%%%%%%%%%%%%%%%
if do_epid==1 || do_smallexp==1
    %Get aggregates ready:
    copyfile('prices_out.txt','prices_bench.txt')
    copyfile('prev_out.txt','prev_bench.txt')
    copyfile('upsilon_all.txt','upsilon_all_bench.txt')
    copyfile('frac_l_healthy_bycirc_m.txt','frac_l_healthy_bycirc_m_bench.txt')
    copyfile('frac_l_m.txt','frac_l_m_bench.txt')
    copyfile('frac_l_f.txt','frac_l_f_bench.txt')
    %Get the policy functions ready:
    copyfile('pi_p_f.txt','pi_p_f_bench.txt')
    copyfile('pi_p_m.txt','pi_p_m_bench.txt')
    copyfile('pi_u_f.txt','pi_u_f_bench.txt')
    copyfile('pi_u_m.txt','pi_u_m_bench.txt')
    copyfile('pi_l_f.txt','pi_l_f_bench.txt')
    copyfile('pi_l_m.txt','pi_l_m_bench.txt')
    copyfile('pi_p_y_f.txt','pi_p_y_f_bench.txt')
    copyfile('pi_p_y_m.txt','pi_p_y_m_bench.txt')
    copyfile('pi_u_y_f.txt','pi_u_y_f_bench.txt')
    copyfile('pi_u_y_m.txt','pi_u_y_m_bench.txt')
    copyfile('pi_l_y_f.txt','pi_l_y_f_bench.txt')
    copyfile('pi_l_y_m.txt','pi_l_y_m_bench.txt')
    %Get the distributions ready:
    copyfile('dist_A_m.txt','dist_A_m_bench.txt')
    copyfile('dist_L_m.txt','dist_L_m_bench.txt')
    copyfile('dist_L_y_m.txt','dist_L_y_m_bench.txt')
    copyfile('dist_L_opp_m.txt','dist_L_opp_m_bench.txt')
    copyfile('dist_L_opp_y_m.txt','dist_L_opp_y_m_bench.txt')
    copyfile('dist_S_m.txt','dist_S_m_bench.txt')
    copyfile('dist_S_y_m.txt','dist_S_y_m_bench.txt')
    copyfile('dist_A_f.txt','dist_A_f_bench.txt')
    copyfile('dist_L_f.txt','dist_L_f_bench.txt')
    copyfile('dist_L_y_f.txt','dist_L_y_f_bench.txt')
    copyfile('dist_L_opp_f.txt','dist_L_opp_f_bench.txt')
    copyfile('dist_L_opp_y_f.txt','dist_L_opp_y_f_bench.txt')
    copyfile('dist_S_f.txt','dist_S_f_bench.txt')
    copyfile('dist_S_y_f.txt','dist_S_y_f_bench.txt')
    %Change the files in the parameters:
    copyfile('parameters_all.txt','parameters_bench.txt')
    copyfile('parameters_exp.txt','parameters_all.txt')
    %write measures for short term markets:
    write_shortterm_measures
end

%Read new parameters:
read_parameters

%Epidemiological experiment:
if do_epid==1
    %set up flags:
    exec_epid=1; %RUN epidemiological experiment
    fid = fopen( 'exec_epid.txt', 'w' );
    fprintf(fid, '%18.0f\n', exec_epid);
    fclose(fid);
    exec_smallexp=0; %do not run small field experiment (for now)
    fid = fopen( 'exec_smallexp.txt', 'w' );
    fprintf(fid, '%18.0f\n', exec_smallexp);
    fclose(fid);
    %Call the executable:
    if strcmp(os,'unix') == 1
        !./aids
    elseif strcmp(os,'win')==1
        !aids.exe
    else
        error('Specify which operating system you are using; variable "os"')
    end
    %Read solution from Fortran executable:
    read_solution
    %Call the results file:
    results_exp
    %Save results on excel file:
    results_epid = [results_exper opt_flag t_p t_u t_l time_all];
    fid = fopen( 'exceloutput_epid.txt', 'w' );
    fprintf(fid, '%18.10f\n', results_epid);
    fclose(fid);
end

%Small scale field experiment:
if do_smallexp==1
    %set up flags:
    exec_smallexp=1; %RUN small field experiment 
    fid = fopen( 'exec_smallexp.txt', 'w' );
    fprintf(fid, '%18.0f\n', exec_smallexp);
    fclose(fid);
    exec_epid=0; %do not run epidemiological experiment (for now)
    fid = fopen( 'exec_epid.txt', 'w' );
    fprintf(fid, '%18.0f\n', exec_epid);
    fclose(fid);
    %Call the executable:
    if strcmp(os,'unix') == 1
        !./aids
    elseif strcmp(os,'win')==1
        !aids.exe
    else
        error('Specify which operating system you are using; variable "os"')
    end
    %Read solution from Fortran executable:
    read_solution
    %Call the results file:
    results_exp
    %Save results on excel file:
    results_smallexp = [results_exper opt_flag t_p t_u t_l time_all];
    fid = fopen( 'exceloutput_smallexp.txt', 'w' );
    fprintf(fid, '%18.10f\n', results_smallexp);
    fclose(fid);
end

%Clean up anything that is needed:
if do_epid==1 || do_smallexp==1
    %Correct parameter files:
    copyfile('parameters_bench.txt','parameters_all.txt')
    delete('parameters_bench.txt')
    %Delete temporary benchmark files:
	delete('*bench.txt')
    %set up flags:
    exec_epid=0; %do not run epidemiological experiment (for the future)
    exec_smallexp=0; %do not run small field experiment (for the future)
    fid = fopen( 'exec_epid.txt', 'w' );
    fprintf(fid, '%18.0f\n', exec_epid);
    fclose(fid);
    fid = fopen( 'exec_smallexp.txt', 'w' );
    fprintf(fid, '%18.0f\n', exec_smallexp);
    fclose(fid);
end

