module mod_param

implicit none

!First, define some technical flags:
integer, parameter :: mykind = 8 !double precision

!Define variables for indexes regarding health status:
integer, parameter :: n_status  = 3 !healthy, sick or treated
integer, parameter :: i_healthy = 1
integer, parameter :: i_sick    = 2
integer, parameter :: i_treated = 3

!Define variables for indexes regarding gender:
integer, parameter :: i_male = 1
integer, parameter :: i_female = 2

!Define variables for dimensions:
integer, parameter :: n_gender = 2 !male and female
integer, parameter :: n_age    = 2 !young and old
integer, parameter :: n_prev   = 8 !protected, unprotected, long term, abstinence (*2 genders)

!Define variables for different markets:
integer, parameter :: i_protected = 1 !protected market
integer, parameter :: i_unprotected = 2 !unprotected market

!Define variables for circumcision:
integer, parameter :: n_circ = 2
integer, parameter :: i_nocirc = 1
integer, parameter :: i_circ = 2

!Define variable for prices:
real(mykind) :: prices(3)

!Define variables for dfovec:
real(mykind) :: guess_optimization(3), step_optimization(3)

!Define the variables contained in myparameters.txt:
real(mykind) :: t_p ! (1);
real(mykind) :: t_u ! (2);
real(mykind) :: t_l ! (3);
real(mykind) :: phi_p_m ! (4);
real(mykind) :: phi_u_m ! (5);
real(mykind) :: phi_l_m ! (6);
real(mykind) :: phi_p_f ! (7);
real(mykind) :: phi_u_f ! (8);
real(mykind) :: phi_l_f ! (9);
real(mykind) :: delta ! (10);
real(mykind) :: delta2 ! (11);
real(mykind) :: epsilon ! (12);
real(mykind) :: alpha ! (13);
real(mykind) :: alpha_treat ! (14);
real(mykind) :: gammau_m ! (15);
real(mykind) :: gammap_m ! (16);
real(mykind) :: gammau_m_treat ! (17);
real(mykind) :: gammap_m_treat ! (18);
real(mykind) :: gammau_f ! (19);
real(mykind) :: gammap_f ! (20);
real(mykind) :: gammau_f_treat ! (21);
real(mykind) :: gammap_f_treat ! (22);
real(mykind) :: q ! (23);
real(mykind) :: y_m ! (24);
real(mykind) :: y_f ! (25);
real(mykind) :: p_m ! (26);
real(mykind) :: p_f ! (27);
real(mykind) :: u_m ! (28);
real(mykind) :: u_f ! (29);
real(mykind) :: l_m ! (30);
real(mykind) :: l_f ! (31);
real(mykind) :: eta ! (32);
real(mykind) :: omega_p ! (33);
real(mykind) :: kappa_p ! (34);
real(mykind) :: b_p ! (35);
real(mykind) :: omega_u ! (36);
real(mykind) :: kappa_u ! (37);
real(mykind) :: b_u ! (38);
real(mykind) :: omega_l ! (39);
real(mykind) :: kappa_l ! (40);
real(mykind) :: b_l ! (41);
real(mykind) :: A_global ! (42);
integer :: do_circ ! (43);
real(mykind) :: frac_circ ! (44);
real(mykind) :: chi_circ ! (45);
real(mykind) :: beta_max ! (46);
real(mykind) :: beta_min ! (47);
real(mykind) :: iota_change ! (48);
integer :: placeholder ! (49)-(50) & (75)-(78);
integer :: n_cores ! (51);
integer :: expnumber ! (52);
integer :: ntypes_m ! (53);
integer :: ntypes_f ! (54);
integer :: do_ge ! (55);
integer :: max_iter ! (56);
real(mykind) :: tol ! (57);
real(mykind) :: ddd ! (58);
real(mykind) :: ddd1 ! (59);
integer :: slow ! (60);
real(mykind) :: slow_size ! (61);
integer :: use_oldstuff ! (62);
integer :: do_mc ! (63);
integer :: do_epid ! (64);
integer :: do_epidplus ! (65);
integer :: do_smallexp ! (66);
integer :: do_smallexpnobehav ! (67);
integer :: nparam_bytype ! (68);
integer :: ngrid ! (69);
integer :: ge_max_iter ! (70);
real(mykind) :: ge_tol ! (71);
real(mykind) :: ge_step ! (72);
integer :: do_misinformed ! (73);
real(mykind) :: frac_misinformed ! (74);
real(mykind) :: al_range ! (75);
real(mykind) :: al1 ! (76);
real(mykind) :: al2 ! (77);
real(mykind) :: al3 ! (78);
integer :: save_prev_iter ! (79);
integer :: fgv_server ! (80);

!Define prevalence arrays:
real(mykind) :: prev_next(8)
real(mykind) :: prev1(8)
real(mykind) :: prev0(8)
real(mykind) :: phi_n_m
real(mykind) :: phi_n_f

!Define arrays for probability of treatment, developing symptoms and non-transmission rates:
real(mykind) :: q_vec(n_status)
real(mykind) :: alpha_vec(n_status)
real(mykind) :: gamma_vec(n_status,n_gender,2) !The number 2 is for protected vs unprotected sex

!Define excess demand vector:
real(mykind) :: ed_vector(3)

!Define vector for convergence flags:
integer :: conv_flags(2) !1 is inner, 2 is outer loop

!Define variables that will determine whether to write files containing distributions and policy functions:
integer :: write_more_files

!Define variables for the small scale experiments:
integer :: exec_bench
integer :: exec_epid
integer :: exec_smallexp

!Define helpful variables:
real(mykind), parameter :: huge_negative = -9999999999999.9

!----------------------------------------------------------------------
!Allocatable arrays:

!Define variables to hold the type-specific parameters:
real(mykind), dimension(:), allocatable :: parameters_bytype_m
real(mykind), dimension(:), allocatable :: parameters_bytype_f

!Define variables for type-specific parameters:
real(mykind), dimension(:), allocatable :: A_m
real(mykind), dimension(:), allocatable :: A_f
real(mykind), dimension(:), allocatable :: iota_m
real(mykind), dimension(:), allocatable :: iota_f
real(mykind), dimension(:), allocatable :: beta_m
real(mykind), dimension(:), allocatable :: beta_f
real(mykind), dimension(:), allocatable :: mu_m
real(mykind), dimension(:), allocatable :: mu_f
real(mykind), dimension(:), allocatable :: chi_m
real(mykind), dimension(:), allocatable :: chi_f
real(mykind), dimension(:), allocatable :: util_l_m
real(mykind), dimension(:), allocatable :: util_l_f

!Define variables for fit search efforts:
real(mykind), dimension(:), allocatable :: mygrid
real(mykind), dimension(:), allocatable :: mygrid_con
real(mykind), dimension(:,:), allocatable :: search_p
real(mykind), dimension(:,:), allocatable :: search_u
real(mykind), dimension(:,:), allocatable :: search_l
real(mykind), dimension(:,:), allocatable :: search_ucon

!Define value functions - males:
real(mykind), dimension(:), allocatable :: v_next_m
real(mykind), dimension(:), allocatable :: v1_m
real(mykind), dimension(:), allocatable :: v0_m

!Define value functions - females:
real(mykind), dimension(:), allocatable :: v_next_f
real(mykind), dimension(:), allocatable :: v1_f
real(mykind), dimension(:), allocatable :: v0_f

!Define distributions - males:
real(mykind), dimension(:), allocatable :: dist_next_m
real(mykind), dimension(:), allocatable :: dist0_m
real(mykind), dimension(:), allocatable :: dist1_m

!Define distributions - females:
real(mykind), dimension(:), allocatable :: dist_next_f
real(mykind), dimension(:), allocatable :: dist0_f
real(mykind), dimension(:), allocatable :: dist1_f

!Define variables that hold prevalence rates at each iteration:
real(mykind), dimension(:), allocatable :: phi_p_m_iter
real(mykind), dimension(:), allocatable :: phi_u_m_iter
real(mykind), dimension(:), allocatable :: phi_l_m_iter
real(mykind), dimension(:), allocatable :: phi_p_f_iter
real(mykind), dimension(:), allocatable :: phi_u_f_iter
real(mykind), dimension(:), allocatable :: phi_l_f_iter

end module mod_param
