%This m-file computes several different moments for men of different
%circumcision statuses

% HIV PREVALENCE RATE:
%%%%%%%%%%%%%%%%%%%%%%

%Prevalence for non-circumcised men:
first_noncirc = 1;
last_noncirc  = ntypes_m/2;
measure_noncirc = sum(sum( dist_S_m(:,first_noncirc:last_noncirc) + dist_S_y_m(:,first_noncirc:last_noncirc) + dist_L_m(:,first_noncirc:last_noncirc) + dist_L_y_m(:,first_noncirc:last_noncirc) + dist_A_m(:,first_noncirc:last_noncirc) ));
sick_noncirc = sum(sum( dist_S_m(2:3,first_noncirc:last_noncirc) + dist_S_y_m(2:3,first_noncirc:last_noncirc) + dist_L_m(2:3,first_noncirc:last_noncirc) + dist_L_y_m(2:3,first_noncirc:last_noncirc) + dist_A_m(2:3,first_noncirc:last_noncirc) ));
percentage_aids_noncirc = sick_noncirc / measure_noncirc;

%Prevalence for circumcised men:
first_circ = ntypes_m/2+1;
last_circ  = ntypes_m;
measure_circ = sum(sum( dist_S_m(:,first_circ:last_circ) + dist_S_y_m(:,first_circ:last_circ) + dist_L_m(:,first_circ:last_circ) + dist_L_y_m(:,first_circ:last_circ) + dist_A_m(:,first_circ:last_circ) ));
sick_circ = sum(sum( dist_S_m(2:3,first_circ:last_circ) + dist_S_y_m(2:3,first_circ:last_circ) + dist_L_m(2:3,first_circ:last_circ) + dist_L_y_m(2:3,first_circ:last_circ) + dist_A_m(2:3,first_circ:last_circ) ));
percentage_aids_circ = sick_circ / measure_circ;

%Consolidate the two and write out to a file:
percentage_aids_bycirc = [percentage_aids_noncirc; percentage_aids_circ];
fid = fopen( 'percentage_aids_bycirc.txt', 'w' );
fprintf(fid, '%18.10f\n', percentage_aids_bycirc);
fclose(fid);

%CONSOLIDATED DISTRIBUTIONS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Non-circumcised:
dist_S_all_noncirc = sum(dist_S_m(:,first_noncirc:last_noncirc)+dist_S_y_m(:,first_noncirc:last_noncirc),2);
dist_L_all_noncirc = sum(dist_L_m(:,first_noncirc:last_noncirc)+dist_L_y_m(:,first_noncirc:last_noncirc),2);
dist_A_all_noncirc = sum(dist_A_m(:,first_noncirc:last_noncirc),2);

%Circumcised:
dist_S_all_circ = sum(dist_S_m(:,first_circ:last_circ)+dist_S_y_m(:,first_circ:last_circ),2);
dist_L_all_circ = sum(dist_L_m(:,first_circ:last_circ)+dist_L_y_m(:,first_circ:last_circ),2);
dist_A_all_circ = sum(dist_A_m(:,first_circ:last_circ),2);

% PARTICIPATION IN DIFFERENT MARKETS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Non-circumcised:
measure_p_noncirc = sum((1.0-pi_l_m(:,first_noncirc:last_noncirc)).*pi_p_m(:,first_noncirc:last_noncirc).*dist_S_m(:,first_noncirc:last_noncirc),2);
measure_u_noncirc = sum((1.0-pi_l_m(:,first_noncirc:last_noncirc)).*pi_u_m(:,first_noncirc:last_noncirc).*dist_S_m(:,first_noncirc:last_noncirc),2);
measure_a_noncirc = sum((1.0-pi_l_m(:,first_noncirc:last_noncirc)).*(1.0-pi_p_m(:,first_noncirc:last_noncirc)-pi_u_m(:,first_noncirc:last_noncirc)).*dist_S_m(:,first_noncirc:last_noncirc),2);
measure_l_noncirc = sum(pi_l_m(:,first_noncirc:last_noncirc).*dist_S_m(:,first_noncirc:last_noncirc),2);
measure_p_y_noncirc = sum((1.0-pi_l_y_m(:,first_noncirc:last_noncirc)).*pi_p_y_m(:,first_noncirc:last_noncirc).*dist_S_y_m(:,first_noncirc:last_noncirc),2);
measure_u_y_noncirc = sum((1.0-pi_l_y_m(:,first_noncirc:last_noncirc)).*pi_u_y_m(:,first_noncirc:last_noncirc).*dist_S_y_m(:,first_noncirc:last_noncirc),2);
measure_a_y_noncirc = sum((1.0-pi_l_y_m(:,first_noncirc:last_noncirc)).*(1.0-pi_p_y_m(:,first_noncirc:last_noncirc)-pi_u_y_m(:,first_noncirc:last_noncirc)).*dist_S_y_m(:,first_noncirc:last_noncirc),2);
measure_l_y_noncirc = sum(pi_l_y_m(:,first_noncirc:last_noncirc).*dist_S_y_m(:,first_noncirc:last_noncirc),2);
measure_p_all_noncirc = measure_p_noncirc + measure_p_y_noncirc;
measure_u_all_noncirc = measure_u_noncirc + measure_u_y_noncirc;
measure_l_all_noncirc = measure_l_noncirc + measure_l_y_noncirc;
measure_a_all_noncirc = measure_a_noncirc + measure_a_y_noncirc;

%Circumcised:
measure_p_circ = sum((1.0-pi_l_m(:,first_circ:last_circ)).*pi_p_m(:,first_circ:last_circ).*dist_S_m(:,first_circ:last_circ),2);
measure_u_circ = sum((1.0-pi_l_m(:,first_circ:last_circ)).*pi_u_m(:,first_circ:last_circ).*dist_S_m(:,first_circ:last_circ),2);
measure_a_circ = sum((1.0-pi_l_m(:,first_circ:last_circ)).*(1.0-pi_p_m(:,first_circ:last_circ)-pi_u_m(:,first_circ:last_circ)).*dist_S_m(:,first_circ:last_circ),2);
measure_l_circ = sum(pi_l_m(:,first_circ:last_circ).*dist_S_m(:,first_circ:last_circ),2);
measure_p_y_circ = sum((1.0-pi_l_y_m(:,first_circ:last_circ)).*pi_p_y_m(:,first_circ:last_circ).*dist_S_y_m(:,first_circ:last_circ),2);
measure_u_y_circ = sum((1.0-pi_l_y_m(:,first_circ:last_circ)).*pi_u_y_m(:,first_circ:last_circ).*dist_S_y_m(:,first_circ:last_circ),2);
measure_a_y_circ = sum((1.0-pi_l_y_m(:,first_circ:last_circ)).*(1.0-pi_p_y_m(:,first_circ:last_circ)-pi_u_y_m(:,first_circ:last_circ)).*dist_S_y_m(:,first_circ:last_circ),2);
measure_l_y_circ = sum(pi_l_y_m(:,first_circ:last_circ).*dist_S_y_m(:,first_circ:last_circ),2);
measure_p_all_circ = measure_p_circ + measure_p_y_circ;
measure_u_all_circ = measure_u_circ + measure_u_y_circ;
measure_l_all_circ = measure_l_circ + measure_l_y_circ;
measure_a_all_circ = measure_a_circ + measure_a_y_circ;

% CASUAL SEX:
%%%%%%%%%%%%%

%Non-circumcised:
prob_s_all_noncirc = sum(measure_p_all_noncirc+measure_u_all_noncirc) / sum(measure_p_all_noncirc+measure_u_all_noncirc+measure_l_all_noncirc+dist_L_all_noncirc);
%Circumcised:
prob_s_all_circ = sum(measure_p_all_circ+measure_u_all_circ) / sum(measure_p_all_circ+measure_u_all_circ+measure_l_all_circ+dist_L_all_circ);


% CONDOM USAGE:
%%%%%%%%%%%%%%%

%Non-circumcised:
prob_condom_all_noncirc = sum(measure_p_all_noncirc) / sum(measure_p_all_noncirc+measure_u_all_noncirc);
%Circumcised:
prob_condom_all_circ = sum(measure_p_all_circ) / sum(measure_p_all_circ+measure_u_all_circ);


% NON-ABSTINENT SINGLES:
%%%%%%%%%%%%%%%%%%%%%%%%

%Non-circumcised:
prob_shortsex_all_noncirc = sum(measure_p_all_noncirc+measure_u_all_noncirc) / sum(measure_p_all_noncirc+measure_u_all_noncirc+measure_a_all_noncirc);
%Circumcised:
prob_shortsex_all_circ = sum(measure_p_all_circ+measure_u_all_circ) / sum(measure_p_all_circ+measure_u_all_circ+measure_a_all_circ);


% FRACTION OF SINGLES:
%%%%%%%%%%%%%%%%%%%%%%

%Non-circumcised:
fraction_single_noncirc = sum(dist_S_all_noncirc+dist_A_all_noncirc) / sum(dist_S_all_noncirc+dist_L_all_noncirc+dist_A_all_noncirc);
%Non-circumcised:
fraction_single_circ = sum(dist_S_all_circ+dist_A_all_circ) / sum(dist_S_all_circ+dist_L_all_circ+dist_A_all_circ);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%CONSOLIDATE MOMENTS INTO ONE SINGLE ARRAY:
results_bycirc = [ percentage_aids_noncirc percentage_aids_circ; ...
                   prob_s_all_noncirc prob_s_all_circ; ...
                   prob_condom_all_noncirc prob_condom_all_circ; ...
                   prob_shortsex_all_noncirc prob_shortsex_all_circ; ...
                   fraction_single_noncirc fraction_single_circ ];

%Write this out into a file:
csvwrite('moments_bycirc.csv',results_bycirc)



