%This file reads the parameters and control flags that will be used to
%solve the model.

%Set some dimensions to help:
n_gender = 2;   %male and female
n_age = 2;      %young and old
n_status = 3;   %healthy, sick or treated

%Set some useful flags:
i_healthy = 1;
i_sick    = 2;
i_treated = 3;

%Load parameters from file:
load parameters_all.txt

%Set the overall parameters:
t_p = parameters_all(1);
t_u = parameters_all(2);
t_l = parameters_all(3);
phi_p_m = parameters_all(4);
phi_u_m = parameters_all(5);
phi_l_m = parameters_all(6);
phi_p_f = parameters_all(7);
phi_u_f = parameters_all(8);
phi_l_f = parameters_all(9);
delta = parameters_all(10);
delta2 = parameters_all(11);
epsilon = parameters_all(12);
alpha = parameters_all(13);
alpha_treat = parameters_all(14);
gammau_m = parameters_all(15);
gammap_m = parameters_all(16);
gammau_m_treat = parameters_all(17);
gammap_m_treat = parameters_all(18);
gammau_f = parameters_all(19);
gammap_f = parameters_all(20);
gammau_f_treat = parameters_all(21);
gammap_f_treat = parameters_all(22);
q = parameters_all(23);
y_m = parameters_all(24);
y_f = parameters_all(25);
p_m = parameters_all(26);
p_f = parameters_all(27);
u_m = parameters_all(28);
u_f = parameters_all(29);
l_m = parameters_all(30);
l_f = parameters_all(31);
eta = parameters_all(32);
omega_p = parameters_all(33);
kappa_p = parameters_all(34);
b_p = parameters_all(35);
omega_u = parameters_all(36);
kappa_u = parameters_all(37);
b_u = parameters_all(38);
omega_l = parameters_all(39);
kappa_l = parameters_all(40);
b_l = parameters_all(41);
A_global = parameters_all(42);
do_circ = parameters_all(43);
frac_circ = parameters_all(44);
chi_circ = parameters_all(45);
beta_max = parameters_all(46);
beta_min = parameters_all(47);
iota_change = parameters_all(48);
%several placeholders
n_cores = parameters_all(51);
expnumber = parameters_all(52);
ntypes_m = parameters_all(53);
ntypes_f = parameters_all(54);
do_ge = parameters_all(55);
max_iter = parameters_all(56);
tol = parameters_all(57);
ddd = parameters_all(58);
ddd1 = parameters_all(59);
slow = parameters_all(60);
slow_size = parameters_all(61);
use_oldstuff = parameters_all(62);
do_mc = parameters_all(63);
do_epid = parameters_all(64);
do_epidplus = parameters_all(65);
do_smallexp = parameters_all(66);
do_smallexpnobehav = parameters_all(67);
nparam_bytype = parameters_all(68);
ngrid = parameters_all(69);
ge_max_iter = parameters_all(70);
ge_tol = parameters_all(71);
ge_step = parameters_all(72);
do_misinformed = parameters_all(73);
frac_misinformed = parameters_all(74);
al_range = parameters_all(75);
al1 = parameters_all(76);
al2 = parameters_all(77);
al3 = parameters_all(78);
save_prev_iter = parameters_all(79);
fgv_server = parameters_all(80);


%SET THE TYPE SPECIFIC PARAMETERS:

if do_misinformed==0
    
    %Set the parameters that are specific by type - females:
    beta_f = linspace(beta_max,beta_min,ntypes_f)' * (1-delta);
    iota_f = iota_change * beta_f;
    A_f = A_global * ones(ntypes_f,1);
    chi_f = ones(ntypes_f,1);
    mu_f = (1/ntypes_f) * ones(ntypes_f,1);
    xaxisvar = linspace(-al_range,al_range,ntypes_f)';
    util_l_f = l_f + al1*xaxisvar + al2*xaxisvar.^2 + al3*xaxisvar.^3;
    
    %Set the parameters that are specific by type - males:
    if do_circ==1
        if mod(ntypes_m,2)~=0
            error('For circumcision, you need an even number of male types')
        end
        A_m = A_global * ones(ntypes_m,1);
        beta_help = linspace(beta_max,beta_min,ntypes_m/2)' * (1-delta);
        beta_m = [beta_help; beta_help];
        iota_m = iota_change * beta_m;
        mu_m_nocirc = (1/ntypes_m) * ones(ntypes_m/2,1) * (1-frac_circ);
        mu_m_circ = (1/ntypes_m) * ones(ntypes_m/2,1) * frac_circ;
        mu_m = [mu_m_nocirc; mu_m_circ];
        mu_m = mu_m/sum(mu_m);
        chi_m_nocirc = ones(ntypes_m/2,1);
        chi_m_circ = chi_circ * ones(ntypes_m/2,1);
        chi_m = [chi_m_nocirc; chi_m_circ];
        xaxisvar = linspace(-al_range,al_range,ntypes_m/2)';
        util_l_m = l_m + al1*xaxisvar + al2*xaxisvar.^2 + al3*xaxisvar.^3;
        util_l_m = [util_l_m; util_l_m];
    else
        beta_m = linspace(beta_max,beta_min,ntypes_m)' * (1-delta);
        iota_m = iota_change * beta_m;
        A_m = A_global * ones(ntypes_m,1);
        chi_m = ones(ntypes_m,1);
        mu_m = (1/ntypes_m) * ones(ntypes_m,1);
        xaxisvar = linspace(-al_range,al_range,ntypes_m)';
        util_l_m = l_m + al1*xaxisvar + al2*xaxisvar.^2 + al3*xaxisvar.^3;
    end
    
elseif do_misinformed==1
    
    %Check if we have enough types to divide among groups:
    if mod(ntypes_m,4)~=0
        error('For misinformation, you need a number of male types divisible by 4')
    end
    if mod(ntypes_f,2)~=0
        error('For misinformation, you need a number of female types divisible by 2')
    end
    
    %Set helpful indices:
    ntypes_m_help = ntypes_m/2;
    ntypes_f_help = ntypes_f/2;
    
    %Set the parameters that are specific by type - females:
    beta_f = zeros(ntypes_f,1);
    beta_f(1:ntypes_f_help) = linspace(beta_max,beta_min,ntypes_f_help)' * (1-delta);
    beta_f(1+ntypes_f_help:ntypes_f) = beta_f(1:ntypes_f_help);
    iota_f = iota_change * beta_f;
    A_f = A_global * ones(ntypes_f,1);
    chi_f = ones(ntypes_f,1);
    mu_f = ones(ntypes_f,1);
    mu_f(1:ntypes_f_help) = (1-frac_misinformed) * (1/ntypes_f_help) * ones(ntypes_f_help,1);
    mu_f(1+ntypes_f_help:ntypes_f) = frac_misinformed * (1/ntypes_f_help) * ones(ntypes_f_help,1);
    xaxisvar = linspace(-al_range,al_range,ntypes_f/2)';
    util_l_f = l_f + al1*xaxisvar + al2*xaxisvar.^2 + al3*xaxisvar.^3;
    util_l_f = [util_l_f; util_l_f];
    
    %Set the parameters that are specific by type - males:
    if do_circ==1
        A_m = A_global * ones(ntypes_m,1);
        beta_help = linspace(beta_max,beta_min,ntypes_m_help/2)' * (1-delta);
        beta_m = [beta_help; beta_help; beta_help; beta_help]; %nocirc-informed,circ-informed,nocirc-misinformed,circ-misinformed
        iota_m = iota_change * beta_m;
        mu_m_nocirc = (1/ntypes_m_help) * ones(ntypes_m_help/2,1) * (1-frac_circ);
        mu_m_circ = (1/ntypes_m_help) * ones(ntypes_m_help/2,1) * frac_circ;
        mu_m = [(1-frac_misinformed)*mu_m_nocirc; (1-frac_misinformed)*mu_m_circ; frac_misinformed*mu_m_nocirc; frac_misinformed*mu_m_circ]; %nocirc-informed,circ-informed,nocirc-misinformed,circ-misinformed
        mu_m = mu_m/sum(mu_m);
        chi_m_nocirc = ones(ntypes_m_help/2,1);
        chi_m_circ = chi_circ * ones(ntypes_m_help/2,1);
        chi_m = [chi_m_nocirc; chi_m_circ; chi_m_nocirc; chi_m_nocirc]; %nocirc-informed,circ-informed,nocirc-misinformed,circ-misinformed
        xaxisvar = linspace(-al_range,al_range,ntypes_m/4)';
        util_l_m = l_m + al1*xaxisvar + al2*xaxisvar.^2 + al3*xaxisvar.^3;
        util_l_m = [util_l_m; util_l_m; util_l_m; util_l_m];
    else
        beta_m(1:ntypes_m_help) = linspace(beta_max,beta_min,ntypes_m_help)' * (1-delta);
        beta_m(1+ntypes_m_help:ntypes_m) = beta_m(1:ntypes_m_help);
        iota_m = iota_change * beta_m;
        A_m = A_global * ones(ntypes_m,1);
        chi_m = ones(ntypes_m,1);
        mu_m(1:ntypes_f_help) = (1-frac_misinformed) * (1/ntypes_m) * ones(ntypes_m_help,1);
        mu_m(1+ntypes_f_help:ntypes_m) = frac_misinformed * (1/ntypes_m) * ones(ntypes_m_help,1);
        xaxisvar = linspace(-al_range,al_range,ntypes_m/2)';
        util_l_m = l_m + al1*xaxisvar + al2*xaxisvar.^2 + al3*xaxisvar.^3;
        util_l_m = [util_l_m; util_l_m];
    end
    
else
    error('No valid choice of do_misinformed')
end

