subroutine smallexp(guess,excess_vector)
!This function solves the small scale experiment for the AIDS/Malawi project given the prices.
!The inputs are the prices of each market (t_p,t_u,t_l): protected, unprotected and long term markets, respectively.
use mod_param
implicit none

!Define dummy variables:
real(mykind), intent(in) :: guess(3)
real(mykind), intent(out) :: excess_vector(3)

!Define variables for the excess demands:
real(mykind) :: ed_p
real(mykind) :: ed_u
real(mykind) :: ed_l
real(mykind) :: normalize

!Define variables for value functions:
real(mykind) :: v_l_y_m(n_status,ntypes_m)
real(mykind) :: v_l_m(n_status,ntypes_m)
real(mykind) :: v_l_y_f(n_status,ntypes_f)
real(mykind) :: v_l_f(n_status,ntypes_f)
real(mykind) :: v_l_y_m0(n_status,ntypes_m)
real(mykind) :: v_l_m0(n_status,ntypes_m)
real(mykind) :: v_l_y_f0(n_status,ntypes_f)
real(mykind) :: v_l_f0(n_status,ntypes_f)
real(mykind) :: v_s_y_m(n_status,ntypes_m)
real(mykind) :: v_s_m(n_status,ntypes_m)
real(mykind) :: v_s_y_f(n_status,ntypes_f)
real(mykind) :: v_s_f(n_status,ntypes_f)

!Define variables for matched value functions (vtilde's) - short term:
real(mykind) :: vtilde_a_y_m(n_status,ntypes_m)
real(mykind) :: vtilde_a_m(n_status,ntypes_m)
real(mykind) :: vtilde_a_y_f(n_status,ntypes_f)
real(mykind) :: vtilde_a_f(n_status,ntypes_f)
real(mykind) :: vtilde_p_y_m(n_status,ntypes_m)
real(mykind) :: vtilde_p_m(n_status,ntypes_m)
real(mykind) :: vtilde_p_y_f(n_status,ntypes_f)
real(mykind) :: vtilde_p_f(n_status,ntypes_f)
real(mykind) :: vtilde_u_y_m(n_status,ntypes_m)
real(mykind) :: vtilde_u_m(n_status,ntypes_m)
real(mykind) :: vtilde_u_y_f(n_status,ntypes_f)
real(mykind) :: vtilde_u_f(n_status,ntypes_f)

!Define variables for matched value functions (vtilde's) - long term:
real(mykind) :: vtilde_l_y_m(n_status,ntypes_m)
real(mykind) :: vtilde_l_m(n_status,ntypes_m)
real(mykind) :: vtilde_l_y_f(n_status,ntypes_f)
real(mykind) :: vtilde_l_f(n_status,ntypes_f)
real(mykind) :: vtilde_l_opp_m(n_status,n_status,ntypes_m)
real(mykind) :: vtilde_l_opp_y_m(n_status,n_status,ntypes_m)
real(mykind) :: vtilde_l_opp_f(n_status,n_status,ntypes_f)
real(mykind) :: vtilde_l_opp_y_f(n_status,n_status,ntypes_f)
real(mykind) :: vtilde_l_opp_m0(n_status,n_status,ntypes_m)
real(mykind) :: vtilde_l_opp_y_m0(n_status,n_status,ntypes_m)
real(mykind) :: vtilde_l_opp_f0(n_status,n_status,ntypes_f)
real(mykind) :: vtilde_l_opp_y_f0(n_status,n_status,ntypes_f)

!Define variables for search effort (pi's):
real(mykind) :: pi_p_y_m(n_status,ntypes_m)
real(mykind) :: pi_p_m(n_status,ntypes_m)
real(mykind) :: pi_p_y_f(n_status,ntypes_f)
real(mykind) :: pi_p_f(n_status,ntypes_f)
real(mykind) :: pi_u_y_m(n_status,ntypes_m)
real(mykind) :: pi_u_m(n_status,ntypes_m)
real(mykind) :: pi_u_y_f(n_status,ntypes_f)
real(mykind) :: pi_u_f(n_status,ntypes_f)
real(mykind) :: pi_l_y_m(n_status,ntypes_m)
real(mykind) :: pi_l_m(n_status,ntypes_m)
real(mykind) :: pi_l_y_f(n_status,ntypes_f)
real(mykind) :: pi_l_f(n_status,ntypes_f)

!Define variables for matched value functions (vtilde's) in the long term, by circumcision type (for females only):
real(mykind) :: vtilde_opp_bycirc_f(n_status,n_circ,ntypes_f)
real(mykind) :: vtilde_opp_bycirc_y_f(n_status,n_circ,ntypes_f)
real(mykind) :: vtilde_opp_bycirc_f0(n_status,n_circ,ntypes_f)
real(mykind) :: vtilde_opp_bycirc_y_f0(n_status,n_circ,ntypes_f)

!Define variables that help with the search efforts (pi's):
real(mykind) :: pi_storage(1,n_status)
real(mykind) :: pi_both(n_status)
real(mykind) :: pi_l_y_m_old(n_status,ntypes_m)
real(mykind) :: pi_l_y_f_old(n_status,ntypes_f)
real(mykind) :: pi_l_m_old(n_status,ntypes_m)
real(mykind) :: pi_l_f_old(n_status,ntypes_f)

!Define variables for distributions - singles:
real(mykind) :: dist_S_m(n_status,ntypes_m)
real(mykind) :: dist_S_y_m(n_status,ntypes_m)
real(mykind) :: dist_S_f(n_status,ntypes_f)
real(mykind) :: dist_S_y_f(n_status,ntypes_f)
real(mykind) :: dist_S_m0(n_status,ntypes_m)
real(mykind) :: dist_S_y_m0(n_status,ntypes_m)
real(mykind) :: dist_S_f0(n_status,ntypes_f)
real(mykind) :: dist_S_y_f0(n_status,ntypes_f)

!Define variables for temporary distributions - singles:
real(mykind) :: dist_S_m_pr(n_status,ntypes_m)
real(mykind) :: dist_S_m_prpr(n_status,ntypes_m)
real(mykind) :: dist_S_f_pr(n_status,ntypes_f)
real(mykind) :: dist_S_f_prpr(n_status,ntypes_f)
real(mykind) :: dist_S_y_m_pr(n_status,ntypes_m)
real(mykind) :: dist_S_y_m_prpr(n_status,ntypes_m)
real(mykind) :: dist_S_y_f_pr(n_status,ntypes_f)
real(mykind) :: dist_S_y_f_prpr(n_status,ntypes_f)

!Define variables for distributions - married:
real(mykind) :: dist_L_m(n_status,ntypes_m)
real(mykind) :: dist_L_y_m(n_status,ntypes_m)
real(mykind) :: dist_L_f(n_status,ntypes_f)
real(mykind) :: dist_L_y_f(n_status,ntypes_f)
real(mykind) :: dist_L_opp_m(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_m(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_f(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_f(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_m0(n_status,ntypes_m)
real(mykind) :: dist_L_y_m0(n_status,ntypes_m)
real(mykind) :: dist_L_f0(n_status,ntypes_f)
real(mykind) :: dist_L_y_f0(n_status,ntypes_f)
real(mykind) :: dist_L_opp_m0(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_m0(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_f0(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_f0(n_status,n_status,ntypes_f)

!Define variables for temporary distributions - married:
real(mykind) :: dist_L_opp_m_pr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_m_prpr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_f_pr(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_f_prpr(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_m_pr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_m_prpr(n_status,n_status,ntypes_m)
real(mykind) :: dist_L_opp_y_f_pr(n_status,n_status,ntypes_f)
real(mykind) :: dist_L_opp_y_f_prpr(n_status,n_status,ntypes_f)

!Define variables for auxiliary distributions for circumcision status of husbands:
real(mykind) :: dist_L_opp_bycirc_f(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_f0(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_f_pr(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_f_prpr(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f0(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f_pr(n_status,n_circ,ntypes_f)
real(mykind) :: dist_L_opp_bycirc_y_f_prpr(n_status,n_circ,ntypes_f)

!Define variables for temporary distributions - with symptoms:
real(mykind) :: dist_A_m(n_status,ntypes_m)
real(mykind) :: dist_A_f(n_status,ntypes_f)
real(mykind) :: dist_A_m0(n_status,ntypes_m)
real(mykind) :: dist_A_f0(n_status,ntypes_f)

!Define variables for infectiousness:
real(mykind) :: upsilon_p_m
real(mykind) :: upsilon_u_m
real(mykind) :: upsilon_p_f
real(mykind) :: upsilon_u_f
real(mykind) :: upsilon_all(4)

!Define variables for who goes into each market:
real(mykind) :: measure_p_m(n_status)
real(mykind) :: measure_p_y_m(n_status)
real(mykind) :: measure_u_m(n_status)
real(mykind) :: measure_u_y_m(n_status)
real(mykind) :: measure_l_m(n_status)
real(mykind) :: measure_l_y_m(n_status)
real(mykind) :: measure_a_m(n_status)
real(mykind) :: measure_a_y_m(n_status)
real(mykind) :: measure_p_f(n_status)
real(mykind) :: measure_p_y_f(n_status)
real(mykind) :: measure_u_f(n_status)
real(mykind) :: measure_u_y_f(n_status)
real(mykind) :: measure_l_f(n_status)
real(mykind) :: measure_l_y_f(n_status)
real(mykind) :: measure_a_f(n_status)
real(mykind) :: measure_a_y_f(n_status)

!Define variables for who goes into each market - consolidated:
real(mykind) :: measure_p_all_m(n_status)
real(mykind) :: measure_u_all_m(n_status)
real(mykind) :: measure_p_all_f(n_status)
real(mykind) :: measure_u_all_f(n_status)

!Define variables for fraction of people with different statuses in long term relationships:
real(mykind) :: frac_healthy_nocirc_m
real(mykind) :: frac_healthy_circ_m
real(mykind) :: frac_sick_m
real(mykind) :: frac_treated_m
real(mykind) :: frac_sick_f
real(mykind) :: frac_treated_f
real(mykind) :: frac_l_m_vec(n_status)
real(mykind) :: frac_l_f_vec(n_status)
real(mykind) :: frac_l_healthy_bycirc_m(n_circ)

!Define variables for the search cost function:
real(mykind) :: thecost1(n_status)
real(mykind) :: thecost2(n_status)

!Define index variables:
integer :: iter, ii, jj, kk, which_entry

!Define other variables:
real(mykind) :: norm, norm_vec(22), prices_vec(3), prev_vec(8)


!----------------------------------------------------------------
!----------------------------------------------------------------
!----------------------------------------------------------------

!GET PRICES:
open(11,file='prices_bench.txt',status='old')
    read(11,*) prices_vec
close(11)
t_p = prices_vec(1)
t_u = prices_vec(2)
t_l = prices_vec(3)

print*, 'Evaluating the small field experiment at the following prices'
print *, 't_p=', t_p, 't_u=', t_u, 't_l=',t_l
print *, ''

!PREVALENCE RATES:
open(11,file='prev_bench.txt',status='old')
    read(11,*) prev_vec
close(11)
phi_p_m = prev_vec(1)
phi_u_m = prev_vec(2)
phi_l_m = prev_vec(3)
phi_p_f = prev_vec(4)
phi_u_f = prev_vec(5)
phi_l_f = prev_vec(6)
phi_n_m = prev_vec(7)
phi_n_f = prev_vec(8)

!Measures in the short term markets:
!Males:
open(11,file='measure_p_all_m.txt',status='old')
	do ii = 1,n_status
    	read(11,*) measure_p_all_m(ii)
    enddo
close(11)
open(11,file='measure_u_all_m.txt',status='old')
	do ii = 1,n_status
    	read(11,*) measure_u_all_m(ii)
    enddo
close(11)
!Females:
open(11,file='measure_p_all_f.txt',status='old')
	do ii = 1,n_status
    	read(11,*) measure_p_all_f(ii)
    enddo
close(11)
open(11,file='measure_u_all_f.txt',status='old')
	do ii = 1,n_status
    	read(11,*) measure_u_all_f(ii)
    enddo
close(11)

!Infectiousness in short term market:
upsilon_p_m = sum( (1.0-gamma_vec(:,i_male,i_protected))*measure_p_all_m ) / sum(measure_p_all_m)
upsilon_u_m = sum( (1.0-gamma_vec(:,i_male,i_unprotected))*measure_u_all_m ) / sum(measure_u_all_m)
upsilon_p_f = sum( (1.0-gamma_vec(:,i_female,i_protected))*measure_p_all_f ) / sum(measure_p_all_f)
upsilon_u_f = sum( (1.0-gamma_vec(:,i_female,i_unprotected))*measure_u_all_f ) / sum(measure_u_all_f)

!Fraction of sick/treated people in the long term market:
!Males:
open(11,file='frac_l_m_bench.txt',status='old')
do jj=1,n_status
    read(11,*) frac_l_m_vec(jj)
enddo
close(11)
frac_sick_m = frac_l_m_vec(2)
frac_treated_m = frac_l_m_vec(3)
open(11,file='frac_l_healthy_bycirc_m_bench.txt',status='old')
    read(11,*) frac_healthy_nocirc_m
    read(11,*) frac_healthy_circ_m
close(11)
frac_l_healthy_bycirc_m(i_nocirc) = frac_healthy_nocirc_m
frac_l_healthy_bycirc_m(i_circ)   = frac_healthy_circ_m
!Females:
open(11,file='frac_l_f_bench.txt',status='old')
do jj=1,n_status
    read(11,*) frac_l_f_vec(jj)
enddo
close(11)
frac_sick_f = frac_l_f_vec(2)
frac_treated_f = frac_l_f_vec(3)

!TAKE INITIAL GUESSES:
dist1_m = dist_next_m
dist1_f = dist_next_f
v1_m = v_next_m
v1_f = v_next_f

!DISTRIBUTIONS:

!Males - short term:
do ii=1,ntypes_m
    !old:
    which_entry = (ii-1)*n_status*n_age
    dist_S_m(i_healthy,ii) = dist1_m(which_entry+i_healthy)
    dist_S_m(i_sick,ii)    = dist1_m(which_entry+i_sick)
    dist_S_m(i_treated,ii) = dist1_m(which_entry+i_treated)
    !young:
    which_entry = (ii-1)*n_status*n_age + n_status
    dist_S_y_m(i_healthy,ii) = dist1_m(which_entry+i_healthy)
    dist_S_y_m(i_sick,ii)    = dist1_m(which_entry+i_sick)
    dist_S_y_m(i_treated,ii) = dist1_m(which_entry+i_treated)
enddo

!Females - short term:
do ii=1,ntypes_f
    !old:
    which_entry = (ii-1)*n_status*n_age
    dist_S_f(i_healthy,ii) = dist1_f(which_entry+i_healthy)
    dist_S_f(i_sick,ii)    = dist1_f(which_entry+i_sick)
    dist_S_f(i_treated,ii) = dist1_f(which_entry+i_treated)
    !young:
    which_entry = (ii-1)*n_status*n_age + n_status
    dist_S_y_f(i_healthy,ii) = dist1_f(which_entry+i_healthy)
    dist_S_y_f(i_sick,ii)    = dist1_f(which_entry+i_sick)
    dist_S_y_f(i_treated,ii) = dist1_f(which_entry+i_treated)
enddo

!Males - long term:
dist_L_m = 2.0*dist_S_m
dist_L_y_m = 2.0*dist_S_y_m
do jj=1,n_status
    dist_L_opp_m(:,jj,:) = 1.0/3.0*dist_L_m
    dist_L_opp_y_m(:,jj,:) = 1.0/3.0*dist_L_y_m
enddo

!Females - long term:
dist_L_f = 2.0*dist_S_f
dist_L_y_f = 2.0*dist_S_y_f
do jj=1,n_status
    dist_L_opp_f(:,jj,:) = 1.0/3.0*dist_L_f
    dist_L_opp_y_f(:,jj,:) = 1.0/3.0*dist_L_y_f
enddo

!With symptoms:
dist_A_m = alpha*dist_S_m
dist_A_f = alpha*dist_S_f

!VALUE FUNCTIONS:
v_l_m = 10.0d0
v_l_y_m = 10.0d0
vtilde_l_opp_m = 11.0d0
vtilde_l_opp_y_m = 11.0d0
v_l_f = 10.0d0
v_l_y_f = 10.0d0
vtilde_l_opp_f = 11.0d0
vtilde_l_opp_y_f = 11.0d0
vtilde_opp_bycirc_f = 11.0d0
vtilde_opp_bycirc_y_f = 11.0d0

!SET INITIAL GUESSES:
dist_S_m0 = dist_S_m
dist_S_y_m0 = dist_S_y_m
dist_L_m0 = dist_L_m
dist_L_y_m0 = dist_L_y_m
dist_L_opp_y_m0 = dist_L_opp_y_m
dist_A_m0 = dist_A_m
dist_S_f0 = dist_S_f
dist_S_y_f0 = dist_S_y_f
dist_L_f0 = dist_L_f
dist_L_y_f0 = dist_L_y_f
dist_L_opp_y_f0 = dist_L_opp_y_f
dist_A_f0 = dist_A_f
dist_L_opp_bycirc_f(:,i_nocirc,:) = dist_L_opp_f(:,i_healthy,:) * (1.0-frac_circ)
dist_L_opp_bycirc_f(:,i_circ,:) = dist_L_opp_f(:,i_healthy,:) * frac_circ
dist_L_opp_bycirc_f0 = dist_L_opp_bycirc_f
dist_L_opp_bycirc_y_f(:,i_nocirc,:) = dist_L_opp_y_f(:,i_healthy,:) * (1.0-frac_circ)
dist_L_opp_bycirc_y_f(:,i_circ,:) = dist_L_opp_y_f(:,i_healthy,:) * frac_circ
dist_L_opp_bycirc_y_f0 = dist_L_opp_bycirc_y_f

v_l_m0 = v_l_m
v_l_y_m0 = v_l_y_m
vtilde_l_opp_m0 = vtilde_l_opp_m
vtilde_l_opp_y_m0 = vtilde_l_opp_y_m
v_l_f0 = v_l_f
v_l_y_f0 = v_l_y_f
vtilde_l_opp_f0 = vtilde_l_opp_f
vtilde_l_opp_y_f0 = vtilde_l_opp_y_f
vtilde_opp_bycirc_f0 = vtilde_opp_bycirc_f
vtilde_opp_bycirc_y_f0 = vtilde_opp_bycirc_y_f

!MAIN LOOP:
iter = 0
do iter=1,max_iter

    !=======================================================
    ! PRELIMINARIES
    !=======================================================

    !Update distributions - singles:
    dist_S_m0 = ddd*dist_S_m + (1.0-ddd)*dist_S_m0
    dist_S_y_m0 = ddd*dist_S_y_m + (1.0-ddd)*dist_S_y_m0
    dist_S_f0 = ddd*dist_S_f + (1.0-ddd)*dist_S_f0
	dist_S_y_f0 = ddd*dist_S_y_f + (1.0-ddd)*dist_S_y_f0

	!Update distributions - married:
	dist_L_m0 = ddd*dist_L_m + (1.0-ddd)*dist_L_m0
	dist_L_y_m0 = ddd*dist_L_y_m + (1.0-ddd)*dist_L_y_m0
    dist_L_opp_m0 = ddd*dist_L_opp_m + (1.0-ddd)*dist_L_opp_m0
    dist_L_opp_y_m0 = ddd*dist_L_opp_y_m + (1.0-ddd)*dist_L_opp_y_m0
    dist_L_f0 = ddd*dist_L_f + (1.0-ddd)*dist_L_f0
	dist_L_y_f0 = ddd*dist_L_y_f + (1.0-ddd)*dist_L_y_f0
    dist_L_opp_f0 = ddd*dist_L_opp_f + (1.0-ddd)*dist_L_opp_f0
    dist_L_opp_y_f0 = ddd*dist_L_opp_y_f + (1.0-ddd)*dist_L_opp_y_f0
    dist_L_opp_bycirc_f0 = ddd*dist_L_opp_bycirc_f +  (1.0-ddd)*dist_L_opp_bycirc_f0
    dist_L_opp_bycirc_y_f0 = ddd*dist_L_opp_bycirc_y_f +  (1.0-ddd)*dist_L_opp_bycirc_y_f0

    !Update distributions - with symptoms:
    dist_A_m0 = ddd*dist_A_m + (1.0-ddd)*dist_A_m0
    dist_A_f0 = ddd*dist_A_f + (1.0-ddd)*dist_A_f0

	!Update value functions for long term relationships - ex ante:
    v_l_m0 = ddd1*v_l_m + (1.0-ddd1)*v_l_m0
    v_l_y_m0 = ddd1*v_l_y_m + (1.0-ddd1)*v_l_y_m0
    v_l_f0 = ddd1*v_l_f + (1.0-ddd1)*v_l_f0
	v_l_y_f0 = ddd1*v_l_y_f + (1.0-ddd1)*v_l_y_f0

	!Update value functions for long term relationships - matched:
    vtilde_l_opp_m0 = ddd1*vtilde_l_opp_m + (1.0-ddd1)*vtilde_l_opp_m0
    vtilde_l_opp_y_m0 = ddd1*vtilde_l_opp_y_m + (1.0-ddd1)*vtilde_l_opp_y_m0
    vtilde_l_opp_f0 = ddd1*vtilde_l_opp_f + (1.0-ddd1)*vtilde_l_opp_f0
    vtilde_l_opp_y_f0 = ddd1*vtilde_l_opp_y_f + (1.0-ddd1)*vtilde_l_opp_y_f0

    !=======================================================
    ! VALUE FUNCTIONS
    !=======================================================

	!Solve the value functions for males:
	do ii=1,ntypes_m
        !VALUE FUNCTIONS - SHORT TERM:
        !Abstinence (old agents):
        call abstinence(ii,i_male,i_healthy,ntypes_m,y_m,v_l_m0,vtilde_a_m(i_healthy,ii))
        call abstinence(ii,i_male,i_sick,ntypes_m,y_m,v_l_m0,vtilde_a_m(i_sick,ii))
        call abstinence(ii,i_male,i_treated,ntypes_m,y_m,v_l_m0,vtilde_a_m(i_treated,ii))
        !Abstinence (young agents):
        call abstinence_y(ii,i_male,i_healthy,ntypes_m,y_m,v_l_m0,v_l_y_m0,vtilde_a_y_m(i_healthy,ii))
        call abstinence_y(ii,i_male,i_sick,ntypes_m,y_m,v_l_m0,v_l_y_m0,vtilde_a_y_m(i_sick,ii))
        call abstinence_y(ii,i_male,i_treated,ntypes_m,y_m,v_l_m0,v_l_y_m0,vtilde_a_y_m(i_treated,ii))
        !Protected (old agents):
        call shortterm(ii,i_male,i_healthy,ntypes_m,y_m-t_p,p_m,upsilon_p_f,v_l_m0,vtilde_p_m(i_healthy,ii))
        call shortterm(ii,i_male,i_sick,ntypes_m,y_m-t_p,p_m,upsilon_p_f,v_l_m0,vtilde_p_m(i_sick,ii))
        call shortterm(ii,i_male,i_treated,ntypes_m,y_m-t_p,p_m,upsilon_p_f,v_l_m0,vtilde_p_m(i_treated,ii))
        !Protected (young agents):
        call shortterm_y(ii,i_male,i_healthy,ntypes_m,y_m-t_p,p_m,upsilon_p_f,v_l_m0,v_l_y_m0,vtilde_p_y_m(i_healthy,ii))
        call shortterm_y(ii,i_male,i_sick,ntypes_m,y_m-t_p,p_m,upsilon_p_f,v_l_m0,v_l_y_m0,vtilde_p_y_m(i_sick,ii))
        call shortterm_y(ii,i_male,i_treated,ntypes_m,y_m-t_p,p_m,upsilon_p_f,v_l_m0,v_l_y_m0,vtilde_p_y_m(i_treated,ii))
        !Unprotected (old agents):
        call shortterm(ii,i_male,i_healthy,ntypes_m,y_m-t_u,u_m,upsilon_u_f,v_l_m0,vtilde_u_m(i_healthy,ii))
        call shortterm(ii,i_male,i_sick,ntypes_m,y_m-t_u,u_m,upsilon_u_f,v_l_m0,vtilde_u_m(i_sick,ii))
        call shortterm(ii,i_male,i_treated,ntypes_m,y_m-t_u,u_m,upsilon_u_f,v_l_m0,vtilde_u_m(i_treated,ii))
        !Unprotected (young agents):
        call shortterm_y(ii,i_male,i_healthy,ntypes_m,y_m-t_u,u_m,upsilon_u_f,v_l_m0,v_l_y_m0,vtilde_u_y_m(i_healthy,ii))
        call shortterm_y(ii,i_male,i_sick,ntypes_m,y_m-t_u,u_m,upsilon_u_f,v_l_m0,v_l_y_m0,vtilde_u_y_m(i_sick,ii))
        call shortterm_y(ii,i_male,i_treated,ntypes_m,y_m-t_u,u_m,upsilon_u_f,v_l_m0,v_l_y_m0,vtilde_u_y_m(i_treated,ii))

        !SEARCH EFFORT - SHORT TERM:
        !Old agents:
        pi_p_m(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_p,n_status,vtilde_p_m(:,ii)-vtilde_a_m(:,ii),pi_storage(1,:))
        where (vtilde_p_m(:,ii)>vtilde_a_m(:,ii)) pi_p_m(:,ii) = pi_storage(1,:)
        pi_u_m(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_u,n_status,vtilde_u_m(:,ii)-vtilde_a_m(:,ii),pi_storage(1,:))
        where (vtilde_u_m(:,ii)>vtilde_a_m(:,ii)) pi_u_m(:,ii) = pi_storage(1,:)
        call interp_linear(1,ngrid,mygrid_con,search_ucon,n_status,vtilde_u_m(:,ii)-vtilde_p_m(:,ii),pi_storage)
        pi_both = pi_p_m(:,ii) + pi_u_m(:,ii)
        where (pi_both>1.0d0)
            pi_u_m(:,ii) = pi_storage(1,:)
            pi_p_m(:,ii) = 1.0 - pi_u_m(:,ii)
        end where
        !Young agents:
        pi_p_y_m(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_p,n_status,vtilde_p_y_m(:,ii)-vtilde_a_y_m(:,ii),pi_storage(1,:))
        where (vtilde_p_y_m(:,ii)>vtilde_a_y_m(:,ii)) pi_p_y_m(:,ii) = pi_storage(1,:)
        pi_u_y_m(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_u,n_status,vtilde_u_y_m(:,ii)-vtilde_a_y_m(:,ii),pi_storage(1,:))
        where (vtilde_u_y_m(:,ii)>vtilde_a_y_m(:,ii)) pi_u_y_m(:,ii) = pi_storage(1,:)
        call interp_linear(1,ngrid,mygrid_con,search_ucon,n_status,vtilde_u_y_m(:,ii)-vtilde_p_y_m(:,ii),pi_storage)
        pi_both = pi_p_y_m(:,ii) + pi_u_y_m(:,ii)
        where (pi_both>1.0d0)
            pi_u_y_m(:,ii) = pi_storage(1,:)
            pi_p_y_m(:,ii) = 1.0 - pi_u_y_m(:,ii)
        end where

        !EXPECTED UTILITY IN THE SHORT TERM MARKET:
        !Old agents:
        call cost_function(n_status,pi_p_m(:,ii),omega_p,kappa_p,b_p,thecost1)
        call cost_function(n_status,pi_u_m(:,ii),omega_u,kappa_u,b_u,thecost2)
        v_s_m(:,ii) = pi_p_m(:,ii)*vtilde_p_m(:,ii) + pi_u_m(:,ii)*vtilde_u_m(:,ii) + (1.0-pi_p_m(:,ii)-pi_u_m(:,ii))*vtilde_a_m(:,ii) - thecost1 - thecost2
        !Young agents:
        call cost_function(n_status,pi_p_y_m(:,ii),omega_p,kappa_p,b_p,thecost1)
        call cost_function(n_status,pi_u_y_m(:,ii),omega_u,kappa_u,b_u,thecost2)
        v_s_y_m(:,ii) = pi_p_y_m(:,ii)*vtilde_p_y_m(:,ii) + pi_u_y_m(:,ii)*vtilde_u_y_m(:,ii) + (1.0-pi_p_y_m(:,ii)-pi_u_y_m(:,ii))*vtilde_a_y_m(:,ii) - thecost1 - thecost2

        !VALUE FUNCTIONS IN THE LONG TERM MARKET:
        !Old agents:
        call longterm(ii,i_male,i_healthy,ntypes_m,y_m-t_l,u_m+l_m,frac_sick_f,frac_treated_f,v_l_m,vtilde_l_opp_m0,vtilde_l_m(i_healthy,ii),vtilde_l_opp_m(i_healthy,:,ii))
        call longterm(ii,i_male,i_sick,ntypes_m,y_m-t_l,u_m+l_m,frac_sick_f,frac_treated_f,v_l_m,vtilde_l_opp_m0,vtilde_l_m(i_sick,ii),vtilde_l_opp_m(i_sick,:,ii))
        call longterm(ii,i_male,i_treated,ntypes_m,y_m-t_l,u_m+l_m,frac_sick_f,frac_treated_f,v_l_m,vtilde_l_opp_m0,vtilde_l_m(i_treated,ii),vtilde_l_opp_m(i_treated,:,ii))
        !Young agents:
        call longterm_y(ii,i_male,i_healthy,ntypes_m,y_m-t_l,u_m+l_m,frac_sick_f,frac_treated_f,v_l_m,v_l_y_m,vtilde_l_opp_m0,vtilde_l_opp_y_m0,vtilde_l_y_m(i_healthy,ii),vtilde_l_opp_y_m(i_healthy,:,ii))
        call longterm_y(ii,i_male,i_sick,ntypes_m,y_m-t_l,u_m+l_m,frac_sick_f,frac_treated_f,v_l_m,v_l_y_m,vtilde_l_opp_m0,vtilde_l_opp_y_m0,vtilde_l_y_m(i_sick,ii),vtilde_l_opp_y_m(i_sick,:,ii))
        call longterm_y(ii,i_male,i_treated,ntypes_m,y_m-t_l,u_m+l_m,frac_sick_f,frac_treated_f,v_l_m,v_l_y_m,vtilde_l_opp_m0,vtilde_l_opp_y_m0,vtilde_l_y_m(i_treated,ii),vtilde_l_opp_y_m(i_treated,:,ii))

        !SEARCH EFFORT - LONG TERM:
        !Old agents:
        pi_l_m(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_l,n_status,vtilde_l_m(:,ii)-v_s_m(:,ii),pi_storage)
        where (vtilde_l_m(:,ii)>v_s_m(:,ii)) pi_l_m(:,ii) = pi_storage(1,:)
        !Young agents:
        pi_l_y_m(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_l,n_status,vtilde_l_y_m(:,ii)-v_s_y_m(:,ii),pi_storage)
        where (vtilde_l_y_m(:,ii)>v_s_y_m(:,ii)) pi_l_y_m(:,ii) = pi_storage(1,:)

        !EXPECTED UTILITY IN THE LONG TERM MARKET:
        !Old agents:
        call cost_function(n_status,pi_l_m(:,ii),omega_l,kappa_l,b_l,thecost1)
        v_l_m(:,ii) = pi_l_m(:,ii)*vtilde_l_m(:,ii) + (1.0-pi_l_m(:,ii))*v_s_m(:,ii) - thecost1
        !Young agents:
        call cost_function(n_status,pi_l_y_m(:,ii),omega_l,kappa_l,b_l,thecost1)
        v_l_y_m(:,ii) = pi_l_y_m(:,ii)*vtilde_l_y_m(:,ii) + (1.0-pi_l_y_m(:,ii))*v_s_y_m(:,ii) - thecost1

    enddo

    !Solve the value functions for females:
	do ii=1,ntypes_f
        !VALUE FUNCTIONS - SHORT TERM:
        !Abstinence (old agents):
        call abstinence(ii,i_female,i_healthy,ntypes_f,y_f,v_l_f0,vtilde_a_f(i_healthy,ii))
        call abstinence(ii,i_female,i_sick,ntypes_f,y_f,v_l_f0,vtilde_a_f(i_sick,ii))
        call abstinence(ii,i_female,i_treated,ntypes_f,y_f,v_l_f0,vtilde_a_f(i_treated,ii))
        !Abstinence (young agents):
        call abstinence_y(ii,i_female,i_healthy,ntypes_f,y_f,v_l_f0,v_l_y_f0,vtilde_a_y_f(i_healthy,ii))
        call abstinence_y(ii,i_female,i_sick,ntypes_f,y_f,v_l_f0,v_l_y_f0,vtilde_a_y_f(i_sick,ii))
        call abstinence_y(ii,i_female,i_treated,ntypes_f,y_f,v_l_f0,v_l_y_f0,vtilde_a_y_f(i_treated,ii))
        !Protected (old agents):
        call shortterm(ii,i_female,i_healthy,ntypes_f,y_f+t_p,p_f,upsilon_p_m,v_l_f0,vtilde_p_f(i_healthy,ii))
        call shortterm(ii,i_female,i_sick,ntypes_f,y_f+t_p,p_f,upsilon_p_m,v_l_f0,vtilde_p_f(i_sick,ii))
        call shortterm(ii,i_female,i_treated,ntypes_f,y_f+t_p,p_f,upsilon_p_m,v_l_f0,vtilde_p_f(i_treated,ii))
        !Protected (young agents):
        call shortterm_y(ii,i_female,i_healthy,ntypes_f,y_f+t_p,p_f,upsilon_p_m,v_l_f0,v_l_y_f0,vtilde_p_y_f(i_healthy,ii))
        call shortterm_y(ii,i_female,i_sick,ntypes_f,y_f+t_p,p_f,upsilon_p_m,v_l_f0,v_l_y_f0,vtilde_p_y_f(i_sick,ii))
        call shortterm_y(ii,i_female,i_treated,ntypes_f,y_f+t_p,p_f,upsilon_p_m,v_l_f0,v_l_y_f0,vtilde_p_y_f(i_treated,ii))
        !Unprotected (old agents):
        call shortterm(ii,i_female,i_healthy,ntypes_f,y_f+t_u,u_f,upsilon_u_m,v_l_f0,vtilde_u_f(i_healthy,ii))
        call shortterm(ii,i_female,i_sick,ntypes_f,y_f+t_u,u_f,upsilon_u_m,v_l_f0,vtilde_u_f(i_sick,ii))
        call shortterm(ii,i_female,i_treated,ntypes_f,y_f+t_u,u_f,upsilon_u_m,v_l_f0,vtilde_u_f(i_treated,ii))
        !Unprotected (young agents):
        call shortterm_y(ii,i_female,i_healthy,ntypes_f,y_f+t_u,u_f,upsilon_u_m,v_l_f0,v_l_y_f0,vtilde_u_y_f(i_healthy,ii))
        call shortterm_y(ii,i_female,i_sick,ntypes_f,y_f+t_u,u_f,upsilon_u_m,v_l_f0,v_l_y_f0,vtilde_u_y_f(i_sick,ii))
        call shortterm_y(ii,i_female,i_treated,ntypes_f,y_f+t_u,u_f,upsilon_u_m,v_l_f0,v_l_y_f0,vtilde_u_y_f(i_treated,ii))

        !SEARCH EFFORT - SHORT TERM:
        !Old agents:
        pi_p_f(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_p,n_status,vtilde_p_f(:,ii)-vtilde_a_f(:,ii),pi_storage(1,:))
        where (vtilde_p_f(:,ii)>vtilde_a_f(:,ii)) pi_p_f(:,ii) = pi_storage(1,:)
        pi_u_f(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_u,n_status,vtilde_u_f(:,ii)-vtilde_a_f(:,ii),pi_storage(1,:))
        where (vtilde_u_f(:,ii)>vtilde_a_f(:,ii)) pi_u_f(:,ii) = pi_storage(1,:)
        call interp_linear(1,ngrid,mygrid_con,search_ucon,n_status,vtilde_u_f(:,ii)-vtilde_p_f(:,ii),pi_storage)
        pi_both = pi_p_f(:,ii) + pi_u_f(:,ii)
        where (pi_both>1.0d0)
            pi_u_f(:,ii) = pi_storage(1,:)
            pi_p_f(:,ii) = 1.0 - pi_u_f(:,ii)
        end where
        !Young agents:
        pi_p_y_f(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_p,n_status,vtilde_p_y_f(:,ii)-vtilde_a_y_f(:,ii),pi_storage(1,:))
        where (vtilde_p_y_f(:,ii)>vtilde_a_y_f(:,ii)) pi_p_y_f(:,ii) = pi_storage(1,:)
        pi_u_y_f(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_u,n_status,vtilde_u_y_f(:,ii)-vtilde_a_y_f(:,ii),pi_storage(1,:))
        where (vtilde_u_y_f(:,ii)>vtilde_a_y_f(:,ii)) pi_u_y_f(:,ii) = pi_storage(1,:)
        call interp_linear(1,ngrid,mygrid_con,search_ucon,n_status,vtilde_u_y_f(:,ii)-vtilde_p_y_f(:,ii),pi_storage)
        pi_both = pi_p_y_f(:,ii) + pi_u_y_f(:,ii)
        where (pi_both>1.0d0)
            pi_u_y_f(:,ii) = pi_storage(1,:)
            pi_p_y_f(:,ii) = 1.0 - pi_u_y_f(:,ii)
        end where

        !EXPECTED UTILITY IN THE SHORT TERM MARKET:
        !Old agents:
        call cost_function(n_status,pi_p_f(:,ii),omega_p,kappa_p,b_p,thecost1)
        call cost_function(n_status,pi_u_f(:,ii),omega_u,kappa_u,b_u,thecost2)
        v_s_f(:,ii) = pi_p_f(:,ii)*vtilde_p_f(:,ii) + pi_u_f(:,ii)*vtilde_u_f(:,ii) + (1.0-pi_p_f(:,ii)-pi_u_f(:,ii))*vtilde_a_f(:,ii) - thecost1 - thecost2
        !Young agents:
        call cost_function(n_status,pi_p_y_f(:,ii),omega_p,kappa_p,b_p,thecost1)
        call cost_function(n_status,pi_u_y_f(:,ii),omega_u,kappa_u,b_u,thecost2)
        v_s_y_f(:,ii) = pi_p_y_f(:,ii)*vtilde_p_y_f(:,ii) + pi_u_y_f(:,ii)*vtilde_u_y_f(:,ii) + (1.0-pi_p_y_f(:,ii)-pi_u_y_f(:,ii))*vtilde_a_y_f(:,ii) - thecost1 - thecost2

        !VALUE FUNCTIONS IN THE LONG TERM MARKET:
        !Old agents:
        call longterm(ii,i_female,i_healthy,ntypes_f,y_f+t_l,u_f+l_f,frac_sick_m,frac_treated_m,v_l_f,vtilde_l_opp_f0,vtilde_l_f(i_healthy,ii),vtilde_l_opp_f(i_healthy,:,ii))
        if (do_circ==1) then
            call longterm_f(ii,i_female,i_sick,ntypes_f,y_f+t_l,u_f+l_f,frac_healthy_nocirc_m,frac_healthy_circ_m,frac_sick_m,frac_treated_m,v_l_f,vtilde_l_opp_f0,vtilde_opp_bycirc_f0,vtilde_l_f(i_sick,ii),vtilde_l_opp_f(i_sick,:,ii),vtilde_opp_bycirc_f(i_sick,:,ii))
            call longterm_f(ii,i_female,i_treated,ntypes_f,y_f+t_l,u_f+l_f,frac_healthy_nocirc_m,frac_healthy_circ_m,frac_sick_m,frac_treated_m,v_l_f,vtilde_l_opp_f0,vtilde_opp_bycirc_f0,vtilde_l_f(i_treated,ii),vtilde_l_opp_f(i_treated,:,ii),vtilde_opp_bycirc_f(i_treated,:,ii))
        else
            call longterm(ii,i_female,i_sick,ntypes_f,y_f+t_l,u_f+l_f,frac_sick_m,frac_treated_m,v_l_f,vtilde_l_opp_f0,vtilde_l_f(i_sick,ii),vtilde_l_opp_f(i_sick,:,ii))
            call longterm(ii,i_female,i_treated,ntypes_f,y_f+t_l,u_f+l_f,frac_sick_m,frac_treated_m,v_l_f,vtilde_l_opp_f0,vtilde_l_f(i_treated,ii),vtilde_l_opp_f(i_treated,:,ii))
        endif
        !Young agents:
        call longterm_y(ii,i_female,i_healthy,ntypes_f,y_f+t_l,u_f+l_f,frac_sick_m,frac_treated_m,v_l_f,v_l_y_f,vtilde_l_opp_f0,vtilde_l_opp_y_f0,vtilde_l_y_f(i_healthy,ii),vtilde_l_opp_y_f(i_healthy,:,ii))
        if (do_circ==1) then
            call longterm_y_f(ii,i_female,i_sick,ntypes_f,y_f+t_l,u_f+l_f,frac_healthy_nocirc_m,frac_healthy_circ_m,frac_sick_m,frac_treated_m,v_l_f,v_l_y_f,vtilde_l_opp_f0,vtilde_l_opp_y_f0,vtilde_opp_bycirc_f0,vtilde_opp_bycirc_y_f0,vtilde_l_y_f(i_sick,ii),vtilde_l_opp_y_f(i_sick,:,ii),vtilde_opp_bycirc_y_f(i_sick,:,ii))
            call longterm_y_f(ii,i_female,i_treated,ntypes_f,y_f+t_l,u_f+l_f,frac_healthy_nocirc_m,frac_healthy_circ_m,frac_sick_m,frac_treated_m,v_l_f,v_l_y_f,vtilde_l_opp_f0,vtilde_l_opp_y_f0,vtilde_opp_bycirc_f0,vtilde_opp_bycirc_y_f0,vtilde_l_y_f(i_treated,ii),vtilde_l_opp_y_f(i_treated,:,ii),vtilde_opp_bycirc_y_f(i_treated,:,ii))
        else
            call longterm_y(ii,i_female,i_sick,ntypes_f,y_f+t_l,u_f+l_f,frac_sick_m,frac_treated_m,v_l_f,v_l_y_f,vtilde_l_opp_f0,vtilde_l_opp_y_f0,vtilde_l_y_f(i_sick,ii),vtilde_l_opp_y_f(i_sick,:,ii))
            call longterm_y(ii,i_female,i_treated,ntypes_f,y_f+t_l,u_f+l_f,frac_sick_m,frac_treated_m,v_l_f,v_l_y_f,vtilde_l_opp_f0,vtilde_l_opp_y_f0,vtilde_l_y_f(i_treated,ii),vtilde_l_opp_y_f(i_treated,:,ii))
        endif

        !SEARCH EFFORT - LONG TERM:
        !Old agents:
        pi_l_f(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_l,n_status,vtilde_l_f(:,ii)-v_s_f(:,ii),pi_storage)
        where (vtilde_l_f(:,ii)>v_s_f(:,ii)) pi_l_f(:,ii) = pi_storage(1,:)
        !Young agents:
        pi_l_y_f(:,ii) = 0.0d0
        call interp_linear(1,ngrid,mygrid,search_l,n_status,vtilde_l_y_f(:,ii)-v_s_y_f(:,ii),pi_storage)
        where (vtilde_l_y_f(:,ii)>v_s_y_f(:,ii)) pi_l_y_f(:,ii) = pi_storage(1,:)

        !EXPECTED UTILITY IN THE LONG TERM MARKET:
        !Old agents:
        call cost_function(n_status,pi_l_f(:,ii),omega_l,kappa_l,b_l,thecost1)
        v_l_f(:,ii) = pi_l_f(:,ii)*vtilde_l_f(:,ii) + (1.0-pi_l_f(:,ii))*v_s_f(:,ii) - thecost1
        !Young agents:
        call cost_function(n_status,pi_l_y_f(:,ii),omega_l,kappa_l,b_l,thecost1)
        v_l_y_f(:,ii) = pi_l_y_f(:,ii)*vtilde_l_y_f(:,ii) + (1.0-pi_l_y_f(:,ii))*v_s_y_f(:,ii) - thecost1

    enddo

    !=======================================================
    ! DISTRIBUTIONS
    !=======================================================

    !Distributions for males:
    do ii=1,ntypes_m
        !SINGLES:
        !Before changing discount factors or treatment status - old agents:
        dist_S_m_pr(i_healthy,ii) = (1.0-delta) * ( &
                                    (1.0-pi_l_m(i_healthy,ii))*dist_S_m0(i_healthy,ii)*( 1.0-pi_p_m(i_healthy,ii)-pi_u_m(i_healthy,ii) + pi_p_m(i_healthy,ii)*(1.0-upsilon_p_f*chi_m(ii)) + pi_u_m(i_healthy,ii)*(1.0-upsilon_u_f*chi_m(ii)) ) + &
                                    sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_m(i_healthy,ii)*dist_S_m0(i_healthy,ii) ) ) &
                                    )
        dist_S_m_pr(i_sick,ii)    = (1.0-delta) * ( &
                                    (1.0-pi_l_m(i_healthy,ii))*dist_S_m0(i_healthy,ii)*( pi_p_m(i_healthy,ii)*upsilon_p_f*chi_m(ii) + pi_u_m(i_healthy,ii)*upsilon_u_f*chi_m(ii) ) + &
                                    (1.0-alpha)*(1.0-pi_l_m(i_sick,ii))*dist_S_m0(i_sick,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii)*( dist_L_opp_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_m(i_healthy,ii)*dist_S_m0(i_healthy,ii) ) ) + &
                                    (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_m0(i_sick,:,ii) + frac_l_f_vec*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) ) ) &
                                    )
        dist_S_m_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                    (1.0-pi_l_m(i_treated,ii))*dist_S_m0(i_treated,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_m0(i_treated,:,ii) + frac_l_f_vec*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) ) ) &
                                    )
        !Before changing discount factors or treatment status - young agents:
        dist_S_y_m_pr(i_healthy,ii) = (1.0-delta) * ( &
                                      (1.0-pi_l_y_m(i_healthy,ii))*dist_S_y_m0(i_healthy,ii)*( 1.0-pi_p_y_m(i_healthy,ii)-pi_u_y_m(i_healthy,ii) + pi_p_y_m(i_healthy,ii)*(1.0-upsilon_p_f*chi_m(ii)) + pi_u_y_m(i_healthy,ii)*(1.0-upsilon_u_f*chi_m(ii)) ) + &
                                      sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_y_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_y_m(i_healthy,ii)*dist_S_y_m0(i_healthy,ii) ) ) &
                                      ) + mu_m(ii)
        dist_S_y_m_pr(i_sick,ii)    = (1.0-delta) * ( &
                                      (1.0-pi_l_y_m(i_healthy,ii))*dist_S_y_m0(i_healthy,ii)*( pi_p_y_m(i_healthy,ii)*upsilon_p_f*chi_m(ii) + pi_u_y_m(i_healthy,ii)*upsilon_u_f*chi_m(ii) ) + &
                                      (1.0-alpha)*(1.0-pi_l_y_m(i_sick,ii))*dist_S_y_m0(i_sick,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii)*( dist_L_opp_y_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_y_m(i_healthy,ii)*dist_S_y_m0(i_healthy,ii) ) ) + &
                                      (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_m0(i_sick,:,ii) + frac_l_f_vec*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) ) ) &
                                      )
        dist_S_y_m_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                      (1.0-pi_l_y_m(i_treated,ii))*dist_S_y_m0(i_treated,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_m0(i_treated,:,ii) + frac_l_f_vec*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) ) ) &
                                      )
        !Changing the discount factors:
        dist_S_m_prpr(:,ii) = dist_S_m_pr(:,ii) + eta*dist_S_y_m_pr(:,ii)
        dist_S_y_m_prpr(:,ii) = (1.0-eta)*dist_S_y_m_pr(:,ii)
        !Changing treatment status:
        dist_S_m(i_healthy,ii) = dist_S_m_prpr(i_healthy,ii)
        dist_S_m(i_sick,ii)    = dist_S_m_prpr(i_sick,ii)*(1.0-q)
        dist_S_m(i_treated,ii) = dist_S_m_prpr(i_sick,ii)*q + dist_S_m_prpr(i_treated,ii)
        dist_S_y_m(i_healthy,ii) = dist_S_y_m_prpr(i_healthy,ii)
        dist_S_y_m(i_sick,ii)    = dist_S_y_m_prpr(i_sick,ii)*(1.0-q)
        dist_S_y_m(i_treated,ii) = dist_S_y_m_prpr(i_sick,ii)*q + dist_S_y_m_prpr(i_treated,ii)

        !MARRIED:
        !Before changing discount factors or treatment status - old agents:
        dist_L_opp_m_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_m(i_healthy,ii)*dist_S_m0(i_healthy,ii) )
        dist_L_opp_m_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_m*( dist_L_opp_m0(i_sick,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) )
        dist_L_opp_m_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_m0(i_sick,i_sick,ii) + frac_sick_f*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) )
        dist_L_opp_m_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_m0(i_sick,i_treated,ii) + frac_treated_f*pi_l_m(i_sick,ii)*dist_S_m0(i_sick,ii) )
        dist_L_opp_m_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_m_treat*( dist_L_opp_m0(i_treated,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) )
        dist_L_opp_m_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_m0(i_treated,i_sick,ii) + frac_sick_f*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) )
        dist_L_opp_m_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_m0(i_treated,i_treated,ii) + frac_treated_f*pi_l_m(i_treated,ii)*dist_S_m0(i_treated,ii) )
        !Before changing discount factors or treatment status - young agents:
        dist_L_opp_y_m_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_female,i_unprotected))*chi_m(ii))*( dist_L_opp_y_m0(i_healthy,:,ii) + frac_l_f_vec*pi_l_y_m(i_healthy,ii)*dist_S_y_m0(i_healthy,ii) )
        dist_L_opp_y_m_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_m*( dist_L_opp_y_m0(i_sick,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) )
        dist_L_opp_y_m_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_y_m0(i_sick,i_sick,ii) + frac_sick_f*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) )
        dist_L_opp_y_m_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_y_m0(i_sick,i_treated,ii) + frac_treated_f*pi_l_y_m(i_sick,ii)*dist_S_y_m0(i_sick,ii) )
        dist_L_opp_y_m_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_m_treat*( dist_L_opp_y_m0(i_treated,i_healthy,ii) + (1.0-frac_sick_f-frac_treated_f)*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) )
        dist_L_opp_y_m_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_y_m0(i_treated,i_sick,ii) + frac_sick_f*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) )
        dist_L_opp_y_m_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_y_m0(i_treated,i_treated,ii) + frac_treated_f*pi_l_y_m(i_treated,ii)*dist_S_y_m0(i_treated,ii) )
        !Changing the discount factors:
        dist_L_opp_m_prpr(:,:,ii) = dist_L_opp_m_pr(:,:,ii) + eta*dist_L_opp_y_m_pr(:,:,ii)
        dist_L_opp_y_m_prpr(:,:,ii) = (1.0-eta)*dist_L_opp_y_m_pr(:,:,ii)
        !Chaging treatment status - old agents:
        dist_L_opp_m(i_healthy,i_healthy,ii) = dist_L_opp_m_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_m(i_healthy,i_sick,ii)    = dist_L_opp_m_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_m(i_healthy,i_treated,ii) = dist_L_opp_m_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_m_prpr(i_healthy,i_treated,ii)
        dist_L_opp_m(i_sick,i_healthy,ii)    = dist_L_opp_m_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_m(i_sick,i_sick,ii)       = dist_L_opp_m_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_m(i_sick,i_treated,ii)    = dist_L_opp_m_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_m_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_m(i_treated,i_healthy,ii) = dist_L_opp_m_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_m_prpr(i_treated,i_healthy,ii)
        dist_L_opp_m(i_treated,i_sick,ii)    = dist_L_opp_m_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_m_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_m(i_treated,i_treated,ii) = dist_L_opp_m_prpr(i_treated,i_treated,ii) + dist_L_opp_m_prpr(i_sick,i_treated,ii)*q + dist_L_opp_m_prpr(i_treated,i_sick,ii)*q + dist_L_opp_m_prpr(i_sick,i_sick,ii)*q*q
        !Changing treatment status - young agents:
        dist_L_opp_y_m(i_healthy,i_healthy,ii) = dist_L_opp_y_m_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_y_m(i_healthy,i_sick,ii)    = dist_L_opp_y_m_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_y_m(i_healthy,i_treated,ii) = dist_L_opp_y_m_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_y_m_prpr(i_healthy,i_treated,ii)
        dist_L_opp_y_m(i_sick,i_healthy,ii)    = dist_L_opp_y_m_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_y_m(i_sick,i_sick,ii)       = dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_y_m(i_sick,i_treated,ii)    = dist_L_opp_y_m_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_y_m(i_treated,i_healthy,ii) = dist_L_opp_y_m_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_y_m_prpr(i_treated,i_healthy,ii)
        dist_L_opp_y_m(i_treated,i_sick,ii)    = dist_L_opp_y_m_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_y_m(i_treated,i_treated,ii) = dist_L_opp_y_m_prpr(i_treated,i_treated,ii) + dist_L_opp_y_m_prpr(i_sick,i_treated,ii)*q + dist_L_opp_y_m_prpr(i_treated,i_sick,ii)*q + dist_L_opp_y_m_prpr(i_sick,i_sick,ii)*q*q
        !Consolidating across partner's type:
        dist_L_m(:,ii) = sum(dist_L_opp_m(:,:,ii),dim=2)
        dist_L_y_m(:,ii) = sum(dist_L_opp_y_m(:,:,ii),dim=2)

        !WITH SYMPTOMS:
        dist_A_m(:,ii) = (1.0-delta)*( (1.0-delta2)*dist_A_m0(:,ii) + (dist_S_m0(:,ii) + dist_S_y_m0(:,ii) + dist_L_m(:,ii) + dist_L_y_m(:,ii))*alpha_vec )

    enddo

    !Distributions for females:
    do ii=1,ntypes_f
        !SINGLES:
        !Before changing discount factors or treatment status - old agents:
        dist_S_f_pr(i_healthy,ii) = (1.0-delta) * ( &
                                    (1.0-pi_l_f(i_healthy,ii))*dist_S_f0(i_healthy,ii)*( 1.0-pi_p_f(i_healthy,ii)-pi_u_f(i_healthy,ii) + pi_p_f(i_healthy,ii)*(1.0-upsilon_p_m*chi_f(ii)) + pi_u_f(i_healthy,ii)*(1.0-upsilon_u_m*chi_f(ii)) ) + &
                                    sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_f(i_healthy,ii)*dist_S_f0(i_healthy,ii) ) ) &
                                    )
        dist_S_f_pr(i_sick,ii)    = (1.0-delta) * ( &
                                    (1.0-pi_l_f(i_healthy,ii))*dist_S_f0(i_healthy,ii)*( pi_p_f(i_healthy,ii)*upsilon_p_m*chi_f(ii) + pi_u_f(i_healthy,ii)*upsilon_u_m*chi_f(ii) ) + &
                                    (1.0-alpha)*(1.0-pi_l_f(i_sick,ii))*dist_S_f0(i_sick,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii)*( dist_L_opp_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_f(i_healthy,ii)*dist_S_f0(i_healthy,ii) ) ) + &
                                    (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_f0(i_sick,:,ii) + frac_l_m_vec*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) ) ) &
                                    )
        dist_S_f_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                    (1.0-pi_l_f(i_treated,ii))*dist_S_f0(i_treated,ii) + &
                                    sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_f0(i_treated,:,ii) + frac_l_m_vec*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) ) ) &
                                    )
        !Before changing discount factors or treatment status - young agents:
        dist_S_y_f_pr(i_healthy,ii) = (1.0-delta) * ( &
                                      (1.0-pi_l_y_f(i_healthy,ii))*dist_S_y_f0(i_healthy,ii)*( 1.0-pi_p_y_f(i_healthy,ii)-pi_u_y_f(i_healthy,ii) + pi_p_y_f(i_healthy,ii)*(1.0-upsilon_p_m*chi_f(ii)) + pi_u_y_f(i_healthy,ii)*(1.0-upsilon_u_m*chi_f(ii)) ) + &
                                      sum( (1.0-(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec))*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_y_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_y_f(i_healthy,ii)*dist_S_y_f0(i_healthy,ii) ) ) &
                                      ) + mu_f(ii)
        dist_S_y_f_pr(i_sick,ii)    = (1.0-delta) * ( &
                                      (1.0-pi_l_y_f(i_healthy,ii))*dist_S_y_f0(i_healthy,ii)*( pi_p_y_f(i_healthy,ii)*upsilon_p_m*chi_f(ii) + pi_u_y_f(i_healthy,ii)*upsilon_u_m*chi_f(ii) ) + &
                                      (1.0-alpha)*(1.0-pi_l_y_f(i_sick,ii))*dist_S_y_f0(i_sick,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii)*( dist_L_opp_y_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_y_f(i_healthy,ii)*dist_S_y_f0(i_healthy,ii) ) ) + &
                                      (1.0-alpha)*sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_f0(i_sick,:,ii) + frac_l_m_vec*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) ) ) &
                                      )
        dist_S_y_f_pr(i_treated,ii) = (1.0-delta)*(1.0-alpha_treat) * ( &
                                      (1.0-pi_l_y_f(i_treated,ii))*dist_S_y_f0(i_treated,ii) + &
                                      sum( (1.0-(1.0-delta)*(1.0-alpha_vec)*(1.0-epsilon))*( dist_L_opp_y_f0(i_treated,:,ii) + frac_l_m_vec*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) ) ) &
                                      )
        !Changing the discount factors:
        dist_S_f_prpr(:,ii) = dist_S_f_pr(:,ii) + eta*dist_S_y_f_pr(:,ii)
        dist_S_y_f_prpr(:,ii) = (1.0-eta)*dist_S_y_f_pr(:,ii)
        !Changing treatment status:
        dist_S_f(i_healthy,ii) = dist_S_f_prpr(i_healthy,ii)
        dist_S_f(i_sick,ii)    = dist_S_f_prpr(i_sick,ii)*(1.0-q)
        dist_S_f(i_treated,ii) = dist_S_f_prpr(i_sick,ii)*q + dist_S_f_prpr(i_treated,ii)
        dist_S_y_f(i_healthy,ii) = dist_S_y_f_prpr(i_healthy,ii)
        dist_S_y_f(i_sick,ii)    = dist_S_y_f_prpr(i_sick,ii)*(1.0-q)
        dist_S_y_f(i_treated,ii) = dist_S_y_f_prpr(i_sick,ii)*q + dist_S_y_f_prpr(i_treated,ii)

        !MARRIED:
        !Before changing discount factors or treatment status - old agents:
        dist_L_opp_f_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_f(i_healthy,ii)*dist_S_f0(i_healthy,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_f_pr(i_sick,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_bycirc_f0(i_sick,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
            dist_L_opp_bycirc_f_pr(i_sick,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-(1.0-gammau_f)*chi_circ)*( dist_L_opp_bycirc_f0(i_sick,i_circ,ii) + frac_healthy_circ_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
            dist_L_opp_f_pr(i_sick,i_healthy,ii) = dist_L_opp_bycirc_f_pr(i_sick,i_nocirc,ii) + dist_L_opp_bycirc_f_pr(i_sick,i_circ,ii)
        else
            dist_L_opp_f_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_f0(i_sick,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
        endif
        dist_L_opp_f_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_f0(i_sick,i_sick,ii) + frac_sick_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
        dist_L_opp_f_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_f0(i_sick,i_treated,ii) + frac_treated_m*pi_l_f(i_sick,ii)*dist_S_f0(i_sick,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_f_pr(i_treated,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_bycirc_f0(i_treated,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
            dist_L_opp_bycirc_f_pr(i_treated,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-(1.0-gammau_f_treat)*chi_circ)*( dist_L_opp_bycirc_f0(i_treated,i_circ,ii) + frac_healthy_circ_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
            dist_L_opp_f_pr(i_treated,i_healthy,ii) = dist_L_opp_bycirc_f_pr(i_treated,i_nocirc,ii) + dist_L_opp_bycirc_f_pr(i_treated,i_circ,ii)
        else
            dist_L_opp_f_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_f0(i_treated,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
        endif
        dist_L_opp_f_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_f0(i_treated,i_sick,ii) + frac_sick_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
        dist_L_opp_f_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_f0(i_treated,i_treated,ii) + frac_treated_m*pi_l_f(i_treated,ii)*dist_S_f0(i_treated,ii) )
        !Before changing discount factors or treatment status - young agents:
        dist_L_opp_y_f_pr(i_healthy,:,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_vec)*(1.0-(1.0-gamma_vec(:,i_male,i_unprotected))*chi_f(ii))*( dist_L_opp_y_f0(i_healthy,:,ii) + frac_l_m_vec*pi_l_y_f(i_healthy,ii)*dist_S_y_f0(i_healthy,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_y_f_pr(i_sick,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_bycirc_y_f0(i_sick,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
            dist_L_opp_bycirc_y_f_pr(i_sick,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-(1.0-gammau_f)*chi_circ)*( dist_L_opp_bycirc_y_f0(i_sick,i_circ,ii) + frac_healthy_circ_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
            dist_L_opp_y_f_pr(i_sick,i_healthy,ii) = dist_L_opp_bycirc_y_f_pr(i_sick,i_nocirc,ii) + dist_L_opp_bycirc_y_f_pr(i_sick,i_circ,ii)
        else
            dist_L_opp_y_f_pr(i_sick,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*gammau_f*( dist_L_opp_y_f0(i_sick,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
        endif
        dist_L_opp_y_f_pr(i_sick,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha)*( dist_L_opp_y_f0(i_sick,i_sick,ii) + frac_sick_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
        dist_L_opp_y_f_pr(i_sick,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha)*(1.0-alpha_treat)*( dist_L_opp_y_f0(i_sick,i_treated,ii) + frac_treated_m*pi_l_y_f(i_sick,ii)*dist_S_y_f0(i_sick,ii) )
        if (do_circ==1) then
            dist_L_opp_bycirc_y_f_pr(i_treated,i_nocirc,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_bycirc_y_f0(i_treated,i_nocirc,ii) + frac_healthy_nocirc_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
            dist_L_opp_bycirc_y_f_pr(i_treated,i_circ,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-(1.0-gammau_f_treat)*chi_circ)*( dist_L_opp_bycirc_y_f0(i_treated,i_circ,ii) + frac_healthy_circ_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
            dist_L_opp_y_f_pr(i_treated,i_healthy,ii) = dist_L_opp_bycirc_y_f_pr(i_treated,i_nocirc,ii) + dist_L_opp_bycirc_y_f_pr(i_treated,i_circ,ii)
        else
            dist_L_opp_y_f_pr(i_treated,i_healthy,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*gammau_f_treat*( dist_L_opp_y_f0(i_treated,i_healthy,ii) + (1.0-frac_sick_m-frac_treated_m)*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
        endif
        dist_L_opp_y_f_pr(i_treated,i_sick,ii)    = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha)*( dist_L_opp_y_f0(i_treated,i_sick,ii) + frac_sick_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
        dist_L_opp_y_f_pr(i_treated,i_treated,ii) = (1.0-delta)*(1.0-delta)*(1.0-epsilon)*(1.0-alpha_treat)*(1.0-alpha_treat)*( dist_L_opp_y_f0(i_treated,i_treated,ii) + frac_treated_m*pi_l_y_f(i_treated,ii)*dist_S_y_f0(i_treated,ii) )
        !Changing the discount factors:
        dist_L_opp_f_prpr(:,:,ii) = dist_L_opp_f_pr(:,:,ii) + eta*dist_L_opp_y_f_pr(:,:,ii)
        dist_L_opp_y_f_prpr(:,:,ii) = (1.0-eta)*dist_L_opp_y_f_pr(:,:,ii)
        !Chaging treatment status - old agents:
        dist_L_opp_f(i_healthy,i_healthy,ii) = dist_L_opp_f_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_f(i_healthy,i_sick,ii)    = dist_L_opp_f_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_f(i_healthy,i_treated,ii) = dist_L_opp_f_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_f_prpr(i_healthy,i_treated,ii)
        dist_L_opp_f(i_sick,i_healthy,ii)    = dist_L_opp_f_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_f(i_sick,i_sick,ii)       = dist_L_opp_f_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_f(i_sick,i_treated,ii)    = dist_L_opp_f_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_f_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_f(i_treated,i_healthy,ii) = dist_L_opp_f_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_f_prpr(i_treated,i_healthy,ii)
        dist_L_opp_f(i_treated,i_sick,ii)    = dist_L_opp_f_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_f_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_f(i_treated,i_treated,ii) = dist_L_opp_f_prpr(i_treated,i_treated,ii) + dist_L_opp_f_prpr(i_sick,i_treated,ii)*q + dist_L_opp_f_prpr(i_treated,i_sick,ii)*q + dist_L_opp_f_prpr(i_sick,i_sick,ii)*q*q
        !Changing treatment status - young agents:
        dist_L_opp_y_f(i_healthy,i_healthy,ii) = dist_L_opp_y_f_prpr(i_healthy,i_healthy,ii)
        dist_L_opp_y_f(i_healthy,i_sick,ii)    = dist_L_opp_y_f_prpr(i_healthy,i_sick,ii)*(1.0-q)
        dist_L_opp_y_f(i_healthy,i_treated,ii) = dist_L_opp_y_f_prpr(i_healthy,i_sick,ii)*q + dist_L_opp_y_f_prpr(i_healthy,i_treated,ii)
        dist_L_opp_y_f(i_sick,i_healthy,ii)    = dist_L_opp_y_f_prpr(i_sick,i_healthy,ii)*(1.0-q)
        dist_L_opp_y_f(i_sick,i_sick,ii)       = dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*(1.0-q)*(1.0-q)
        dist_L_opp_y_f(i_sick,i_treated,ii)    = dist_L_opp_y_f_prpr(i_sick,i_treated,ii)*(1.0-q) + dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*(1.0-q)*q
        dist_L_opp_y_f(i_treated,i_healthy,ii) = dist_L_opp_y_f_prpr(i_sick,i_healthy,ii)*q + dist_L_opp_y_f_prpr(i_treated,i_healthy,ii)
        dist_L_opp_y_f(i_treated,i_sick,ii)    = dist_L_opp_y_f_prpr(i_treated,i_sick,ii)*(1.0-q) + dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*q*(1.0-q)
        dist_L_opp_y_f(i_treated,i_treated,ii) = dist_L_opp_y_f_prpr(i_treated,i_treated,ii) + dist_L_opp_y_f_prpr(i_sick,i_treated,ii)*q + dist_L_opp_y_f_prpr(i_treated,i_sick,ii)*q + dist_L_opp_y_f_prpr(i_sick,i_sick,ii)*q*q
        !Change discount factors and treatment status for distributions by circumcision:
        if (do_circ==1) then
            !Change the discount factors:
            dist_L_opp_bycirc_f_prpr(:,:,ii) = dist_L_opp_bycirc_f_pr(:,:,ii) + eta*dist_L_opp_bycirc_y_f_pr(:,:,ii)
            dist_L_opp_bycirc_y_f_prpr(:,:,ii) = (1.0-eta)*dist_L_opp_bycirc_y_f_pr(:,:,ii)
            !Change the treatment status:
            dist_L_opp_bycirc_f(i_sick,:,ii) = dist_L_opp_bycirc_f_prpr(i_sick,:,ii)*(1.0-q)
            dist_L_opp_bycirc_f(i_treated,:,ii) = dist_L_opp_bycirc_f_prpr(i_sick,:,ii)*q + dist_L_opp_bycirc_f_prpr(i_treated,:,ii)
            dist_L_opp_bycirc_y_f(i_sick,:,ii) = dist_L_opp_bycirc_y_f_prpr(i_sick,:,ii)*(1.0-q)
            dist_L_opp_bycirc_y_f(i_treated,:,ii) = dist_L_opp_bycirc_y_f_prpr(i_sick,:,ii)*q + dist_L_opp_bycirc_y_f_prpr(i_treated,:,ii)
        endif
        !Consolidating across partner's type:
        dist_L_f(:,ii) = sum(dist_L_opp_f(:,:,ii),dim=2)
        dist_L_y_f(:,ii) = sum(dist_L_opp_y_f(:,:,ii),dim=2)

        !WITH SYMPTOMS:
        dist_A_f(:,ii) = (1.0-delta)*( (1.0-delta2)*dist_A_f0(:,ii) + (dist_S_f0(:,ii) + dist_S_y_f0(:,ii) + dist_L_f(:,ii) + dist_L_y_f(:,ii))*alpha_vec )

    enddo

    !=======================================================
    ! AGGREGATES
    !=======================================================

    !These are kept constant in the small field experiment.
    !No updates!!!

    !=======================================================
    ! CLOSING THIS ITERATION
    !=======================================================

    !Compute the difference between this iteration and the last:
    norm_vec(1)  = maxval(abs(dist_S_m - dist_S_m0))
    norm_vec(2)  = maxval(abs(dist_S_y_m - dist_S_y_m0))
    norm_vec(3)  = maxval(abs(dist_S_f - dist_S_f0))
    norm_vec(4)  = maxval(abs(dist_S_y_f - dist_S_y_f0))
    norm_vec(5)  = maxval(abs(dist_L_m - dist_L_m0))
    norm_vec(6)  = maxval(abs(dist_L_y_m - dist_L_y_m0))
    norm_vec(7)  = maxval(abs(dist_L_opp_m - dist_L_opp_m0))
    norm_vec(8)  = maxval(abs(dist_L_opp_y_m - dist_L_opp_y_m0))
    norm_vec(9)  = maxval(abs(dist_L_f - dist_L_f0))
    norm_vec(10) = maxval(abs(dist_L_y_f - dist_L_y_f0))
    norm_vec(11) = maxval(abs(dist_L_opp_f - dist_L_opp_f0))
    norm_vec(12) = maxval(abs(dist_L_opp_y_f - dist_L_opp_y_f0))
    norm_vec(13) = maxval(abs(dist_A_m - dist_A_m0))
    norm_vec(14) = maxval(abs(dist_A_f - dist_A_f0))
    norm_vec(15) = maxval(abs(v_l_m - v_l_m0))
    norm_vec(16) = maxval(abs(v_l_y_m - v_l_y_m0))
    norm_vec(17) = maxval(abs(v_l_f - v_l_f0))
    norm_vec(18) = maxval(abs(v_l_y_f - v_l_y_f0))
    norm_vec(19) = maxval(abs(vtilde_l_opp_m - vtilde_l_opp_m0))
    norm_vec(20) = maxval(abs(vtilde_l_opp_y_m - vtilde_l_opp_y_m0))
    norm_vec(21) = maxval(abs(vtilde_l_opp_f - vtilde_l_opp_f0))
    norm_vec(22) = maxval(abs(vtilde_l_opp_y_f - vtilde_l_opp_y_f0))
    norm = maxval(norm_vec)

    !Check for convergence:
    if (norm < tol) exit

enddo

!Prevalence rates: (same as started, no upgrade in smallexp)
prev1 = prev_vec

!How many iterations?
print *, "number of iterations: ", iter

!Set the convergence flag for the inner loop:
if (iter>=max_iter-1) then
	conv_flags(1) = 0
else
	conv_flags(1) = 1
endif

!EXCESS DEMANDS:
!constant for normalization -> we're taking the steady state value of the
!population size disregarding deaths due to AIDS. We're also assuming that
!the singles account for about 10% of the entire population. Then, we use
!this number of singles to normalize the excess demands.
normalize = (1.0/10.0)*sum(mu_m)/delta

!PROTECTED
ed_p = sum(measure_p_m+measure_p_y_m) - sum(measure_p_f+measure_p_y_f)
ed_p = ed_p/normalize

!UNPROTECTED
ed_u = sum(measure_u_m+measure_u_y_m) - sum(measure_u_f+measure_u_y_f)
ed_u = ed_u/normalize

!LONG TERM
ed_l = sum(measure_l_m+measure_l_y_m) - sum(measure_l_f+measure_l_y_f)
ed_l = ed_l/normalize

!ALL MARKETS INTO A VECTOR:
excess_vector(1) = ed_p
excess_vector(2) = ed_u
excess_vector(3) = ed_l

if (iter>=max_iter-1) then
	excess_vector = 100.0d0
endif


!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------

!In the following, we write out the distributions, policy functions for search effort
!and the value functions into files that will be read by Matlab in order to compute
!the desired model statistics.

if (write_more_files==1) then

    !DISTRIBUTIONS:
    !Males:
    open(11,file='dist_S_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_S_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_S_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_S_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_L_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_L_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_A_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) dist_A_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_m.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_m(jj,kk,ii)
            enddo
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_y_m(jj,kk,ii)
            enddo
        enddo
    enddo
    !Females:
    open(11,file='dist_S_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_S_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_S_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_S_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_L_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_L_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_A_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) dist_A_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_f.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_f(jj,kk,ii)
            enddo
        enddo
    enddo
    close(11)
    open(11,file='dist_L_opp_y_f.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) dist_L_opp_y_f(jj,kk,ii)
            enddo
        enddo
    enddo

    !SEARCH EFFORT:
    !Males:
    open(11,file='pi_p_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_p_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_p_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_p_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_u_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_u_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_l_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) pi_l_y_m(jj,ii)
        enddo
    enddo
    close(11)
    !Females:
    open(11,file='pi_p_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_p_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_p_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_p_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_u_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_u_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_u_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_l_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='pi_l_y_f.txt',status='replace')
    do ii=1,ntypes_f
        do jj=1,n_status
            write(11,*) pi_l_y_f(jj,ii)
        enddo
    enddo
    close(11)

    !VALUES
    !Males:
    open(11,file='v_l_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) v_l_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='v_l_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) v_l_y_m(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='vtilde_l_opp_m.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) vtilde_l_opp_m(jj,kk,ii)
            enddo
        enddo
    enddo
    close(11)
    open(11,file='vtilde_l_opp_y_m.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) vtilde_l_opp_y_m(jj,kk,ii)
            enddo
        enddo
    enddo
    !Females:
    open(11,file='v_l_f.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) v_l_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='v_l_y_f.txt',status='replace')
    do ii=1,ntypes_m
        do jj=1,n_status
            write(11,*) v_l_y_f(jj,ii)
        enddo
    enddo
    close(11)
    open(11,file='vtilde_l_opp_f.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) vtilde_l_opp_f(jj,kk,ii)
            enddo
        enddo
    enddo
    close(11)
    open(11,file='vtilde_l_opp_y_f.txt',status='replace')
    do ii=1,ntypes_m
        do kk=1,n_status
            do jj=1,n_status
                write(11,*) vtilde_l_opp_y_f(jj,kk,ii)
            enddo
        enddo
    enddo

    !FRACTION OF PEOPLE IN THE LONG TERM MARKET:
    !Males:
    open(11,file='frac_l_m.txt',status='replace')
    do jj=1,n_status
        write(11,*) frac_l_m_vec(jj)
    enddo
    close(11)
    !Healthy Males - by circumcision type:
    open(11,file='frac_l_healthy_bycirc_m.txt',status='replace')
    do jj=1,n_circ
        write(11,*) frac_l_healthy_bycirc_m(jj)
    enddo
    close(11)
    !Females:
    open(11,file='frac_l_f.txt',status='replace')
    do jj=1,n_status
        write(11,*) frac_l_f_vec(jj)
    enddo
    close(11)

    !INFECTIOUSNESS IN THE SHORT TERM MARKETS:
    upsilon_all(1) = upsilon_p_m
    upsilon_all(2) = upsilon_u_m
    upsilon_all(3) = upsilon_p_f
    upsilon_all(4) = upsilon_u_f
    open(11,file='upsilon_all.txt',status='replace')
    do jj=1,4
        write(11,*) upsilon_all(jj)
    enddo
    close(11)

endif !write_more_files

end subroutine smallexp
